Package: PP
Type: Package
Title: Person Parameter Estimation
Version: 0.6.4-1
Date: 2026-04-21
Authors@R: c(
        person(given = "Jan", 
         family = "Steinfeld", 
         email = "jan.d.steinfeld@gmail.com", 
         role = c("cre","aut"),
         comment = c(ORCID = "0000-0001-9853-8260")
         ),
         person(given = "Manuel", 
         family = "Reif", 
         email = "manuel.reif@gmail.com", 
         role = c("aut")
         )
    )
Author: Jan Steinfeld [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-9853-8260>),
  Manuel Reif [aut]
Maintainer: Jan Steinfeld <jan.d.steinfeld@gmail.com>
Description: The PP package includes estimation of (MLE, WLE, MAP, EAP, ROBUST)
    person parameters for the 1,2,3,4-PL model and the GPCM (generalized
    partial credit model). The parameters are estimated under the assumption
    that the item parameters are known and fixed. The package is useful e.g. in
    the case that items from an item pool / item bank with known item parameters
    are administered to a new population of test-takers and an ability
    estimation for every test-taker is needed.
URL: https://github.com/jansteinfeld/PP
BugReports: https://github.com/jansteinfeld/PP/issues
License: GPL-3
LazyLoad: yes
LazyData: true
VignetteBuilder: knitr
Depends: R (>= 3.0)
Encoding: UTF-8
Suggests: testthat, roxygen2, knitr, eRm, data.table, prettydoc,
        rmarkdown
Imports: Rcpp (>= 0.11.2)
LinkingTo: Rcpp
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-21 12:55:51 UTC; steinfeldj
Repository: CRAN
Date/Publication: 2026-04-21 13:22:09 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-04-21 14:17:52 UTC; unix
Archs: PP.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(JKpp,fourpl)
S3method(JKpp,gpcm)
S3method(JKpp,gpcm4pl)
S3method(PPass,Rm)
S3method(PPass,default)
S3method(PV,fourpl)
S3method(PV,gpcm)
S3method(PV,gpcm4pl)
S3method(Pfit,fourpl)
S3method(Pfit,gpcm)
S3method(print,jk)
S3method(print,ppeo)
S3method(print,pv)
S3method(summary,jk)
S3method(summary,ppeo)
S3method(summary,pv)
export(JKpp)
export(PP_4pl)
export(PP_gpcm)
export(PPall)
export(PPass)
export(PV)
export(Pfit)
export(Pfit_lzstarmix)
export(findmodel)
export(lzstar_mixed)
export(print_personfit_summary)
export(sim_4pl)
export(sim_gpcm)
import(graphics)
import(stats)
import(utils)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
useDynLib(PP)
useDynLib(PP, .registration = TRUE)

## ----settings-----------------------------------------------------------------
knitr::opts_chunk$set(message = FALSE, results='hide')

## ----start, message=FALSE, warning=FALSE--------------------------------------
library(PP)

suppressWarnings(RNGversion("3.5.0"))
set.seed(1337)

# simulate some intercepts
diffpar <- seq(-3,3,length=15)
# simulate some slope parameters
sl     <- round(runif(15,0.5,1.5),2)
la     <- round(runif(15,0,0.25),2)
ua     <- round(runif(15,0.8,1),2)

# simulate response matrix (not following any IRT model)
awm <- matrix(sample(0:1,100*15,replace=TRUE),ncol=15)

## ----1pl, message=FALSE, warning=FALSE----------------------------------------
# MLE
res1plmle <- PP_4pl(respm = awm,thres = diffpar,type = "mle")
# WLE
res1plwle <- PP_4pl(respm = awm,thres = diffpar,type = "wle")

## ----234-pl, message=FALSE, warning=FALSE-------------------------------------

# ------------------------------------------------------------------------
## 2PL model #####
# ------------------------------------------------------------------------
# MLE
res2plmle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "mle")
# WLE
res2plwle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "wle")

# ------------------------------------------------------------------------
## 3PL model #####
# ------------------------------------------------------------------------
# MLE
res3plmle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,type = "mle")
# WLE
res3plwle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,type = "wle")
# ------------------------------------------------------------------------
## 4PL model #####
# ------------------------------------------------------------------------
# MLE
res4plmle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "mle")
# WLE
res4plwle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "wle")

## ----pfit, message=FALSE, warning=FALSE---------------------------------------

# ------------------------------------------------------------------------
## 1PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
pfit1pl_lz <- Pfit(respm=awm,pp=res1plwle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit #####
pfit1pl_li <- Pfit(respm=awm,pp=res1plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------
## 2PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
pfit2pl_lz <- Pfit(respm=awm,pp=res2plwle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit #####
pfit2pl_li <- Pfit(respm=awm,pp=res2plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------
## 3PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
pfit3pl_lz <- Pfit(respm=awm,pp=res3plwle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit #####
pfit3pl_li <- Pfit(respm=awm,pp=res3plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------
## 4PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
pfit4pl_lz <- Pfit(respm=awm,pp=res4plwle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit #####
pfit4pl_li <- Pfit(respm=awm,pp=res4plwle,fitindices=c("lzstar","infit","outfit"))

## ----wle, message=FALSE, warning=FALSE----------------------------------------
# ------------------------------------------------------------------------
## 1PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
## mle ####
pfit1pl_mle_l <- Pfit(respm=awm,pp=res1plmle,fitindices="lzstar")
## wle ####
pfit1pl_wle_l <- Pfit(respm=awm,pp=res1plwle,fitindices="lzstar")

## ----example-1, message=FALSE, warning=FALSE----------------------------------
# eine Grafik erzeugen

res.pp <- Pfit(respm=awm,pp=res1plmle,fitindices=c("lzstar"),SE=TRUE)
x<-seq(-4,4,length=200)
s <- 1
mu <- 0
y <- (1/(s*sqrt(2*pi))) * exp(-((x-mu)^2)/(2*s^2))
plot(x,y, type="l", lwd=2, col = "blue", xlim = c(-8.5,8.5),xlab="", ylab="")
title(main="Density plot of lz* Person-Fit", xlab="density", ylab="score")
lines(density(res.pp$lzstar[,"lzstar"], bw = 0.5), lwd = 2, lty = 2)
rug(res.pp$lzstar[,"lzstar"],col="red")

# zweite Grafik erzeugen
x <- 1:nrow(res.pp$lzstar)
avg <- res.pp$lzstar[,"lzstar"]
sdev <- res.pp$lzstar[,"lzs_se"]

plot(avg, x,
     xlim=range(c(avg-sdev, avg+sdev)),
     pch=19, ylab="Person", xlab="Person-Fit +/- SD",
     main="Plot of Person-Fit with SE"
)
arrows(avg-sdev, x, avg+sdev, length=0.05, angle=90, code=3)
abline(v=0,col = "red", lwd = 3)

## ----example-2, message=FALSE, warning=FALSE----------------------------------
data(pp_amt)
betas <- pp_amt$betas$Itemparameter
diffpar <- pp_amt$Itemparameter
# slope parameters
awm <- pp_amt$daten_amt[,grep("i\\d{1,3}",colnames(pp_amt$daten_amt))]

# estimate ability parameter and personfit
# the computation of the standard error takes a while, therefore we use only a part of the provided data
suppressWarnings(RNGversion("3.5.0"))
set.seed(1800)
# sample items
sampi <- order(sample(1:ncol(awm),40,replace = F))
# sample persons
sampp <- order(sample(1:nrow(awm),100,replace = F))
awm.samp <- awm[sampp,sampi]
# apply(awm.samp, 2, function(x)!all(is.na(x)))
awm.samp <- awm.samp[apply(awm.samp, 1, function(x)!all(is.na(x))),] #only persons with no NA
out <- PPass(respdf = awm.samp,thres = betas[sampi], items="all",type = "wle",
             mod=c("1PL"), fitindices= c("lz","lzstar","infit","outfit"),SE=TRUE)

# first example of illustration
lim <- max(abs(c(min(out$estimate),max(out$estimate))))
x <- seq(-lim,lim,length=200)
s  <- 1
mu <- 0
y <- (1/(s*sqrt(2*pi))) * exp(-((x-mu)^2)/(2*s^2))
plot(x,y, type="l", lwd=2, col = "blue", xlim = c(-lim,lim),xlab="", ylab="")
title(main="Density plot of lz* Person-Fit", xlab="density", ylab="score")
lines(density(out[,"lzstar"], bw = 0.5), lwd = 2, lty = 2)
rug(out[,"lzstar"],col="red")

# second example of illustration
x <- 1:nrow(out)
avg <- out[,"lzstar"]
sdev <- out[,"lzs_se"]

plot(avg, x,
     yaxt="n",
     xlim=range(c(avg-sdev, avg+sdev)),
     pch=19, ylab="Person", xlab="Person-Fit +/- SD",
     main="Plot of Person-Fit with SE"
)
axis(side=2, at = c(1:nrow(out)),labels = c(1:nrow(out)), las = 2,cex.axis=0.66)
arrows(avg-sdev, x, avg+sdev, length=0.05, angle=90, code=3)
abline(v=0,col = "red", lwd = 3)

## ----get----------------------------------------------------------------------

library(PP)

data(fourpl_df)

dim(fourpl_df)

head(fourpl_df)

# extracting the information
diff_par <- attr(fourpl_df,"diffpar")
slope_par <- attr(fourpl_df,"slopes")


## ----check--------------------------------------------------------------------

# extract items and transform the data.frame to matrix
itmat <- as.matrix(fourpl_df[,-(1:2)])


# are there any full scores?
fullsc <- apply(itmat,1,function(x) (sum(x,na.rm=TRUE)+sum(is.na(x))) == length(x))
any(fullsc)

# are there 0 scores?
nullsc <- apply(itmat,1,function(x) sum(x,na.rm=TRUE) == 0)
any(nullsc)

# are there missing values? how many and where?
nasc <- apply(itmat,1,function(x) sum(is.na(x)))
any(nasc > 0)
#which(nasc)

# is our data dichotomous as expected?
apply(itmat,2,function(x) table(x))

# are there any duplicates?
rdup <- duplicated(itmat)
sum(rdup)



## ----decide-------------------------------------------------------------------
library(PP)

res1plmle <- PP_4pl(respm = itmat, thres = diff_par, slopes = slope_par, type = "mle")

summary(res1plmle)


## ----edit---------------------------------------------------------------------

dafest <- data.frame(fourpl_df, res1plmle$resPP$resPP)

head(dafest,10)

## ----rerun--------------------------------------------------------------------
library(PP)

res1plwle <- PP_4pl(respm = itmat,thres = diff_par, slopes = slope_par, type = "wle")

summary(res1plwle)

## ----ppass--------------------------------------------------------------------

pres <- PPass(fourpl_df, items = 3:14, mod="2PL", thres = diff_par, slopes = slope_par, type = "wle")

#### PPass function - brings it all together

#' Person Assessment function
#' 
#' Estimate Person Paramters and calculate Person Fit in one step to gain resonse pattern assessment. Submit a data.frame which contains item responses, or an fitted model (Rasch Model and Partial Credit Model are supported) of the \code{eRm} package.
#' 
#' @param \ldots Submit arguments to the underlying functions: \code{PP_4pl}, \code{PP_gpcm} and \code{PPall} (see documentation files) for person parameter estimation.
#' 
#' @rdname PPass
#' 
#' @example R/.example_ppass.R
#' 
#' @export
#'

PPass <- function(...) UseMethod("PPass")

# ---------------------------------------------------------------------



#' @param respdf A data.frame which contains the items, and perhaps other informations. Each row is a person related resonse patter. Each column denotes a variable.
#' 
#' @param items A numeric (integer) vector which indicates the positions of the items in the data.frame (\code{respdf}). If \code{items} = 'all', \bold{all columns} are treated as items.
#' 
#' @param mod Choose your data generating model. This argument switches between the three person parameter estimating functions \code{PP_4pl}, \code{PP_gpcm} and \code{PPall}.
#' 
#' @param fitindices A character vector which denotes the fit indices to compute.
#' 
#' @details PPass fuses Person Parameter estimation and Person Fit computation into a single function.
#' 
#' @return The original data.frame and
#' 
#' \itemize{
#' 
#' \item The Person Parameter estimates incl. Standard Errors (2 columns)
#' 
#' \item Person Fit Indices you chose (1 or more)
#' 
#' }
#' 
#' @author Manuel Reif, Jan Steinfeld
#' 
#' @method PPass default
#' 
#' @rdname PPass
#' 
#' @export
#' 
#' @seealso \link{PP_4pl}, \link{PP_gpcm}, \link{PPall}, \link{Pfit}
#' 

PPass.default <- function(respdf, items="all", mod=c("1PL","2PL","3PL","4PL","PCM","GPCM","MIXED"), fitindices= c("lz","lzstar","infit","outfit"), ...)
{
  
  # catch additional arguments
  all_pts <- list(respdf,...)
  fitindices <- match.arg(fitindices, several.ok = TRUE)  
  
  ########### ESTIMATE PERSON PARAMETERS ###############################  
  
  ## checks concering the input  
  stopifnot(is.data.frame(respdf)) # muss ein df sein als input
  stopifnot((is.numeric(items) & all(items >= 1)) | all(items == "all")) # indices oder alles
  
  ## create matrix, keep the rest
  if(all(items == "all")) # all variables are items
  {
    respm <- as.matrix(respdf)
  } else {
    
    rest  <- respdf[ , -items, drop=FALSE]
    respm <- as.matrix(respdf[ , items, drop=FALSE])
  }
  
  # collect the arguments and change data argument
  all_pts$respdf <- respm
  names(all_pts)[grep("respdf",names(all_pts))] <- "respm"
  args_4pl  <- setdiff(names(formals(PP_4pl)), all_pts)
  args_4pl <- all_pts[names(all_pts)%in%args_4pl]
  args_gpcm <- setdiff(names(formals(PP_gpcm)), all_pts)
  args_gpcm <- all_pts[names(all_pts)%in%args_gpcm]
  args_all <- setdiff(names(formals(PPall)), all_pts)
  args_all <- all_pts[names(all_pts)%in%args_all]
  args_pfit <- setdiff(names(formals(Pfit)), all_pts)
  args_pfit <- all_pts[args_pfit]
  args_pfit$fitindices <- fitindices
  # check if first element is character
  if(is.character(respm[1,1])) stop("At least one response is of type character!\n")
  
  if(mod %in% c("1PL","2PL","3PL","4PL"))
  {
    pp_est <- do.call(PP_4pl,args_4pl)
  } else if(mod %in% c("PCM", "GPCM"))
  {
    pp_est <- PP_gpcm(respm, ...)
    
  } else if("MIXED"){ # mixed
    pp_est <- PPall(respm, ...)
  }
  
  ########### CALCULATE PERSON FIT ###############################
  # first we have to store the estimated person parameter
  args_pfit[[2]] <- pp_est
  names(args_pfit)[2] <- "pp"
  # this is needed to make shure, that no 'NA' arguments are in the list
  args_pfit <- args_pfit[names(args_pfit)%in%names(formals(Pfit))] 
  fit_calc <- do.call(Pfit,args_pfit)
  # cbind all pers fits together
  fit_calc <- do.call(cbind,fit_calc)
  ########### PUT IT ALL TOGETHER ############################### 
  #out <- list("personparameter"=pp_est,"personfit"=fit_calc)
  out <- data.frame(pp_est[["resPP"]][["resPP"]], fit_calc)
  return(out)
  
}





# ================= for eRm input =================================


#' @param RMobj A fitted Rasch Model (\code{RM()}) object which stems from the \code{eRm} package.
#' 
#' @rdname PPass
#' 
#' @export
#' 
#' @method PPass Rm
#' 
#' 
PPass.Rm <- function(RMobj, fitindices= c("lz","lzstar","infit","outfit"), ...)
{
  
  # catch additional arguments
  all_pts <- list(RMobj,...)
  fitindices <- match.arg(fitindices, several.ok = TRUE)  
  
  # ---------------------------------------------------
  # collect the arguments and change data argument
  all_pts$RMobj <- RMobj$X
  names(all_pts)[grep("RMobj",names(all_pts))] <- "respm"
  # --------------
  args_4pl  <- setdiff(names(formals(PP_4pl)), all_pts)
  args_4pl <- all_pts[names(all_pts)%in%args_4pl]
  # --------------
  args_gpcm <- setdiff(names(formals(PP_gpcm)), all_pts)
  args_gpcm <- all_pts[names(all_pts)%in%args_gpcm]
  # --------------
  args_pfit <- setdiff(names(formals(Pfit)), all_pts)
  args_pfit <- all_pts[args_pfit]
  args_pfit$fitindices <- fitindices
  # ---------------------------------------------------
  
  # geht leider nicht anders weil sowohl PCM als auch RM die Klassen Rm als auch eRm haben.
  if(RMobj$model == "RM")
  {
    args_4pl <- append( args_4pl,list("thres"=RMobj$betapar * (-1)) )
   
    # pp_est <- PP_4pl(respm=RMobj$X, thres=RMobj$betapar * (-1), ...)
    pp_est <- do.call(PP_4pl,args_4pl)
    
    
  } else if(RMobj$model == "PCM")
  {
    
    # get threshold parameters:
    
    # create threshold matrix:
    #tps <- eRm::thresholds(RMobj)$threshpar
    
    #len1        <- apply(RMobj$X, 2, function(x) length(unique(x))-1)
    #itemss      <- unlist(lapply(1:length(len1), function(x) rep(x, each=len1[x])))
    #wohin_zeile <- unlist(lapply(len1, function(x) 1:x))
    
    #names(wohin_zeile) <- NULL
    
    #thres <- matrix(NA,ncol=ncol(RMobj$X), nrow=max(len1))
    
    #for(i in 1:length(tps))
    #{
    #  thres[wohin_zeile[i], itemss[i]] <- tps[i]
    #}
    tps <- eRm::thresholds(RMobj)$threshtable[[1]]
    tps <- t(tps)
    tps[1,] <- 0
    thres <- tps
    # thres <- rbind(0,thres)
    slopes <- rep(1,ncol(thres))
    
    ########### ESTIMATE PERSON PARAMETERS ###############################  
    args_gpcm <- append( args_gpcm,list("thres"=thres, slopes=slopes) )
    # pp_est <- PP_gpcm(respm=RMobj$X, thres=thres, slopes=slopes, ...)
    pp_est <- do.call(PP_gpcm,args_gpcm)
    
  } else {
    
    stop("I don't know this model!")
    
  }
  
  
  ########### CALCULATE PERSON FIT ###############################  
  # first we have to store the estimated person parameter
  args_pfit[[2]] <- pp_est
  names(args_pfit)[2] <- "pp"
  
  # this is needed to make shure, that no 'NA' arguments are in the list
  args_pfit <- args_pfit[names(args_pfit)%in%names(formals(Pfit))] 
  fit_calc <- do.call(Pfit,args_pfit)
  # fit_calc <- Pfit(respm=RMobj$X,pp=pp_est,fitindices=fitindices,...)
  # cbind all pers fits together
  fit_calc <- do.call(cbind,fit_calc)
  ########### PUT IT ALL TOGETHER ############################### 
  #out <- list("personparameter"=pp_est,"personfit"=fit_calc)
  
  # if(class(pp_est)[1] == "gpcm")
  #   {
  #   out <- data.frame(pp_est[["resPP"]][["resPP"]], fit_calc)
  #   } else {
  #          out <- data.frame(pp_est[["resPP"]][["resPP"]], fit_calc)  
  #          }
  out <- data.frame(pp_est[["resPP"]][["resPP"]], fit_calc) 
  return(out)
}

#' PP: Estimation of Person Parameters and Person Fit
#'
#' The PP package provides functions for estimating person parameters for the
#' 1PL, 2PL, 3PL, and 4PL models, as well as the generalized partial credit
#' model (GPCM). Supported estimation methods include maximum likelihood (ML),
#' weighted likelihood (WL; Warm, 1989), maximum a posteriori (MAP), expected
#' a posteriori (EAP), and robust estimation.
#'
#' In addition, the package includes routines for person-fit analysis, including
#' infit, outfit, lz, and lzstar statistics. The implementation is designed for
#' efficient computation and includes compiled code for fast estimation.
#'
#' For an introduction to the package workflow, see the package vignettes.
#'
#' @author Jan Steinfeld and Manuel Reif
#' @seealso [PPass()], [PP_gpcm()], [PP_4pl()], [PPall()], [Pfit()]
#' @references
#' Barton, M. A., & Lord, F. M. (1981). An upper asymptote for the
#' three-parameter logistic item-response model.
#'
#' Birnbaum, A. (1968). Some latent trait models and their use in inferring an
#' examinee's ability. In F. M. Lord & M. R. Novick (Eds.), \emph{Statistical
#' theories of mental test scores}. Reading, MA: Addison-Wesley.
#'
#' Drasgow, F., Levine, M. V., & Williams, E. A. (1985). Appropriateness
#' measurement with polychotomous item response models and standardized indices.
#' \emph{British Journal of Mathematical and Statistical Psychology},
#' \bold{38}(1), 67--86.
#'
#' Muraki, E. (1992). A generalized partial credit model: Application of an EM
#' algorithm. \emph{Applied Psychological Measurement}, \bold{16}, 159--176.
#'
#' Samejima, F. (1993). An approximation of the bias function of the maximum
#' likelihood estimate of a latent variable for the general case where the item
#' responses are discrete. \emph{Psychometrika}, \bold{58}, 119--138.
#'
#' Snijders, T. B. (2001). Asymptotic null distribution of person fit statistics
#' with estimated person parameter. \emph{Psychometrika}, \bold{66}(3), 331--342.
#'
#' Warm, T. A. (1989). Weighted likelihood estimation of ability in item
#' response theory. \emph{Psychometrika}, \bold{54}, 427--450.
#'
#' Wright, B. D., & Masters, G. N. (1990). Computation of OUTFIT and INFIT
#' statistics. \emph{Rasch Measurement Transactions}, \bold{3}(4), 84--85.
#'
#' Yen, Y.-C., Ho, R.-G., Liao, W.-W., Chen, L.-J., & Kuo, C.-C. (2012). An
#' empirical evaluation of the slip correction in the four parameter logistic
#' models with computerized adaptive testing. \emph{Applied Psychological
#' Measurement}, \bold{36}, 75--87.
#'
#' @examples
#' set.seed(1522)
#'
#' diffpar <- seq(-3, 3, length = 12)
#' sl <- round(runif(12, 0.5, 1.5), 2)
#' la <- round(runif(12, 0, 0.25), 2)
#' ua <- round(runif(12, 0.8, 1), 2)
#'
#' awm <- matrix(sample(0:1, 10 * 12, replace = TRUE), ncol = 12)
#'
#' res3plmle <- PP_4pl(
#'   respm = awm,
#'   thres = diffpar,
#'   slopes = sl,
#'   lowerA = la,
#'   type = "mle"
#' )
#'
#' res3plwle <- PP_4pl(
#'   respm = awm,
#'   thres = diffpar,
#'   slopes = sl,
#'   lowerA = la,
#'   type = "wle"
#' )
#'
#' res3plmap <- PP_4pl(
#'   respm = awm,
#'   thres = diffpar,
#'   slopes = sl,
#'   lowerA = la,
#'   type = "map"
#' )
#'
#' res3plmlepfit <- Pfit(
#'   respm = awm,
#'   pp = res3plmle,
#'   fitindices = c("infit", "outfit")
#' )
#'
#' out <- PPass(
#'   respdf = data.frame(awm),
#'   thres = diffpar,
#'   items = "all",
#'   mod = c("1PL"),
#'   fitindices = c("lz", "lzstar", "infit", "outfit")
#' )
#'
#' @keywords internal
"_PACKAGE"

Outfitpoly <- function( data,
                        thetas,
                        thresholds, 
                        slopes=NULL
){
  # ------------------------------------------------------------------------------------------------
  betas <- as.vector(apply(thresholds[-1,],2,cumsum))
  betas <- betas[!is.na(betas)]
  if(is.null(slopes)) slope <- rep(1,length(betas))
  # ------------------------------------------------------------------------------------------------
  #  information
  X <- data
  N.mat <- apply(X, 1, function(x){ length(na.exclude(x)) })
  k <- apply(X,2,max,na.rm=TRUE)
  # processed Items 
  Xproc <- 1 * !is.na(X)  
  # ------------------------------------------------------------------------------------------------
  # parameter information
  ai <- slope
  # ------------------------------------------------------------------------------------------------
  # category information
  k_seq <- sequence(k)
  k_seq0 <- integer(0)
  for (i in k) k_seq0 <- c(k_seq0, 0:i)
  k_seq1 <- integer(0)
  for (i in k) k_seq1 <- c(k_seq1, c(1,1:i))
  # ------------------------------------------------------------------------------------------------
  # Itemspecific information
  k_item <- rep(1:length(k),times = k)
  k_item0 <- rep(1:length(k),times = k+1)
  theta_mat <- matrix( thetas, nrow = nrow( X ), ncol = sum( k ) ) 
  # ------------------------------------------------------------------------------------------------
  theta_mat_kat <- tcrossprod(theta_mat , diag(k_seq))
  # ---------------------------------------------
  submatrix_pijx <- (t(theta_mat_kat) - betas) #* ai 
  
  cat_pijx_0 <- tapply(1:length(betas),k_item,function(x) {
    submat <- rbind(rep(0,times = ncol(submatrix_pijx)),submatrix_pijx[x,])
    
    tmp <- apply(submat,2,function(xx){ 
      kat <- exp(xx)
      nom <- sum(kat)
      dev <- kat / nom
      return(dev)
    })
    return(tmp)   
  })

  cat_pijx <- lapply(cat_pijx_0,function(x){
    x <- x[-1,]
  })
  
  Pijx <- t(do.call(rbind,cat_pijx))
  Pijx <- Xproc[,k_item] * Pijx
  
  Emat.l <- tcrossprod(Pijx , t(diag(k_seq)))
  Emat <- tapply(1:length(betas),k_item,function(x) rowSums(Emat.l[,x],na.rm=TRUE))
  
  Eij <- t(do.call(rbind,Emat))
  Eij <- Xproc * Eij
  
  Pijx.0 <- t(do.call(rbind,cat_pijx_0))
  Pijx.0 <- Xproc[,k_item0] * Pijx.0
  
  Eij.0 <- t( apply(Eij[,k_item0],1,function(x) {k_seq0 - x}) )
  Eij.0 <- Xproc[,k_item0] * Eij.0
  
  # Variance
  Vmat.cat <- (Eij.0^2)*Pijx.0
  Vmat.l <- tapply(1:length(k_seq0),k_item0,function(x) rowSums(Vmat.cat[,x],na.rm=TRUE))
  Wni.mat <- Xproc * t(do.call(rbind,Vmat.l))
  
  Cmat.cat <- (Eij.0)^4*Pijx.0
  Cmat.l <- tapply(1:length(k_seq0),k_item0, function(x) {rowSums(Cmat.cat[,x],na.rm=TRUE)})
  Cni.mat <- Xproc * t(do.call(rbind,Cmat.l))

  Yni.mat <- X - Eij
  Zni.mat <- Yni.mat / sqrt( Wni.mat )
  zni2.mat <- Zni.mat^2
  
  # OUTFIT MEANSQ
  Un <- rowSums( ((Zni.mat)^2), na.rm=TRUE ) / N.mat
  
  # standardized INFIT
  # Variance term
  varInfit <- rowSums(Cni.mat / Wni.mat^2,na.rm=TRUE ) / (N.mat^2) - (1/N.mat)
  # outfit
  tu <- ( (Un^(1/3)) - 1)*(3/sqrt(varInfit))+(sqrt(varInfit)/3)
  
  # additional output
  chisq   <- rowSums( (zni2.mat), na.rm=TRUE )
  pvalue  <- 1 - pchisq(rowSums( (zni2.mat), na.rm=TRUE ), N.mat-1 )
  df      <- N.mat - 1 
  
  out <- cbind(
    "outfit"   = round(Un,6),
    "ou_t"     = round(tu,6),
    "ou_chisq" = round(chisq,3),
    "ou_df"    = df,
    "ou_pv"    = round(pvalue,3)
    
  )
  return(out)
}

# LZ-Statistik
lz <- function( data, 
                thetas, 
                betas, 
                lowerAs=NULL, 
                slopes=NULL,
                higherAs=NULL,...){ 
  if(is.null(slopes)) slopes <- rep(1,length(betas))
  if(is.null(lowerAs))lowerAs <- rep(0,length(betas))
  if(is.null(higherAs))higherAs <- rep(1,length(betas))
  if(!all(apply(data,2,function(x) { all(na.omit(x) %in% 0:1) }))) stop("Please check the input, only 0/1/NA are allowed \n")

  
     ai <- slopes
     ci <- lowerAs
     di <- higherAs
     
     # processed Items 
     Xproc <- 1 * !is.na(data)  
     Nproc <- rowSums(Xproc)
     
  # calculate the propability of each person to solve an item ("-x" within the probabilit.)
  submatrix <- (matrix( thetas, ncol = nrow( data ), nrow = ncol( data ) ,byrow = TRUE) - betas) * ai

   Pi     <- t( ci + (di-ci)/(1+exp(-submatrix)) )
   Pi_1   <- 1-Pi

  l0      <- rowSums( ( (data * log(Pi)) + ((1-data) * log(Pi_1)) )*Xproc/Nproc,na.rm=TRUE)
  mean_l0 <- rowSums( Xproc*((Pi * log(Pi)) + ((Pi_1) * log(Pi_1))) ,na.rm=TRUE)
  var_l0  <- rowSums( Xproc*(Pi * (Pi_1)) * (log(Pi/Pi_1)^2) ,na.rm=TRUE)
  lz      <- (Nproc*l0 - mean_l0) / sqrt(var_l0) 
  lz <- round(lz,6)
  l0 <- round(l0,6)
  out <- cbind("lz"=lz,"lz_unst"=l0)
  return(out)
  
}

#' Estimate Person Parameters for the GPCM
#' 
#' Compute person parameters for the GPCM and choose between five common estimation techniques: ML, WL, MAP, EAP and a robust estimation. All item parameters are treated as fixed.
#'
#'
#' Please note, that \code{robust} estimation with (Huber ability estimate) polytomous items is still experimental!
#'
#'
#' The probability choosing the k-th category is as follows:
#'
#'\deqn{P(x_{ij} = k | \hat \alpha_i, \hat\beta_{iv}, \theta_j) = \frac{exp(\sum_{v=0}^{(k-1)}\hat \alpha_{i}(\theta_j - \hat \beta_{iv}))}{\,\sum_{c=0}^{m_i - 1}exp(\sum_{v=0}^{c}\hat \alpha_{i}(\theta_j - \hat \beta_{iv})))}}
#'
#'In our case \eqn{\theta} is to be estimated. The item parameters are assumed as fixed (usually these are estimates of a former scaling procedure).
#'
#'The model simplifies to the Partial Credit Model by setting \eqn{\alpha_{i} = 1, \forall i}.
#'
#' 
#'@param respm An integer matrix, which contains the examinees responses. A persons x items matrix is expected.
#'@param thres A numeric matrix which contains the threshold parameter for each item. If the first row of the matrix is not set to zero (only zeroes in the first row) - then a row-vector with zeroes is added by default.
#'@param slopes A numeric vector, which contains the slope parameters for each item - one parameter per item is expected.
#'@param theta_start A vector which contains a starting value for each person. If NULL is submitted, the starting values are set automatically. If a scalar is submitted, this start value is used for each person.
#'@param mu A numeric vector of location parameters for each person in case of MAP or EAP estimation. If nothing is submitted this is set to 0 for each person for MAP estimation.
#'@param sigma2 A numeric vector of variance parameters for each person in case of MAP  or EAP estimation. If nothing is submitted this is set to 1 for each person for MAP estimation.
#'@param type Which maximization should be applied? There are five valid entries possible: "mle", "wle", "map", "eap" and "robust". To choose between the methods, or just to get a deeper understanding the papers mentioned below are quite helpful. The default is \code{"wle"} which is a good choice in many cases.
#'@param maxsteps The maximum number of steps the NR Algorithm will take. Default = 100.
#'@param exac How accurate are the estimates supposed to be? Default is 0.001.
#'@param H In case \code{type = "robust"} a Huber ability estimate is performed, and \code{H} modulates how fast the downweighting takes place (for more Details read Schuster & Yuan 2011).
#'@param ctrl more controls
#'\itemize{
#' \item \code{killdupli}: Should duplicated response pattern be removed for estimation (estimation is faster)? This is especially resonable in case of a large number of examinees and a small number of items.  Use this option with caution (for map and eap), because persons with different \code{mu} and \code{sigma2} will have different ability estimates despite they responded identically. Default value is \code{FALSE}.
#'
#'\item \code{skipcheck}: Default = FALSE. If TRUE data matrix and arguments are not checked - this saves time e.g. when you use this function for simulations.
#'}
#'
#'
#'@template resulttemplate
#'
#' @seealso \link{PPass}, \link{PPall}, \link{PP_4pl}, \link{JKpp}, \link{PV}
#'
#'@export
#'
#'@author Manuel Reif
#'@references Baker, Frank B., and Kim, Seock-Ho (2004). Item Response Theory - Parameter Estimation Techniques. CRC-Press.
#'
#'Masters, G. N. (1982). A Rasch model for partial credit scoring. Psychometrika, 47(2), 149-174.
#'
#'Muraki, Eiji (1992). A Generalized Partial Credit Model: Application of an EM Algorithm. Applied Psychological Measurement, 16, 159-176.
#'
#'Muraki, Eiji (1993). Information Functions of the Generalized Partial Credit Model. Applied Psychological Measurement, 17, 351-363.
#'
#'Samejima, Fumiko (1993). The bias function of the maximum likelihood estimate of ability for the dichotomous response level. Psychometrika,  58, 195-209.
#'
#'Samejima, Fumiko (1993). An approximation of the bias function of the maximum likelihood estimate of a latent variable for the general case where the item responses are discrete. Psychometrika,  58, 119-138.
#'
#' Schuster, C., & Yuan, K. H. (2011). Robust estimation of latent ability in item response models. Journal of Educational and Behavioral Statistics, 36(6), 720-735.
#'
#'Wang, S. and Wang, T. (2001). Precision of Warm's Weighted Likelihood Estimates for a Polytomous Model in Computerized Adaptive Testing. Applied Psychological Measurement, 25, 317-331.
#'
#'Warm, Thomas A. (1989). Weighted Likelihood Estimation Of Ability In Item Response Theory. Psychometrika, 54, 427-450.
#'

#'@example ./R/.examples_gpcm.R
#'@keywords Person Parameters GPCM
#'@rdname PP_gpcm
#'




PP_gpcm <- function(respm, thres, slopes, theta_start=NULL,
                    mu = NULL, sigma2 = NULL, type="wle", maxsteps=100, exac=0.001,H=1,ctrl=list())
{
  
  ### 
  call <- match.call()  
  attr(call, "date") <- date() 
  attr(call,"version") <- packageVersion("PP")
  ###
  
  
  ## --------- user controls
  cont <- list(killdupli=FALSE,skipcheck=FALSE)
  
  user_ctrlI <- match(names(ctrl),names(cont))
  if(any(is.na(user_ctrlI)))
  {
    notex <- names(ctrl)[is.na(user_ctrlI)]
    warning("The following options in ctrl do not exist: ", paste(notex,collapse=", "))
    ctrl       <- ctrl[!is.na(user_ctrlI)]
    user_ctrlI <- user_ctrlI[!is.na(user_ctrlI)]
  }
  
  cont[user_ctrlI] <- ctrl
  


  

## --------- threshold matrix

# If the user did not add a row with zeroes only
  if(any(thres[1,] != 0))
    {
      thres <- rbind(0,thres)
    }
  
  # compute the maximal score per item
  maxsc <- apply(thres,2,function(x)(length(x) - sum(is.na(x)))-1)
  
  
  ## starting values
if(is.null(theta_start))
  {
  thresv <- apply(thres[2,,drop=FALSE],2, function(tr) mean(tr,na.rm=TRUE))
  theta_start <- apply(respm, 1, function(st)
    {
    mean(thresv[!is.na(st)])  
    })
  } else if(length(theta_start) == 1)
      {
      theta_start <- rep(theta_start, nrow(respm))  
      }
  

if(!cont$skipcheck) # to save time e.g. when running simulations
{
## --------- check user inputs
checkINP(respm, thres, slopes, theta_start, lowerA=NULL, upperA=NULL, type)
# -----------------------------------------------------  
}

# in case map is chosen and no mu and/or sigma2 is/are submitted.
if( (any(is.null(mu)) | any(is.null(sigma2))))
{
  if(any(is.null(mu))) 
  {
    mu <- rep(0,nrow(respm))
    if(type %in% c("map","eap")) warning("all mu's are set to 0! \n")
  }
  
  if(any(is.null(sigma2))) 
  {
    sigma2 <- rep(1,nrow(respm))
    if(type %in% c("map","eap")) warning("all sigma2's are set to 1! \n")
  }
  
}

  
  
# ----- ----- -------------#


# add NA and Inf in case of mle estimation and full oder 0 score
if(type=="mle"| type=="robust")
{
  resPPx <- ansol(respm,maxsc)  
  respm <- respm[!is.na(resPPx[,2]),,drop=FALSE]
}



# ----- remove duplicated -------------# 

if(cont$killdupli)
{
  dupvec <- make_dup(respm)
  respm <- respm[dupvec$ndpat,,drop=FALSE]
}


cat("Estimating: GPCM ... \n")
cat("type =",type,"\n")


if(type %in% c("mle","wle","map"))
  {
    # ----- estimation procedure -------------# 
    
    resPP <- NR_GPCM(respm,thres,slopes,theta_start,type,maxsteps,exac,mu,sigma2,H=H) 
    
    resPP$resPP[,2] <- sqrt(resPP$resPP[,2])
  } else if(type == "robust")
    {
    warning("Robust estimation for GPCM is still very experimental! \n")
    resPP <- NR_GPCM(respm,thres,slopes,theta_start,type,maxsteps,exac,mu,sigma2,H=H) 
      
    resPP$resPP[,2] <- sqrt(resPP$resPP[,2])
    
    } else if(type == "eap")
      {
        resPP <- list()
        resPP$resPP <- eap_gpcm(respm, thres, slopes,
                                mu = mu, sigma2 = sigma2)
        resPP$nsteps <- 0    
      }


    
  ### result preperation --------------------------
  
  # add duplicated values
  if(cont$killdupli)
    {
      # expanding up the matrix
      resPP$resPP <- resPP$resPP[dupvec$posvec,]
    }
  
  # add Inf and -Inf 
  if(type=="mle" | type=="robust")
    {
      resPPx[!is.na(resPPx[,2]),] <- resPP$resPP
      resPP$resPP <- resPPx
    }
  
  colnames(resPP$resPP) <- c("estimate","SE")
  

  ipar <- list(respm=respm,thres=thres,slopes=slopes,theta_start=theta_start,mu=mu,sigma2=sigma2,cont=cont,H=H)
  
  
  if(cont$killdupli)
    {
      ipar$dupvec <- dupvec 
    }

  ## ---------------------------------------------
  cat("Estimation finished!\n")
  rescall <- list(resPP=resPP,call=call,type=type,ipar=ipar)
  class(rescall) <- c("gpcm","ppeo")

  
  
  
return(rescall)  
  
}

context("GPCM")


THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)

sl     <- c(0.5,1,1.5,1.1,1,0.98)
awmatrix <- matrix(c(1,0,2,0,1,1,1,0,0,1
                     ,2,0,0,0,0,0,0,0,0,1,1,2,2,1,1,1,1,0,0,1),byrow=TRUE,nrow=5)


awmatrix_2 <- awmatrix
awmatrix_2[2,1] <- 3

THRESna  <- THRES

THRESna[,1] <- NA

test_that("errors and warnings are thrown in the gpcm when necessary",{
  expect_that(PP_gpcm(respm = awmatrix,thres = THRES, slopes = sl,type = "robust"),gives_warning())
  expect_that(PP_gpcm(respm = awmatrix,thres = THRES, slopes = sl,type = "A"),throws_error())
  expect_that(PP_gpcm(respm = awmatrix[,-1],thres = THRES, slopes = sl),throws_error())
  expect_that(PP_gpcm(respm = awmatrix[,-1],thres = THRES[,-1], slopes = sl),throws_error())
  expect_that(PP_gpcm(respm = awmatrix[,-1],thres = THRES, slopes = sl[-1]),throws_error())
  expect_that(PP_gpcm(respm = awmatrix,thres = THRESna, slopes = sl),throws_error())
  expect_that(PP_gpcm(respm = awmatrix_2,thres = THRES, slopes = sl),throws_error())
  #expect_that(PP_gpcm(respm = awmatrix,thres = THRES, slopes = sl,theta_start = 2),throws_error())
})

Infit <- function( data, 
                   thetas, 
                   betas, 
                   lowerAs=NULL, 
                   slopes=NULL, 
                   higherAs=NULL,... ){
  if(is.null(slopes)) slopes <- rep(1,length(betas))
  if(is.null(lowerAs))lowerAs <- rep(0,length(betas))
  if(is.null(higherAs))higherAs <- rep(1,length(betas))
  
  if(!all(apply(data,2,function(x) { all(na.omit(x) %in% 0:1) }))) stop("Please check the input, only 0/1/NA are allowed \n")
  X <- data
  # processed Items 
  Xproc <- 1 * !is.na(X)  
  N <- as.numeric(apply(X,1,function(x) sum(!is.na(x))))
  
  ai <- slopes
  ci <- lowerAs
  di <- higherAs
  # calculate the propability of each pearson to not solve an item
  submatrix <- (matrix( thetas, ncol = nrow( data ), nrow = ncol( data ) ,byrow = TRUE) - betas) * ai
  
  # --------------------------------------------------------------------------------
  # probabilit. of solving an Item Expected Response (in der Literatur Pi nik)
  Pni     <- t( ci + (di-ci)/(1+exp(-submatrix)) )
  # --------------------------------------------------------------------------------
  
  # second: calculate for each the Prop
  # ------------------------------------------------  
  Qni <- Xproc * (Pni * ( 1-Pni ))
  
  # Variance of xni
  Wni <- Xproc * ( (((0-Pni)^2) * (1-Pni) ) + (((1-Pni)^2) * Pni) )
  
  # Kurtosis of xni
  Cni <- Xproc * ( (((0-Pni)^4) * (1-Pni)) + (((1-Pni)^4) * Pni) )
  
  # Score Residual: (Pni in literature Eni)
  Yni <- Xproc * (X - Pni)
  
  # Standardized Residual (Qni in literature Wni)
  Zni <- Yni / sqrt( Qni )
  
  # Fit Mean Square
  
  # INFIT  (weighted)
  Vn <- rowSums( (Yni)^2, na.rm=TRUE ) / rowSums( Qni, na.rm=TRUE )
  
  # standardized INFIT
  # Variance term
  qni2 <- rowSums(Cni - Wni^2,na.rm=TRUE ) / (rowSums(Wni,na.rm=TRUE))^2
  # infit.z
  ti <- ( (Vn^(1/3)) - 1)*(3/sqrt(qni2))+(sqrt(qni2)/3)
  

  
  # additional output
  chisq   <- rowSums( ((Zni)^2), na.rm=TRUE )
  Zni2 <- rowSums( ((Zni)^2), na.rm=TRUE )
  pvalue  <- 1 - pchisq(Zni2, N-1 )
  df      <- N - 1 
  
  out <- cbind(
    "infit"    = round(Vn,6),
    "in_t"     = round(ti,6),
    "in_chisq" = round(chisq,3),
    "in_df"    = df,
    "in_pv"    = round(pvalue,3)

  )
  return(out)
  
}

#' Create a model-type vector template
#' 
#' This is a small helper function which creates a vector template  quick and easily for the \code{PPall()} function. Modify this template as you like.
#'
#' This function tries to guess the model which was applied to each item by using the matrix of threshold parameters. It only discriminates between GPCM and 4-PL model, and returns a character vector of length equal to the number of items, that contains \code{"GPCM"} or \code{"4PL"} entries depending on the structure of the thres matrix.
#' 
#'@param thres A numeric matrix which contains the threshold parameter for each item. NA is allowed - in fact expected!
#'
#' @seealso \link{PPall}
#'
#'@export
#'
#'@author Manuel Reif
#'
#'@example ./R/.examples.R
#'@keywords Person Parameters
findmodel <- function(thres)
{

if(any(thres[1,] != 0))
  {
   thres <- rbind(0,thres)
  }  
  
maxsc <- apply(thres,2,function(x)(length(x) - sum(is.na(x)))-1)
model2est <- ifelse(maxsc > 1,"GPCM","4PL")

model2est  
}

context("GPCM&4PL model")

# some threshold parameters
THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
# slopes
sl     <- c(0.5,1,1.5,1.1,1,0.98)

THRESx <- THRES
THRESx[2,1:3] <- NA
THRESx2 <- THRESx

THRESx2[,4] <- NA

#
la     <- c(0.02,0.1,0,0,0,0)
ua     <- c(0.97,0.91,1,1,1,1)

awm <- matrix(c(1,0,1,0,1,1,1,0,0,1
                     ,2,0,0,0,0,0,0,0,0,1
                     ,0,2,2,1,1,1,1,0,0,1),byrow=TRUE,nrow=5)

# create model2est
# this function tries to help finding the appropriate 
# model by inspecting the THRESx.
model2est <- findmodel(THRESx)

awm3 <- awm2 <- awm 
awm2[3,1] <- 3

#awm3[,1] <- NA

# ------------------------- testing 1>>>

#t
test_that("errors - warnings misspelling and length #1",{
  expect_that(PPall(respm = awm,thres = THRESx, slopes = sl,lowerA=la,upperA=ua,type = "aaa",model2est=model2est), throws_error())
  
  expect_that(PPall(respm = awm,thres = THRESx[,-1], slopes = sl,lowerA=la,upperA=ua,model2est=model2est), throws_error())
  
  expect_that(PPall(respm = awm,thres = THRESx, slopes = sl[-1],lowerA=la,upperA=ua,model2est=model2est), throws_error())
  
  expect_that(PPall(respm = awm,thres = THRESx, slopes = sl,lowerA=la[-1],upperA=ua,model2est=model2est), throws_error())
  
  expect_that(PPall(respm = awm,thres = THRESx, slopes = sl,lowerA=la,upperA=ua[-1],model2est=model2est), throws_error())
  
  expect_that(PPall(respm = awm[,-1],thres = THRESx, slopes = sl,lowerA=la,upperA=ua,model2est=model2est), throws_error())
  
  expect_that(PPall(respm = awm2,thres = THRESx, slopes = sl,lowerA=la,upperA=ua,model2est=model2est), throws_error())
  
  expect_that(PPall(respm = awm2,thres = THRESx2, slopes = sl,lowerA=la,upperA=ua,model2est=model2est), throws_error())
  
  expect_that(PPall(respm = awm,thres = THRESx, slopes = sl,lowerA=la,upperA=ua,model2est=model2est, type="robust"), gives_warning())
})

jackknife <- function(data,pp,fit){
  out <- matrix(NA,ncol=ncol(data),nrow=nrow(data))
  data <- as.matrix(data)
  beta <- pp$ipar$thres[2,]
  if(fit=="lz"){
    for(i in 1:ncol(data)){
  capture.output(pp.lz <- PP_4pl(respm=data[,-i],thres=pp$ipar$thres[2,][-i], lowerA=pp$ipar$lowerA[-i], slopes=pp$ipar$slopes[-i], upperA=pp$ipar$upperA[-i],type=pp$type, mu=pp$ipar$mu[-i], sigma2=sqrt(pp$ipar$sigma2[-i])))
      out[,i] <- lz(data=data[,-i], thetas=pp.lz$resPP$resPP[,"estimate"], betas=pp.lz$ipar$thres[2,], lowerAs=pp.lz$ipar$lowerA, slopes=pp.lz$ipar$slopes, higherAs=pp.lz$ipar$upperA)[,fit]
    }
  }
  if(fit=="lzstar"){
    for(i in 1:ncol(data)){
      capture.output(pp.ls <- PP_4pl(respm=data[,-i],thres=pp$ipar$thres[2,][-i], lowerA=pp$ipar$lowerA[-i], slopes=pp$ipar$slopes[-i], upperA=pp$ipar$upperA[-i],type=pp$type, mu=pp$ipar$mu[-i], sigma2=sqrt(pp$ipar$sigma2[-i])))
      out[,i] <- lzstar(data=data[,-i], thetas=pp.ls$resPP$resPP[,"estimate"], betas=pp.ls$ipar$thres[2,], lowerAs=pp.ls$ipar$lowerA, slopes=pp.ls$ipar$slopes, higherAs=pp.ls$ipar$upperA,method=pp.ls$type, mu=pp.ls$ipar$mu, sigma=sqrt(pp.ls$ipar$sigma2))
    }
  }
    
  out.se <- sqrt( ((ncol(data)-1)/ncol(data)) * rowSums((out - rowMeans(out, na.rm=TRUE))^2, na.rm=TRUE) )
  return(out.se)
}

#' Draw Plausible values
#' 
#' This function draws \code{npv} plausible values for each person from their posterior density.
#' 
#' 
#' @param estobj An object which originates from using \code{PP_4pl()}, \code{PP_gpcm()} or \code{PPall()}. EAP estimation is strongly recommended (\code{type = "eap"}), when plausible values are drawn afterwards, because the EAP estimate is used as starting point for the MH algorithm.
#' @param npv The number of (effectively returned) plausible values - default is 10.
#' @param approx Whether a normal approximation \code{N(mu,sigma2)} is used to draw the plausible values. Default = TRUE. If FALSE a Metropolitan-Hastings-Algorithm will draw the values.
#' @param thinning A numeric vector of length = 1. If approx = FALSE, a Metropolitan-Hastings-Algorithm draws the plausible values. To avoid autocorrelation, thinning takes every \bold{kth} value as effective plausible value. The default is 6 (every 6th value is taken), which works appropriately in almost all cases here (with default settings).
#' @param burnin How many draws should be discarded at the chains beginning? Default is 10 - and this seems reasonable high (probably 5 will be enough as well), because starting point is the EAP.
#' @param mult Multiplication constant (default = 2). Use this parameter to vary the width of the proposal distribution - which is \code{N(theta_v,mult*SE_eap)} - when a MH-Algorithm is applied. So the constant quantifies the width in terms of multiples of the EAP standard error. 2 works fine with the default thinning. If the supplied value is large, thinning can take lower values without causing autocorrelation.
#' @param ... More arguments
#'
#'
#' @return The function returns a list which main element is \code{pvdraws}. This is a matrix of size number_of_persons x npv - so if 10 plausible values are requested for 100 persons, a 100x10 matrix is returned.
#'
#'@references Mislevy, R. J. (1991). Randomization-based inference about latent variables from complex samples. Psychometrika, 56(2), 177-196.
#'
#'Von Davier, M., Gonzalez, E., & Mislevy, R. (2009). What are plausible values and why are they useful. IERI monograph series, 2, 9-36.
#'
#'Kruschke, J. (2010). Doing Bayesian data analysis: A tutorial introduction with R. Academic Press.
#'
#'
#' @seealso \link{PP_gpcm}, \link{PP_4pl}, \link{JKpp}
#'
#' @rdname PV
#' @example ./R/.example_pv.R
#' @export
#' @author Manuel Reif

PV <- function(estobj,...) UseMethod("PV")


#' @rdname PV
#' @method PV fourpl
#' @export
PV.fourpl <- function(estobj,npv=10,approx=TRUE,thinning=6,burnin=10,mult=2,...)
{
  
  call <- match.call()  
  attr(call, "date") <- date() 
  attr(call,"version") <- packageVersion("PP")
  
  # grep objects 
  respm <- estobj$ipar$respm
  thres <- estobj$ipar$thres
  slopes <- estobj$ipar$slopes
  lowerA <- estobj$ipar$lowerA
  upperA <- estobj$ipar$upperA
  theta_start <- estobj$ipar$theta_start
  mu <- estobj$ipar$mu
  sigma2 <- estobj$ipar$sigma2
  cont <- estobj$ipar$cont
  
  ppresults <- estobj$resPP$resPP

  if(approx)  
  {
    
    pvs <- sapply(1:nrow(ppresults), function(gretel)
    {
      rnorm(npv,ppresults[gretel,1],ppresults[gretel,2])
    })
    
  } else 
  {
    
    # metropolitan-hastings-algorithm  ########
    #################################################
    jederdritte <- (1:(npv*thinning))[1:(npv*thinning) %% thinning == 0]
    
    pvs <- sapply(1:nrow(respm), function(gr)
    {
      
      theta <- ppresults[gr,1] # eap estimate
      # 20 for burnin
      PVvec <- vector(length=npv*thinning+burnin,mode="numeric")
      
      lauf <- 1
      #       zaehl <- 1
      while(lauf <= length(PVvec))
        {
          
          
          Li <- LIK4pl(awv=respm[gr,], thres=thres, slopes=slopes,
                       lowerA=lowerA, upperA=upperA, theta=theta) 
          Post <- Li * dnorm(theta)
          
          # ---- prop >>>
          
          proposed <- rnorm(5,theta,mult*ppresults[gr,2])
          
          for(PROP in proposed)
            {
              
            Li1 <- LIK4pl(awv=respm[gr,], thres=thres, slopes=slopes,
                          lowerA=lowerA, upperA=upperA, theta=PROP) 
            
            Post1 <- Li1 * dnorm(PROP)
            
            # ---- move? >>>
            
            Pmove <- Post1/Post
            
            if(runif(1) <= Pmove)
              {
                theta <- PROP
                PVvec[lauf] <- theta
                lauf <- lauf + 1
                break
              }
  
            }
          
   
        }
        
      PVvec[-(1:burnin)][jederdritte] 
    })
    
    
  }  
  

pvdraws <- t(pvs)  

retpv <- list(pvdraws=pvdraws,call=call)
class(retpv) <- "pv"
return(retpv)

}





#' @rdname PV
#' @method PV gpcm
#' @export
PV.gpcm <- function(estobj,npv=10,approx=TRUE,thinning=6,burnin=10,mult=2,...)
{
  
  call <- match.call()  
  attr(call, "date") <- date() 
  attr(call,"version") <- packageVersion("PP")
  
  # grep objects 
  respm <- estobj$ipar$respm
  thres <- estobj$ipar$thres
  slopes <- estobj$ipar$slopes
  theta_start <- estobj$ipar$theta_start
  mu <- estobj$ipar$mu
  sigma2 <- estobj$ipar$sigma2
  cont <- estobj$ipar$cont
  
  ppresults <- estobj$resPP$resPP
  
  if(approx)  
  {
    
    pvs <- sapply(1:nrow(ppresults), function(gretel)
    {
      rnorm(npv,ppresults[gretel,1],ppresults[gretel,2])
    })
    
  } else 
  {
    
    # metropolitan-hastings-algorithm  ########
    #################################################
    jederdritte <- (1:(npv*thinning))[1:(npv*thinning) %% thinning == 0]
    
    pvs <- sapply(1:nrow(respm), function(gr)
    {
      
      theta <- ppresults[gr,1] # eap estimate
      # 20 for burnin
      PVvec <- vector(length=npv*thinning+burnin,mode="numeric")
      
      lauf <- 1
      #       zaehl <- 1
      while(lauf <= length(PVvec))
      {
        
        Li <- Likgpcm(respm[gr,], thres, slopes,theta) 
        Post <- Li * dnorm(theta)
        
        # ---- prop >>>
        
        proposed <- rnorm(5,theta,mult*ppresults[gr,2])
        
        for(PROP in proposed)
        {
          Li1 <- Likgpcm(respm[gr,], thres, slopes,PROP) 

          
          Post1 <- Li1 * dnorm(PROP)
          
          # ---- move? >>>
          
          Pmove <- Post1/Post
          
          if(runif(1) <= Pmove)
          {
            theta <- PROP
            PVvec[lauf] <- theta
            lauf <- lauf + 1
            break
          }
          
        }
        
        
      }
      
      PVvec[-(1:burnin)][jederdritte] 
    })
    
    
  }  
  
  
  pvdraws <- t(pvs)  
  
  retpv <- list(pvdraws=pvdraws,call=call)
  class(retpv) <- "pv"
  return(retpv)
}





#' @rdname PV
#' @method PV gpcm4pl
#' @export
PV.gpcm4pl <- function(estobj,npv=10,approx=TRUE,thinning=6,burnin=10,mult=2,...)
{
  
  call <- match.call()  
  attr(call, "date") <- date() 
  attr(call,"version") <- packageVersion("PP")
  
  # grep objects 
  respm <- estobj$ipar$respm
  thres <- estobj$ipar$thres
  slopes <- estobj$ipar$slopes
  lowerA <- estobj$ipar$lowerA
  upperA <- estobj$ipar$upperA
  theta_start <- estobj$ipar$theta_start
  mu <- estobj$ipar$mu
  sigma2 <- estobj$ipar$sigma2
  cont <- estobj$ipar$cont
  model2est <- estobj$ipar$model2est
  
  ## where is which model?
  wheregpcm <- model2est == "GPCM"
  where4pl  <- model2est == "4PL"
  
  respm_gpcm <- respm[,wheregpcm, drop=FALSE]
  respm_4pl  <- respm[,where4pl , drop=FALSE]
  
  # 4pl part
  thres4pl  <- thres[1:2,where4pl,drop=FALSE]
  slopes4pl  <- slopes[where4pl]
  lowerA4pl <- lowerA[where4pl]
  upperA4pl <- upperA[where4pl]
  # gpcm part
  thresgpcm <- thres[,wheregpcm,drop=FALSE]
  slopegpcm <- slopes[wheregpcm]
  
  ppresults <- estobj$resPP$resPP
  
  if(approx)  
  {
    
    pvs <- sapply(1:nrow(ppresults), function(gretel)
    {
      rnorm(npv,ppresults[gretel,1],ppresults[gretel,2])
    })
    
  } else 
  {
    
    # metropolitan-hastings-algorithm  ########
    #################################################
    jederdritte <- (1:(npv*thinning))[1:(npv*thinning) %% thinning == 0]
    
    pvs <- sapply(1:nrow(respm), function(gr)
    {
      
      theta <- ppresults[gr,1] # eap estimate
      # 20 for burnin
      PVvec <- vector(length=npv*thinning+burnin,mode="numeric")
      
      lauf <- 1
      #       zaehl <- 1
      while(lauf <= length(PVvec))
      {
        
        Li4pl     <- LIK4pl(awv=respm_4pl[gr,], thres=thres4pl, slopes=slopes4pl,
                             lowerA=lowerA4pl, upperA=upperA4pl, theta=theta) 
        
        Ligpcm    <- Likgpcm(respm_gpcm[gr,],thresgpcm,slopegpcm,theta)
        
        # all together
        Li <- Li4pl * Ligpcm
        
        Post <- Li * dnorm(theta)
        
        # ---- prop >>>
        
        proposed <- rnorm(5,theta,mult*ppresults[gr,2])
        
        for(PROP in proposed)
        {
          
          Li4pl1     <- LIK4pl(awv=respm_4pl[gr,], thres=thres4pl, slopes=slopes4pl,
                              lowerA=lowerA4pl, upperA=upperA4pl, theta=PROP) 

          Ligpcm1    <- Likgpcm(respm_gpcm[gr,],thresgpcm,slopegpcm,PROP)
          
          # all together
          Li1 <- Li4pl1 * Ligpcm1
          
          Post1 <- Li1 * dnorm(PROP)
          
          # ---- move? >>>
          
          Pmove <- Post1/Post
          
          if(runif(1) <= Pmove)
          {
            theta <- PROP
            PVvec[lauf] <- theta
            lauf <- lauf + 1
            break
          }
          
        }
        
        
      }
      
      PVvec[-(1:burnin)][jederdritte] 
    })
    
    
  }  
  

  pvdraws <- t(pvs)  
  retpv <- list(pvdraws=pvdraws,call=call)
  class(retpv) <- "pv"
  return(retpv)
}














########## estimate likelihood of 4pl
LIK4pl <- function(awv, thres, slopes, lowerA, upperA, theta)
{
  
  P <- lowerA + (upperA - lowerA) * exp(slopes*(theta - thres[-1,]))/(1+exp(slopes*(theta - thres[-1,])))
  Q <- 1-P
  
  Li <- P*awv + Q*(1-awv)  
  prod(Li)
  
}

checkINP <- function(respm, thres, slopes, theta_start, lowerA, upperA, type)
{
  #browser()
# dim checks
if(ncol(respm) != ncol(thres)) stop("ncol(respm) != ncol(thres)\n")  
if(ncol(respm) != length(slopes) & !is.null(slopes)) stop("ncol(respm) != length(slopes)")

if(exists("lowerA"))
  {
  if(ncol(respm) != length(lowerA) & !is.null(lowerA)) stop("ncol(respm) != length(lowerA)")
  }

if(exists("upperA"))
  {
    if(ncol(respm) != length(upperA) & !is.null(upperA)) stop("ncol(respm) != length(upperA)")  
  }

# type input
match.arg(type,c("mle","wle","map","eap","robust"))
if(length(type) != 1) stop("Submit a single value as 'type'!\n")

# respm  input
if(!is.matrix(respm)) stop("respm must be a matrix!\n")
if(!is.matrix(thres)) stop("thres must be a matrix!\n")

# thres
allNA  <- any(apply(thres[-1,,drop=FALSE],2,function(x) all(is.na(x))))
thresl <- apply(thres,2,function(x) length(x) - sum(is.na(x)))
if(allNA){stop("Check thres input! Some colums contain only NA.\n")}

# respm
itmax <- apply(respm,2,function(x) max(x, na.rm=TRUE))
if(any(itmax > (thresl-1))) stop("Responses in categories which so not exist according to thres!\n")
  
# theta
if(length(theta_start) != nrow(respm)) stop("Invalid length of theta_start!\n")

}

eap_4pl <- function(respm, thres, slopes, lowerA=NULL, upperA=NULL,
                    mu = NULL, sigma2 = NULL)
{
  
  nodmat <- matrix(0,nrow(respm),60)  
  weimat <- matrix(0,nrow(respm),60) 
  # #persons x #nodes
  
  for(qrun in 1:nrow(respm))
  {
    qres <- quadIT(nodes = 60,mu = mu[qrun],sigma = sqrt(sigma2[qrun]),absrange = 6)
    nodmat[qrun,]  <- qres[[1]]$nodes
    weimat[qrun,]  <- qres[[1]]$weights
  }
  
  
  FPL_eap <- sapply(1:nrow(respm),function(ep)
    {
      
      oben <- exp(t(outer(nodmat[ep,],thres[-1,],"-"))*slopes)
      oben2 <- oben * (upperA-lowerA)
      P <- lowerA + oben2/(1+oben)
      # item x nodes
      Q <- 1 - P
      
      xi <- respm[ep,]
      ximi <- 1-xi
      pxi <- P * xi
      qxi <- Q * ximi
      li  <- pxi + qxi
      
      Li <- apply(li,2,function(x)prod(x,na.rm=TRUE)) 
      EAP <- sum(Li * weimat[ep,] * nodmat[ep,])/sum((Li*weimat[ep,]))
      
      EAP_v <- sum(Li * weimat[ep,] * (nodmat[ep,]-EAP)^2)/sum((Li*weimat[ep,]))
      
      c(EAP,sqrt(EAP_v))
    })
  
  t(FPL_eap)   
}





eap_gpcm <- function(respm, thres, slopes, mu = NULL, sigma2 = NULL)
  {
    
    nodmat <- matrix(0,nrow(respm),60)  
    weimat <- matrix(0,nrow(respm),60) 
    # #persons x #nodes
    
    for(qrun in 1:nrow(respm))
    {
      qres <- quadIT(nodes = 60,mu = mu[qrun],sigma = sqrt(sigma2[qrun]),absrange = 6)
      nodmat[qrun,]  <- qres[[1]]$nodes
      weimat[qrun,]  <- qres[[1]]$weights
    }
    
    
  FPL_eap <- sapply(1:nrow(respm),function(ep)
      {
        
        Li  <- Likgpcm(respm[ep,],thres,slopes,nodmat[ep,])
  
        EAP <- sum(Li * weimat[ep,] * nodmat[ep,])/sum((Li*weimat[ep,]))
        
        EAP_v <- sum(Li * weimat[ep,] * (nodmat[ep,]-EAP)^2)/sum((Li*weimat[ep,]))
        
        c(EAP,sqrt(EAP_v))
      })
    
  t(FPL_eap)  
  
  }



## ------------------ mixed 4pl und gpcm

eap_mixed <- function(respm, thres, slopes, lowerA, upperA, mu = NULL, sigma2 = NULL,model2est)
  {
    
    nodmat <- matrix(0,nrow(respm),60)  
    weimat <- matrix(0,nrow(respm),60) 
    # #persons x #nodes
    
    wheregpcm <- model2est == "GPCM"
    where4pl  <- model2est == "4PL"
    
    respm_gpcm <- respm[,wheregpcm, drop=FALSE]
    respm_4pl  <- respm[,where4pl , drop=FALSE]
    
    
    for(qrun in 1:nrow(respm))
    {
      qres <- quadIT(nodes = 60,mu = mu[qrun],sigma = sqrt(sigma2[qrun]),absrange = 6)
      nodmat[qrun,]  <- qres[[1]]$nodes
      weimat[qrun,]  <- qres[[1]]$weights
    }
    
      # 4pl part
      thres4pl  <- thres[,where4pl,drop=FALSE]
      slopes4pl  <- slopes[where4pl]
      lowerA4pl <- lowerA[where4pl]
      upperA4pl <- upperA[where4pl]
      # gpcm part
      thresgpcm <- thres[,wheregpcm,drop=FALSE]
      slopegpcm <- slopes[wheregpcm]
      
    
    FPL_eap <- sapply(1:nrow(respm),function(ep)
    {
      
      oben <- exp(t(outer(nodmat[ep,],thres4pl[2,],"-"))*slopes4pl)
      oben2 <- oben * (upperA4pl - lowerA4pl)
      P <- lowerA4pl + oben2/(1+oben)
      # item x nodes
      Q <- 1 - P
      
      xi <- respm_4pl[ep,]
      ximi <- 1-xi
      pxi <- P * xi
      qxi <- Q * ximi
      li  <- pxi + qxi
      
      Li4pl <- apply(li,2,function(x)prod(x,na.rm=TRUE))
      
      Ligpcm    <- Likgpcm(respm_gpcm[ep,],thresgpcm,slopegpcm,nodmat[ep,])
      
      # all together
      Li <- Li4pl * Ligpcm
      
      EAP <- sum(Li * weimat[ep,] * nodmat[ep,])/sum((Li*weimat[ep,]))
      
      EAP_v <- sum(Li * weimat[ep,] * (nodmat[ep,]-EAP)^2)/sum((Li*weimat[ep,]))
      
      c(EAP,sqrt(EAP_v))
    })
    
    t(FPL_eap)  
    
  }

#' Estimate Person Parameters
#' 
#' Compute person parameters for the 1,2,3,4-PL model and for the GPCM. Choose between ML, WL, MAP, EAP and robust estimation. Use this function if 4-PL items and GPCM items are mixed for each person.
#'
#' 
#' For a test with both: dichotomous and polytomous items which have been scaled under 1/2/3/4-PL model or the GPCM, use this function to estimate the person ability parameters. You have to define the appropriate model for each item. 
#' 
#'Please note, that  \code{robust} estimation with (Huber ability estimate) polytomous items is still experimental!
#' 
#'@param respm An integer matrix, which contains the examinees responses. A persons x items matrix is expected.
#'@param thres A numeric matrix which contains the threshold parameter for each item. If the first row of the matrix is not set to zero (only zeroes in the first row) - then a row-vector with zeroes is added by default.
#'@param slopes A numeric vector, which contains the slope parameters for each item - one parameter per item is expected. 
#'@param lowerA A numeric vector, which contains the lower asymptote parameters (kind of guessing parameter) for each item. In the case of polytomous items, the value must be 0.
#'@param upperA numeric vector, which contains the upper asymptote parameters for each item. In the case of polytomous items, the value must be 1.
#'@param theta_start A vector which contains a starting value for each person. If NULL is submitted, the starting values are set automatically. If a scalar is submitted, this start value is used for each person.
#'@param mu A numeric vector of location parameters for each person in case of MAP estimation. If nothing is submitted this is set to 0 for each person for MAP estimation.
#'@param sigma2 A numeric vector of variance parameters for each person in case of MAP or EAP estimation. If nothing is submitted this is set to 1 for each person for MAP estimation.
#'@param type Which maximization should be applied? There are five valid entries possible: "mle", "wle", "map", "eap" and "robust". To choose between the methods, or just to get a deeper understanding the papers mentioned below are quite helpful. The default is \code{"wle"} which is a good choice in many cases.
#'
#'@param model2est A character vector with length equal to the number of submitted items. It defines itemwise the response model under which the item parameter was estimated. There are 2 valid inputs up to now: \code{"GPCM"} and \code{"4PL"}.
#'
#'@param maxsteps The maximum number of steps the NR algorithm will take. Default = 100.
#'@param exac How accurate are the estimates supposed to be? Default is 0.001.
#'@param H In case \code{type = "robust"} a Huber ability estimate is performed, and \code{H} modulates how fast the downweighting takes place (for more Details read Schuster & Yuan 2011).
#'@param ctrl More controls:
#'
#'\itemize{
#' \item \code{killdupli}: Should duplicated response pattern be removed for estimation (estimation is faster)? This is especially resonable in case of a large number of examinees and a small number of items.  Use this option with caution (for map and eap), because persons with different \code{mu} and \code{sigma2} will have different ability estimates despite they responded identically. Default value is \code{FALSE}.
#'
#'\item \code{skipcheck}: Default = FALSE. If TRUE data matrix and arguments are not checked - this saves time e.g. when you use this function for simulations.
#'
#'}
#'
#'@template resulttemplate
#'
#' @seealso \link{PPass}, \link{PP_gpcm}, \link{PP_4pl}, \link{JKpp}, \link{PV}
#'
#'@export
#'
#'@author Manuel Reif
#'
#'@references Baker, Frank B., and Kim, Seock-Ho (2004). Item Response Theory - Parameter Estimation Techniques. CRC-Press.
#'
#'Barton, M. A., & Lord, F. M. (1981). An Upper Asymptote for the Three-Parameter Logistic Item-Response Model.
#'
#'Magis, D. (2013). A note on the item information function of the four-parameter logistic model. Applied Psychological Measurement, 37(4), 304-315.
#'
#'Muraki, Eiji (1992). A Generalized Partial Credit Model: Application of an EM Algorithm. Applied Psychological Measurement, 16, 159-176.
#'
#'Muraki, Eiji (1993). Information Functions of the Generalized Partial Credit Model. Applied Psychological Measurement, 17, 351-363.
#'
#'Samejima, Fumiko (1993). The bias function of the maximum likelihood estimate of ability for the dichotomous response level. Psychometrika,  58, 195-209.
#'
#'Samejima, Fumiko (1993). An approximation of the bias function of the maximum likelihood estimate of a latent variable for the general case where the item responses are discrete. Psychometrika,  58, 119-138.
#'
#'Schuster, C., & Yuan, K. H. (2011). Robust estimation of latent ability in item response models. Journal of Educational and Behavioral Statistics, 36(6), 720-735.
#'
#'Wang, S. and Wang, T. (2001). Precision of Warm's Weighted Likelihood Estimates for a Polytomous Model in Computerized Adaptive Testing. Applied Psychological Measurement, 25, 317-331.
#'
#'Warm, Thomas A. (1989). Weighted Likelihood Estimation Of Ability In Item Response Theory. Psychometrika, 54, 427-450.
#'
#'Yen, Y.-C., Ho, R.-G., Liao, W.-W., Chen, L.-J., & Kuo, C.-C. (2012). An empirical evaluation of the slip correction in the four parameter logistic models with computerized adaptive testing. Applied Psychological Measurement, 36, 75-87.
#'

#'@example ./R/.examples.R
#'@rdname PPall
#'
PPall <- function(respm, thres, slopes, lowerA, upperA, theta_start=NULL,
                  mu = NULL, sigma2 = NULL, type="wle", model2est,
                  maxsteps=100, exac=0.001, H=1, ctrl=list())
{
### 
call <- match.call()  
attr(call, "date") <- date() 
attr(call,"version") <- packageVersion("PP")
###


## --------- user controls
cont <- list(killdupli=FALSE,skipcheck=FALSE)

user_ctrlI <- match(names(ctrl),names(cont))
if(any(is.na(user_ctrlI)))
  {
    notex <- names(ctrl)[is.na(user_ctrlI)]
    warning("The following options in ctrl do not exist: ", paste(notex,collapse=", "))
    ctrl       <- ctrl[!is.na(user_ctrlI)]
    user_ctrlI <- user_ctrlI[!is.na(user_ctrlI)]
  }

cont[user_ctrlI] <- ctrl


######## take care of the threshold matrix

if(is.matrix(thres))
  {
  #iimm <- nrow(thres) == 1
  
  if(any(thres[1,] != 0))
    {
    thres <- rbind(0,thres)
    }
  
  # compute the maximal score per item
   maxsc <- apply(thres,2,function(x)(length(x) - sum(is.na(x)))-1)
  # are there any items with more than 2 categories?
   #allebigger <- any(maxsc > 1)
  }

  ## starting values
if(is.null(theta_start))
  {
  thresv <- apply(thres[2,,drop=FALSE],2, function(tr) mean(tr,na.rm=TRUE))
  theta_start <- apply(respm, 1, function(st)
    {
    mean(thresv[!is.na(st)])  
    })
  } else if(length(theta_start) == 1)
      {
      theta_start <- rep(theta_start, nrow(respm))  
      }
  


## ----------------------


# check for errors and warnings etc  --------------

## --------- check user inputs
if(!cont$skipcheck) # to save time e.g. when running simulations
{
  ## --------- check user inputs
  checkINP(respm, thres, slopes, theta_start, lowerA, upperA, type)
  # -----------------------------------------------------  
}

model2est <- match.arg(model2est,c("GPCM","4PL"),several.ok = TRUE)
if(length(model2est) < ncol(respm)) stop("Check the properties for the submitted model2est vector!\n")



# in case map is chosen and no mu and/or sigma2 is/are submitted.
if( (any(is.null(mu)) | any(is.null(sigma2))))
  {
  if(any(is.null(mu))) 
    {
    mu <- rep(0,nrow(respm))
    if(type == "map") warning("all mu's are set to 0! \n")
    }
  
  if(any(is.null(sigma2))) 
    {
      sigma2 <- rep(1,nrow(respm))
      if(type == "map") warning("all sigma2's are set to 1! \n")
    }
  
  }

## ----------------------



# ----- MLE: Inf and NA ---------------#
if(type=="mle" | type=="robust")
  {
    resPPx <- ansol(respm,maxsc)  
    respm <- respm[!is.na(resPPx[,2]),]
  }



# ----- remove duplicated -------------# 

if(cont$killdupli)
  {
  # this vector contains the indices, to recreate the large matrix in the end of the estimation
  dupvec <- make_dup(respm)
  respm <- respm[dupvec$ndpat,,drop=FALSE]
  }


# ----- estimation procedure -------------# 

    
     if(any(lowerA[model2est == "GPCM"] != 0)) warning("At least for one item with #categories > 1, there is an lower asymptote != 0 submitted. This will be ignored.\n")

     if(any(upperA[model2est == "GPCM"] != 1)) warning("At least for one item with #categories > 1, there is an upper asymptote != 1 submitted. This will be ignored.\n")
       
modelcat <- paste(unique(model2est),collapse=", ")

cat("Estimating: mixed ",modelcat, "... \n")
cat("type =",type,"\n")



if(type %in% c("mle","wle","map"))
  {
    # ----- estimation procedure -------------# 
    
    resPP <-  NR_mixed(respm,DELTA = thres,ALPHA = slopes, LOWA = lowerA, UPPA = upperA, THETA = theta_start,model=model2est, wm=type,maxsteps,exac,mu,sigma2,H)
    
    resPP$resPP[,2] <- sqrt(resPP$resPP[,2])
    
  } else if(type == "robust")
  {
    warning("Robust estimation for GPCM is still very experimental! \n")
    
    resPP <- NR_mixed(respm,DELTA = thres,ALPHA = slopes, LOWA= lowerA, UPPA = upperA, THETA = theta_start,model=model2est, wm=type,maxsteps,exac,mu,sigma2,H=H) 
    
    resPP$resPP[,2] <- sqrt(resPP$resPP[,2])
    
  } else if(type == "eap")
    {
      resPP <- list()
      resPP$resPP <- eap_mixed(respm, thres, slopes, lowerA=lowerA,
                               upperA=upperA, mu = mu, sigma2 = sigma2, model2est=model2est)
      resPP$nsteps <- 0    
    }


  

### result preperation --------------------------

if(cont$killdupli)
  {
   resPP$resPP <- resPP$resPP[dupvec$posvec,]
  }

if(type=="mle" | type=="robust")
  {
   resPPx[!is.na(resPPx[,2]),] <- resPP$resPP
   resPP$resPP <- resPPx
  }

## ---------------------------------------------

colnames(resPP$resPP) <- c("estimate","SE")

ipar <- list(respm=respm,thres=thres,slopes=slopes,lowerA=lowerA,
             upperA=upperA,theta_start=theta_start,mu=mu,sigma2=sigma2,
             cont=cont,model2est=model2est,H=H)


if(cont$killdupli)
  {
    ipar$dupvec <- dupvec 
  }


## ---------------------------------------------
cat("Estimation finished!\n")
rescall <- list(resPP=resPP,call=call,type=type,ipar=ipar)
class(rescall) <- c("gpcm4pl","ppeo")

  


return(rescall)
}

#' Compute lz and lzstar person-fit statistics for mixed-format tests
#'
#' @param scores Matrix of item scores (n_persons x n_items).
#' @param theta Vector of ability estimates, length must equal nrow(scores).
#' @param vtheta Vector of estimated variances Var(theta_hat), length must equal nrow(scores).
#' @param a Vector of discrimination parameters, length must equal ncol(scores).
#' @param c Vector of lower asymptote (guessing) parameters, length must equal ncol(scores).
#' @param b Matrix of thresholds, with nrow(b) = ncol(scores) and NA padding for shorter items.
#' @param extreme_handling Policy for extreme raw scores (all-0 or perfect):
#'   "na" returns NA for that person, "cap" truncates theta to +/- theta_cap, "none" does nothing.
#' @param theta_cap Cap used when extreme_handling = "cap".
#'
#' @return A matrix with columns "lz" and "lzstar".
#'
#' @details
#' The function implements the mixed-format person-fit statistic using 3PL for dichotomous
#' items and GPCM for polytomous items. The calculation is kept unchanged: dichotomous item
#' contributions are accumulated with pr3PL, polytomous item contributions are accumulated
#' with prGPCM_stable and d1prGPCM, and lz and lzstar are computed from
#' the same denominators used in the implementation.
#'
#' @references
#' Sinharay, S. (2026). Refining the asymptotically correct standardization of person-fit
#' statistics for mixed-format tests. \emph{British Journal of Mathematical and Statistical
#' Psychology}, \bold{00}, 1--24.
#'
#' Sinharay, S. (2016). Asymptotically correct standardization of person-fit statistics beyond
#' dichotomous items. \emph{Psychometrika}, \bold{81}(4), 992--1013.
#'
#' @examples
#' scores <- matrix(c(
#'   1, 0, 2,
#'   0, 1, 1,
#'   1, 1, 3
#' ), nrow = 3, byrow = TRUE)
#' theta  <- c(-0.5, 0.2, 1.1)
#' vtheta <- c(0.12, 0.10, 0.15)
#' a      <- c(1.1, 0.9, 1.3)
#' cpar   <- c(0.15, 0.10, 0.00)
#' b <- matrix(c(
#'   0.0,  NA,
#'  -0.2,  NA,
#'  -1.0,  0.3
#' ), nrow = 3, byrow = TRUE)
#' lzstar_mixed(scores, theta, vtheta, a, cpar, b)
#'
#' scores2 <- matrix(c(
#'   0, 1,
#'   1, 0
#' ), nrow = 2, byrow = TRUE)
#' theta2  <- c(-1, 1)
#' vtheta2 <- c(0.08, 0.08)
#' a2      <- c(1.0, 1.2)
#' c2      <- c(0.05, 0.10)
#' b2 <- matrix(c(
#'   0.0, NA,
#'   0.3, NA
#' ), nrow = 2, byrow = TRUE)
#' lzstar_mixed(scores2, theta2, vtheta2, a2, c2, b2)
#'
#' @export
lzstar_mixed <- function(scores, theta, vtheta, a, c, b,
                         extreme_handling = c("na", "cap", "none"),
                         theta_cap = 4) {
  extreme_handling <- match.arg(extreme_handling)

  if (!is.matrix(scores)) scores <- as.matrix(scores)

  n_persons <- nrow(scores)
  n_items   <- ncol(scores)

  if (length(theta)  != n_persons) stop("Length(theta) must equal nrow(scores)")
  if (length(vtheta) != n_persons) stop("Length(vtheta) must equal nrow(scores)")
  if (length(a)      != n_items)   stop("Length(a) must equal ncol(scores)")
  if (length(c)      != n_items)   stop("Length(c) must equal ncol(scores)")
  if (nrow(b)        != n_items)   stop("nrow(b) must equal ncol(scores)")

  mxscores <- apply(scores, 2, max, na.rm = TRUE)
  dich     <- which(mxscores == 1)
  poly     <- which(mxscores > 1)

  total        <- rowSums(scores, na.rm = TRUE)
  max_possible <- sum(mxscores, na.rm = TRUE)
  is_extreme   <- (total == 0) | (total == max_possible)

  if (extreme_handling == "cap") {
    theta <- pmax(pmin(theta, theta_cap), -theta_cap)
  }

  out <- matrix(NA_real_, nrow = n_persons, ncol = 2,
                dimnames = list(NULL, c("lz", "lzstar")))

  for (j in seq_len(n_persons)) {
    if (extreme_handling == "na" && is_extreme[j]) next
    if (!is.finite(theta[j]) || !is.finite(vtheta[j]) || vtheta[j] < 0) next

    lznum    <- 0
    lzden    <- 0
    lzdenadd <- 0

    if (length(dich) > 0) {
      pr <- pr3PL(theta[j], a[dich], b[dich, 1], c[dich])
      pr <- pmax(pmin(pr, 1 - 1e-10), 1e-10)
      w  <- log(pr / (1 - pr))
      e  <- exp(a[dich] * (theta[j] - b[dich, 1]))
      p1 <- (1 - c[dich]) * a[dich] * e / ((1 + e)^2)

      lznum    <- lznum    + sum((scores[j, dich] - pr) * w)
      lzden    <- lzden    + sum(w^2 * pr * (1 - pr))
      lzdenadd <- lzdenadd + sum(p1 * w)
    }

    if (length(poly) > 0) {
      for (i in poly) {
        thres_i <- b[i, ]
        thres_i <- thres_i[!is.na(thres_i)]
        if (length(thres_i) == 0) next

        probs <- prGPCM_stable(theta[j], a[i], thres_i)
        probs <- pmax(probs, 1e-10)
        probs <- probs / sum(probs)
        log_probs <- log(probs)
        vmat <- diag(probs) - outer(probs, probs)
        der1 <- d1prGPCM(theta[j], a[i], thres_i)
        obs  <- scores[j, i]
        if (is.na(obs) || obs < 0 || obs >= length(probs)) next

        lznum    <- lznum    + (log_probs[obs + 1] - sum(probs * log_probs))
        lzden    <- lzden    + as.numeric(t(log_probs) %*% vmat %*% log_probs)
        lzdenadd <- lzdenadd + sum(der1 * log_probs)
      }
    }

    if (is.finite(lzden) && lzden > 0) {
      out[j, "lz"] <- lznum / sqrt(lzden)
    }

    lzstden <- lzden - vtheta[j] * (lzdenadd^2)
    if (!is.finite(lzstden)) {
      out[j, "lzstar"] <- NA_real_
      next
    }
    if (lzstden <= 0 && lzstden > -1e-6) {
      lzstden <- 1e-6
    }
    if (lzstden > 0) {
      out[j, "lzstar"] <- lznum / sqrt(lzstden)
    }
  }

  out
}

# 3PL probability: P(X = 1 | theta)
pr3PL <- function(theta, a, b, c) {
  c + (1 - c) / (1 + exp(-a * (theta - b)))
}

# Category probabilities under GPCM with numerical stability
prGPCM_stable <- function(theta, a, b) {
  n_cat <- length(b) + 1
  log_probs <- numeric(n_cat)
  log_probs[1] <- 0
  for (k in seq(2, n_cat)) {
    log_probs[k] <- log_probs[k - 1] + a * (theta - b[k - 1])
  }
  log_probs <- log_probs - max(log_probs)
  probs <- exp(log_probs)
  probs / sum(probs)
}

# Derivative of category probabilities under GPCM
# dP_k/dtheta = a * P_k * (score_k - E(score))
d1prGPCM <- function(theta, a, b) {
  probs  <- prGPCM_stable(theta, a, b)
  scores <- seq(0, length(b))
  a * probs * (scores - sum(scores * probs))
}

#' Universal PPall person-fit wrapper with full diagnostics
#'
#' @param respm Response matrix.
#' @param ppobj PPall output.
#' @param model2est Original model specification.
#' @param orig_thres Original thres input to PPall.
#' @param orig_slopes Original slopes input.
#' @param orig_lowerA Original lowerA input.
#'
#' @details
#' This wrapper tries multiple extraction strategies and then calls
#' \code{lzstar_mixed()} without changing the core calculation.
#'
#' @export
Pfit_lzstarmix <- function(respm, ppobj, model2est = NULL,
                           orig_thres = NULL, orig_slopes = NULL,
                           orig_lowerA = NULL) {
  n_persons <- nrow(respm)
  n_items   <- ncol(respm)

  cat(sprintf("Input dimensions: %d persons x %d items\n", n_persons, n_items))

  resPP <- ppobj$resPP$resPP
  if (is.matrix(resPP) || is.data.frame(resPP)) {
    theta    <- as.numeric(resPP[, 1])
    se_theta <- as.numeric(resPP[, 2])
  } else if (is.vector(resPP)) {
    warning("resPP is a vector; setting SE = 0.5")
    theta    <- as.numeric(resPP)
    se_theta <- rep(0.5, length(theta))
  } else {
    stop("Unsupported resPP format in ppobj")
  }

  vtheta <- se_theta^2

  a_final <- NULL
  c_final <- NULL
  b_final <- NULL

  possible_a <- c("slopes", "a", "sl")
  possible_c <- c("lowerA", "c", "la")
  possible_b <- c("thres", "b", "THRESx")

  a_candidates <- list()
  c_candidates <- list()
  b_candidates <- list()

  for (a_name in possible_a) {
    if (a_name %in% names(ppobj$ipar)) {
      a_cand <- ppobj$ipar[[a_name]]
      if (length(a_cand) == n_items) a_candidates[[a_name]] <- a_cand
    }
  }
  for (c_name in possible_c) {
    if (c_name %in% names(ppobj$ipar)) {
      c_cand <- ppobj$ipar[[c_name]]
      if (length(c_cand) == n_items) c_candidates[[c_name]] <- c_cand
    }
  }
  for (b_name in possible_b) {
    if (b_name %in% names(ppobj$ipar)) {
      b_cand <- ppobj$ipar[[b_name]]
      if (is.matrix(b_cand) && nrow(b_cand) == n_items) {
        b_candidates[[b_name]] <- b_cand
      } else if (is.vector(b_cand) && length(b_cand) == n_items) {
        b_candidates[[paste0(b_name, "_converted")]] <- matrix(b_cand, ncol = 1)
      }
    }
  }

  if (!is.null(orig_slopes) && length(orig_slopes) == n_items) {
    a_final <- orig_slopes
  } else if (length(a_candidates) > 0) {
    a_final <- a_candidates[[1]]
  } else {
    stop("Cannot find valid slopes (a) parameters")
  }

  if (!is.null(orig_lowerA) && length(orig_lowerA) == n_items) {
    c_final <- orig_lowerA
  } else if (length(c_candidates) > 0) {
    c_final <- c_candidates[[1]]
  } else {
    c_final <- rep(0, n_items)
  }

  if (!is.null(orig_thres)) {
    if (is.vector(orig_thres)) {
      b_final <- matrix(orig_thres, ncol = 1)
    } else if (is.matrix(orig_thres)) {
      b_final <- orig_thres
    } else if (is.list(orig_thres)) {
      if ("thres" %in% names(orig_thres)) {
        b_final <- orig_thres$thres
        if (!is.matrix(b_final)) b_final <- matrix(b_final, ncol = 1)
      } else {
        b_final <- orig_thres[[1]]
        if (length(b_final) != n_items) b_final <- matrix(b_final, ncol = 1)
      }
    } else {
      b_final <- matrix(as.numeric(orig_thres), ncol = 1)
    }
  } else if (length(b_candidates) > 0) {
    b_final <- b_candidates[[1]]
  } else {
    b_final <- matrix(rnorm(n_items * 2), n_items, 2)
  }

  if (length(a_final) != n_items || length(c_final) != n_items || nrow(b_final) != n_items) {
    stop("Parameter dimensions still do not match!")
  }

  cat("\n--- Computing Person-Fit Statistics ---\n")
  results <- lzstar_mixed(respm, theta, vtheta, a_final, c_final, b_final)

  cat(sprintf("Success! Computed %d statistics\n", nrow(results)))
  cat(sprintf(" lz mean: %.3f (sd: %.3f)\n",
              mean(results[, "lz"], na.rm = TRUE),
              sd(results[, "lz"], na.rm = TRUE)))
  cat(sprintf(" lzstar mean: %.3f (sd: %.3f)\n",
              mean(results[, "lzstar"], na.rm = TRUE),
              sd(results[, "lzstar"], na.rm = TRUE)))

  results
}

#' Print summary of person-fit results
#'
#' @param results Matrix from lzstar_mixed.
#' @param alpha Significance level for flagging.
#'
#' @export
print_personfit_summary <- function(results, alpha = 0.05) {
  cat("\n=== Person-Fit Statistics Summary ===\n\n")
  z_crit <- qnorm(1 - alpha / 2)

  cat("lz statistics:\n")
  cat(sprintf(
    " Mean: %.3f (SD: %.3f)\n",
    mean(results[, 1], na.rm = TRUE),
    sd(results[, 1], na.rm = TRUE)
  ))
  cat(sprintf(
    " Range: [%.3f, %.3f]\n",
    min(results[, 1], na.rm = TRUE),
    max(results[, 1], na.rm = TRUE)
  ))
  flagged_lz <- sum(abs(results[, 1]) > z_crit, na.rm = TRUE)
  cat(sprintf(
    " Flagged (|z| > %.2f): %d (%.1f%%)\n\n",
    z_crit, flagged_lz, 100 * flagged_lz / nrow(results)
  ))

  cat("lzstar statistics (asymptotically correct):\n")
  cat(sprintf(
    " Mean: %.3f (SD: %.3f)\n",
    mean(results[, 2], na.rm = TRUE),
    sd(results[, 2], na.rm = TRUE)
  ))
  cat(sprintf(
    " Range: [%.3f, %.3f]\n",
    min(results[, 2], na.rm = TRUE),
    max(results[, 2], na.rm = TRUE)
  ))
  flagged_lzstar <- sum(abs(results[, 2]) > z_crit, na.rm = TRUE)
  cat(sprintf(
    " Flagged (|z| > %.2f): %d (%.1f%%)\n",
    z_crit, flagged_lzstar,
    100 * flagged_lzstar / sum(!is.na(results[, 2]))
  ))
  cat("\n")
}

#' Person-Fit statistics
#' 
#' Compute several person fit statistic for the 1-PL, 2-PL, 3-PL, 4-PL and PCM. 
#'
#' Please note that currently only the likelihood based LZ-Index (Drasgow, Levine, and Williams, 1985) and LZ*-Index (Snijders, 2001) are implemented. Also the INFIT-OUTIFT (Wright and Masters, 1982, 1990) statistic as well as the polytomouse version of INFIT-OUTFIT are supported. Other person fit statistics will be added soon.
#' 
#' The calculation of the person fit statistics requires the numeric response-matrix as well as an object of the fourpl-class. So first you should estimate the person parameter and afterwards calculate the person fit statistics. You could also use our PPass-function to estimate the person parameter and calculate the desired person fit simultaneously.
#' It is possible to calculate several person fit statistics at once, you only have to specify them in a vector.
#' 
#' For the Partial Credit model we currently support the infit-outfit statistic. Please submit also the numeric response-matrix as well as the estimated person parameter with an gpcm-class.
#' 
#'@param respm	      numeric response matrix
#'@param pp 		      object of the class fourpl with estimated person parameter
#'@param fitindices		character vector of desired person fit statistics c("lz","lzstar","infit","outfit")
#'@param SE		        logical: if true standard errors are computed using jackknife method
#'
#' @return list of person-fits for each person-fit statistic
#' 
#' \itemize{
#' \item the list of person-fits contains the calculated person-fit (like lz, lzstar) and also additional information like p-value or standard error if desired.
#' \item the additional information is provided after the short form of the personfit
#' \item lz (lz)
#' \item lzstar (lzs)
#' \item infit the mean-square statistic (in)
#' \item outfit the mean-square statistic (ou)
#' \item _unst: unstandardised
#' \item _se: standard error
#' \item _t: t-value
#' \item _chisq: $chi^2$-value
#' \item _df: defrees of freedom
#' \item _pv: p-value
#' }
#' 
#' @rdname pfit
#' @seealso \link{PPall}, \link{PP_4pl}, \link{PPass}
#'
#'
#'@author Jan Steinfeld
#'@references 
#'
#' Armstrong, R. D., Stoumbos, Z. G., Kung, M. T. & Shi, M. (2007). On the performance of the lz person-fit statistic.  \emph{Practical Assessment, Research & Evaluation}, \bold{12(16)}. Chicago	
#' 
#' De La Torre, J., & Deng, W. (2008). Improving Person-Fit Assessment by Correcting the Ability Estimate and Its Reference Distribution. Journal of Educational Measurement, \bold{45(2)}, 159-177.
#' 
#' Drasgow, F., Levine, M. V. & Williams, E. A. (1985) Appropriateness measurement with polychotomous item response models and standardized indices. \emph{British Journal of Mathematical and Statistical Psychology}, \bold{38(1)}, 67--86.
#' 
#' Efron, B., & Stein, C. (1981). The jackknife estimate of variance. \emph{The Annals of Statistics}, \bold{9(3)}, 586-596.	
#' 
#' Karabatsos, G. (2003) Comparing the Aberrant Response Detection Performance of Thirty-Six Person-Fit Statistics. \emph{Applied Measurement In Education}, \bold{16(4)}, 277--298.
#' 
#' Magis, D., Raiche, G. & Beland, S. (2012) A didactic presentation of Snijders's l[sub]z[/sub] index of person fit with emphasis on response model selection and ability estimation. \emph{Journal of Educational and Behavioral Statistics}, \bold{37(1)}, 57--81.
#' 
#' Meijer, R. R. & Sijtsma, K. (2001) Methodology review: Evaluating person fit. \emph{Applied Psychological Measurement}, \bold{25(2)}, 107--135.
#' 
#' Molenaar, I. W. & Hoijtink, H. (1990) The many null distributions of person fit indices. \emph{Psychometrika}, \bold{55(1)}, 75--106. 
#' 
#' Mousavi, A. & Cui, Y. Evaluate the performance of and of person fit: A simulation study.
#' 
#' Reise, S. P. (1990). A comparison of item-and person-fit methods of assessing model-data fit in IRT.  \emph{Applied Psychological Measurement}, \bold{14(2)}, 127-137.
#' 
#' Snijders, T. B. (2001) Asymptotic null distribution of person fit statistics with estimated person parameter. \emph{Psychometrika}, \bold{66(3)}, 331--342. 
#' 
#' Wright, B. D. & Masters, G. N. (1990). Computation of OUTFIT and INFIT Statistics.  \emph{Rasch Measurement Transactions}, 3:4, 84-85.
#' 
#' Wright, B. D., & Masters, G. N. (1982). \emph{Rating Scale Analysis. Rasch Measurement.} MESA Press, 5835 S. Kimbark Avenue, Chicago, IL 60637.
#' 
#' @example ./R/.examples_pfit.R
#' @keywords Person fit LZ-Index Infit-Outfit
#' @export
Pfit <- function(respm,pp,fitindices,SE=FALSE) UseMethod("Pfit",object=pp)
# ------------------------------------------------------------------------------------------------------------------------------------------------------------


#'@method Pfit fourpl
#'@export
  Pfit.fourpl <- function(respm, pp, fitindices=c("lz","lzstar","infit","outfit"),SE=FALSE){

    if(any(pp$type%in%c("eap","robust"))) stop("Only 'mle','wle' and 'map' ability estimates are supported \n")

    pfitfunctions <- list("lz" = lz,
                          "lzstar" = lzstar,
                          "infit" = Infit,
                          "outfit" = Outfit
    )
    
    fitindices <- match.arg(fitindices, several.ok = TRUE)  

    pfitfunctions_red <- pfitfunctions[names(pfitfunctions)%in%fitindices]
    
    args <- list(list("data"=respm, 
         "thetas"=pp$resPP$resPP[,"estimate"], 
         "betas"=pp$ipar$thres[2,], 
         "lowerAs"=pp$ipar$lowerA, 
         "slopes"=pp$ipar$slopes, 
         "higherAs"=pp$ipar$upperA,
         "method"=pp$type, 
         "mu"=pp$ipar$mu, 
         "sigma"=sqrt(pp$ipar$sigma2)
    ))
    
    out <- mapply(function(x,y) do.call("y",x), x=args, y=pfitfunctions_red,SIMPLIFY = FALSE)
    names(out) <- names(pfitfunctions_red)
    
    if(SE){
      if(any(fitindices=="lz")){
        lz.se <- jackknife(data = respm, pp=pp, fit="lz")
        out$lz <- cbind(out$lz,"lz_se"=lz.se)
      }
      if(any(fitindices=="lzstar")){
        lzstar.se <- jackknife(data = respm, pp=pp, fit="lzstar")
        out$lzstar <- cbind(out$lzstar,"lzs_se"=lzstar.se)
      }
    }
    
    class(out) <- append(class(out),"PPfit")
    return(out)
  }
  
  # ------------------------------------------------------------------------------------------------------------------------------------------------------------
  
  #'@rdname pfit
  #' 
  #' @method Pfit gpcm
  #' @export
  Pfit.gpcm <- function(respm, pp, fitindices=c("infit","outfit"),SE=FALSE){
    if(SE){warning("There is no jacknife currently supported. \n")}
    if(any(pp$type%in%c("map","eap","robust"))) stop("Only 'mle' and 'wle' ability estimates are supported \n")
    
    if(!all(fitindices%in%c("infit","outfit"))){ warning("Only 'infit and outfit' are currently supported. The calculation is executed with infit outfit \n"); fitindices <- c("infit","outfit")}
    if(any(pp$ipar$slopes>1)) warning("Currently only the PCM-Modell is supported \n")
    
    pfitfunctions <- list("infit" = Infitpoly,
                          "outfit" = Outfitpoly)

     fitindices <- match.arg(fitindices, several.ok = TRUE)  

    pfitfunctions_red <- pfitfunctions[names(pfitfunctions)%in%fitindices]
    
    args <- list(list("data"=respm, 
                      "thetas"=pp$resPP$resPP[,"estimate"], 
                      "thresholds"=pp$ipar$thres, 
                      "slopes"=NULL
    ))
    
    out <- mapply(function(x,y) do.call("y",x), x=args, y=pfitfunctions_red,SIMPLIFY = FALSE)
    names(out) <- names(pfitfunctions_red)
    class(out) <- append(class(out),"PPfit")
    return(out)
  }
  # ------------------------------------------------------------------------------------------------------------------------------------------------------------
  
 # #' @rdname pfit
#  #' 
#  #' @method Pfit gpcm4pl
#  #' @export
#  Pfit.gpcm4pl <- function(respm, pp, fitindices){
#      cat("the mixed method for person fits is not yet implemented \n")
#  }
  # ------------------------------------------------------------------------------------------------------------------------------------------------------------

Outfit <- function( data, 
                   thetas, 
                   betas, 
                   lowerAs=NULL, 
                   slopes=NULL, 
                   higherAs=NULL,... ){
  if(is.null(slopes)) slopes <- rep(1,length(betas))
  if(is.null(lowerAs))lowerAs <- rep(0,length(betas))
  if(is.null(higherAs))higherAs <- rep(1,length(betas))
  
  if(!all(apply(data,2,function(x) { all(na.omit(x) %in% 0:1) }))) stop("Please check the input, only 0/1/NA are allowed \n")
  X <- data
  # processed Items 
  Xproc <- 1 * !is.na(X)  
 
  N <- as.numeric(apply(X,1,function(x) sum(!is.na(x))))
  
  ai <- slopes
  ci <- lowerAs
  di <- higherAs
  # calculate the propability of each pearson to not solve an item
  submatrix <- (matrix( thetas, ncol = nrow( data ), nrow = ncol( data ) ,byrow = TRUE) - betas) * ai
  
  
  # --------------------------------------------------------------------------------
  # probabilit. of solving an Item Expected Response (in der Literatur Pi nik)
  Pni     <- t( ci + (di-ci)/(1+exp(-submatrix)) )
  # --------------------------------------------------------------------------------
  # second: calculate for each the Prop
  
  # ------------------------------------------------  
  Qni <- Xproc * Pni * ( 1-Pni )
  
  # Variance of xni
  Wni <- Xproc * ( (((0-Pni)^2) * (1-Pni)) + (((1-Pni)^2) * Pni) )
  
  # Kurtosis of xni
  Cni <- Xproc * ( (((0-Pni)^4) * (1-Pni)) + (((1-Pni)^4) * Pni) )
  
  # Score Residual: (Pni ist in der Literatru Eni)
  Yni <- Xproc * (X - Pni)
  
  # Standardized Residual (Qni ist in der Literatur Wni)
  Zni <- Yni / sqrt( Qni )
  
  # Fit Mean Square
  
  # OUTFIT  (unweighted)
  Un <- rowSums( ((Zni)^2), na.rm=TRUE ) / N
  
  # Variance term
  varInfit <- rowSums(Cni / Wni^2,na.rm=TRUE ) / (N^2) - (1/N)
  # outfit.z
  tu <- ( (Un^(1/3)) - 1)*(3/sqrt(varInfit))+(sqrt(varInfit)/3)
  
  # additional output
  chisq   <- rowSums( ((Zni)^2), na.rm=TRUE )
  Zni2 <- rowSums( ((Zni)^2), na.rm=TRUE )
  pvalue  <- 1 - pchisq(Zni2, N-1 )
  df      <- N - 1 
  
  out <- cbind(
    "outfit"   = round(Un,6),
    "ou_t"     = round(tu,6),
    "ou_chisq" = round(chisq,3),
    "ou_df"    = df,
    "ou_pv"    = round(pvalue,3)

  )
  return(out)
  
}

#' Run a jackknife
#' 
#' This function uses a jackknife approach to compute person parameters. The jackknife ability measure is based on primarily estimated models (\code{PP_4pl()}, \code{PP_gpcm()} or \code{PPall()}) - so the function is applied on the estimation objects, and jackknifed ability measures are returned.
#' 
#' Please use the Jackknife \bold{Standard-Error} output with \bold{caution}! It is implemented as suggested in Wainer and Wright (1980), but the results seem a bit strange, because the JK-SE is supposed to overestimate the SE compared to the MLE-SE. Actually, in all examples an underestimation of the SE was observed compared to the MLE/WLE-SE! 
#' 
#' 
#' \bold{AMT-robustified jackknife:} When choosing \code{cmeth = AMT}, the jackknife ability subsample estimates and the original supplied ability estimate are combined to a single jackknife-ability value by the Sine M-estimator. The AMT (or Sine M-estimator) is one of the winners in the Princeton Robustness Study of 1972. To get a better idea how the estimation process works, take a closer look to the paper which is mentioned below (Wainer & Wright, 1980).
#' 
#' @param estobj An object which originates from using \code{PP_gpcm()}, \code{PP_4pl()} or \code{PPall()}.
#' 
#' @param ...  More input.
#' @export
#' @rdname Jkpp
#' @references
#' Wainer, H., & Wright, B. D. (1980). Robust estimation of ability in the Rasch model. Psychometrika, 45(3), 373-391.
#'
#'
#' @seealso \link{PP_gpcm}, \link{PP_4pl}, \link{PPall}
#' 
#'@example ./R/.examples_JK.R
#'@author Manuel Reif
JKpp <- function(estobj,...) UseMethod("JKpp")


# ---------------------------------------------------------------------


#' @rdname Jkpp
#' @param cmeth Choose the centering method, to summarize the n jackknife results to one single ability estimate. There are three valid entries: "mean", "median" and "AMT" (see Details for further description).
#' @param maxsteps The maximum number of steps the NR Algorithm will take.
#' @param exac How accurate are the estimates supposed to be? Default is 0.001.
#' @param fullmat Default = FALSE. If TRUE, the function returns the whole jackknife matrix, which is the basis for the jackknife estimator.
#' @param ctrl More controls
#' 
#' @method JKpp fourpl
#' @export
JKpp.fourpl <- function(estobj, cmeth="mean", maxsteps=500,
                        exac=0.001, fullmat=FALSE,ctrl=list(), ...)
{
  
call <- match.call()  
attr(call, "date") <- date() 
attr(call,"version") <- packageVersion("PP")

# pick out objects  
respm <- estobj$ipar$respm
thres <- estobj$ipar$thres
slopes <- estobj$ipar$slopes
lowerA <- estobj$ipar$lowerA
upperA <- estobj$ipar$upperA
#theta_start <- estobj$ipar$theta_start
theta_start <- rep(0,nrow(respm))
mu <- estobj$ipar$mu
sigma2 <- estobj$ipar$sigma2
cont <- estobj$ipar$cont
H <- estobj$ipar$H

# in case the killdupli option = TRUE 
## if not, create a vector which indicates merely each row
if(!is.null(estobj$ipar$dupvec$posvec))
  {
    
  POS <- estobj$ipar$dupvec$posvec
  } else 
    {
      POS <- 1:nrow(respm)  
    }
    
type <- estobj$type 
loa <- 1:ncol(respm)
# save the responses into this matrix
jk_mat <- matrix(0,nrow=nrow(respm),ncol=length(loa))

#browser()
  
# run the 4pl jackknife 
if(type %in% c("mle","wle","map","robust"))
  {
    
  for(jkrun in loa)
    {
    
    jk_mat[,jkrun] <- NR_4PL(respm[,-jkrun,drop=FALSE],DELTA = thres[,-jkrun,drop=FALSE],ALPHA = slopes[-jkrun], LOWA = lowerA[-jkrun],UPPA = upperA[-jkrun], THETA = theta_start, wm=type,maxsteps,exac,mu,sigma2,H=H)$resPP[,1] 
        
    }
  
  } else if(type == "eap")
      {
    
    
      for(jkrun in loa)
        {
            jk_mat[,jkrun] <- eap_4pl(respm[,-jkrun,drop=FALSE], thres[,-jkrun,drop=FALSE],
                                      slopes[-jkrun], lowerA=lowerA[-jkrun],
                                      upperA=upperA[-jkrun],mu = mu, sigma2 = sigma2)[,1]
        }
      }


notna <- !is.na(estobj$resPP$resPP[,2])

RES <- estobj$resPP$resPP[notna,1]

psvalues  <- RES *ncol(respm) - jk_mat[POS,] * (ncol(respm) - 1)

output_jk <- cco(psvalues,cmeth) 

### jk standard errors - see page 388
L <- ncol(respm)
Lm1 <- L - 1

jk_se <- sqrt(rowSums((jk_mat[POS,] - output_jk)^2,na.rm=TRUE)/(L*Lm1))


resjk_raw <- cbind(output_jk,jk_se)

if(cont$killdupli)
  {
    resjk_raw <- resjk_raw[POS,]
  }
  

resjk <- estobj$resPP$resPP
resjk[notna,] <- resjk_raw
  
if(fullmat)
  {
  
  endres <- list(resjk=resjk,call=call,type=type,jk_mat=jk_mat)  
    
  } else 
    {
      
    endres <- list(resjk=resjk,call=call,type=type)
      
    }

class(endres) <- c("jk")
return(endres)

}







# ---------------------------------------------------------------------

#' @rdname Jkpp
#' @method JKpp gpcm
#' @export
JKpp.gpcm <- function(estobj, cmeth="mean", maxsteps=500, 
                      exac=0.001, fullmat=FALSE,  ctrl=list(), ...)
{
  
  call <- match.call()  
  attr(call, "date") <- date() 
  attr(call,"version") <- packageVersion("PP")
  
  
  # pick out objects  
  respm <- estobj$ipar$respm
  thres <- estobj$ipar$thres
  slopes <- estobj$ipar$slopes
#   lowerA <- estobj$ipar$lowerA
#   upperA <- estobj$ipar$upperA
  #theta_start <- estobj$ipar$theta_start
  theta_start <- rep(0,nrow(respm))
  mu <- estobj$ipar$mu
  sigma2 <- estobj$ipar$sigma2
  cont <- estobj$ipar$cont
  H <- estobj$ipar$H  


if(!is.null(estobj$ipar$dupvec$posvec))
  {
    
    POS <- estobj$ipar$dupvec$posvec
  } else 
    {
      POS <- 1:nrow(respm)  
    }

# skip -inf   

  type <- estobj$type 
  loa <- 1:ncol(respm)
  # save the responses into this matrix
  jk_mat <- matrix(0,nrow=nrow(respm),ncol=length(loa))
  
  


if(type %in% c("mle","wle","map","robust"))
{
  
  for(jkrun in loa)
  {
    
    
    jk_mat[,jkrun] <- NR_GPCM(respm[,-jkrun,drop=FALSE], thres[,-jkrun,drop=FALSE],
                              slopes[-jkrun], theta_start, type,
                              maxsteps, exac, mu, sigma2,H=H)$resPP[,1] 
    
  }
  
} else if(type == "eap")
    {
      
      for(jkrun in loa)
        {
        jk_mat[,jkrun] <- eap_gpcm(respm[,-jkrun,drop=FALSE], thres[,-jkrun,drop=FALSE], slopes[-jkrun],
                                   mu = mu, sigma2 = sigma2)[,1]
        }
    }





  
  notna <- !is.na(estobj$resPP$resPP[,2])
  
  RES <- estobj$resPP$resPP[notna,1]
  
  psvalues  <- RES *ncol(respm) - jk_mat[POS,] * (ncol(respm) - 1)
  
  output_jk <- cco(psvalues,cmeth) 
  
  ### jk standard errors - see page 388
  L <- ncol(respm)
  Lm1 <- L - 1
  
  jk_se <- sqrt(rowSums((jk_mat[POS,] - output_jk)^2,na.rm=TRUE)/(L*Lm1))
  
  resjk_raw <- cbind(output_jk,jk_se)
  
  if(cont$killdupli)
    {
      resjk_raw <- resjk_raw[POS,]
    }

  
  resjk <- estobj$resPP$resPP
  resjk[notna,] <- cbind(output_jk,jk_se)
  
  if(fullmat)
    {
      
      endres <- list(resjk=resjk,call=call,type=type,jk_mat=jk_mat)  
      
    } else 
      {
        
        endres <- list(resjk=resjk,type=type,call=call)
        
      }

  class(endres) <- c("jk")
  return(endres)

}





# ---------------------------------------------------------------------

#' @rdname Jkpp
#' @method JKpp gpcm4pl
#' @export 
JKpp.gpcm4pl <- function(estobj, cmeth="mean", maxsteps=500,
                         exac=0.001, fullmat=FALSE, ctrl=list(), ...)
{
  
  call <- match.call()  
  attr(call, "date") <- date() 
  attr(call,"version") <- packageVersion("PP")
  
  # pick out objects  
  respm  <- estobj$ipar$respm
  thres  <- estobj$ipar$thres
  slopes <- estobj$ipar$slopes
  lowerA <- estobj$ipar$lowerA
  upperA <- estobj$ipar$upperA
  #theta_start <- estobj$ipar$theta_start
  theta_start <- rep(0,nrow(respm))
  mu <- estobj$ipar$mu
  sigma2 <- estobj$ipar$sigma2
  cont <- estobj$ipar$cont
  H <- estobj$ipar$H
  model2est <- estobj$ipar$model2est

  if(!is.null(estobj$ipar$dupvec$posvec))
    {
      
      POS <- estobj$ipar$dupvec$posvec
    } else 
      {
        POS <- 1:nrow(respm)  
      }
  
  
  type <- estobj$type 
  loa <- 1:ncol(respm)
  # save the responses into this matrix
  jk_mat <- matrix(0,nrow=nrow(respm),ncol=length(loa))
  
  
  
  if(type %in% c("mle","wle","map","robust"))
  {
    
    for(jkrun in loa)
    {
      
      
      jk_mat[,jkrun] <- NR_mixed(awm=respm[,-jkrun,drop=FALSE],DELTA = thres[,-jkrun,drop=FALSE],ALPHA = slopes[-jkrun],LOWA = lowerA[-jkrun],UPPA = upperA[-jkrun],THETA = theta_start, model=model2est[-jkrun],wm=type,maxsteps=maxsteps,
                                 exac=exac,mu=mu,sigma2=sigma2,H=H)$resPP[,1] 
      
    }
    
  } else if(type == "eap")
    {
      
      for(jkrun in loa)
        {
        jk_mat[,jkrun] <- eap_mixed(respm[,-jkrun,drop=FALSE], thres[,-jkrun,drop=FALSE],
                                    slopes[-jkrun], lowerA=lowerA[-jkrun],
                                    upperA=upperA[,-jkrun], mu = mu, sigma2 = sigma2,
                                    model2est=model2est[-jkrun])[,1]
        }
    }
    
  

  

  notna <- !is.na(estobj$resPP$resPP[,2])
  
  RES <- estobj$resPP$resPP[notna,1]
  
  psvalues  <- RES *ncol(respm) - jk_mat[POS,] * (ncol(respm) - 1)
  
  output_jk <- cco(psvalues,cmeth) 
  
  ### jk standard errors - see page 388
  L <- ncol(respm)
  Lm1 <- L - 1
  
  jk_se <- sqrt(rowSums((jk_mat[POS,] - output_jk)^2,na.rm=TRUE)/(L*Lm1))
  
  resjk_raw <- cbind(output_jk,jk_se)
  
  if(cont$killdupli)
    {
      resjk_raw <- resjk_raw[POS,]
    }
    
  
  resjk <- estobj$resPP$resPP
  resjk[notna,] <- cbind(output_jk,jk_se)
  
  
  if(fullmat)
    {
      
      endres <- list(resjk=resjk,call=call,type=type,jk_mat=jk_mat)  
      
    } else 
      {
        
        endres <- list(resjk=resjk,type=type,call=call)
        
      }
  class(endres) <- c("jk")
  return(endres)
  
}










####### compute AMT
AMTnew <- function(TT,xj)
{
  suppressWarnings(zw <- sin((xj + TT)/2.1))
  nullk <- abs(xj) < 2.1*pi
  zw[!nullk] <- 0
  
  sum(zw)
}


###### compute composed PP

cco <- function(psvalues,cmeth)
{

  if(cmeth=="mean")
  {
    
    jkest <- rowMeans(psvalues,na.rm=TRUE)
    
  } else if(cmeth=="median")
  {
    jkest <- apply(psvalues,1,function(cen) median(cen,na.rm=TRUE))
    
  } else if(cmeth=="AMT")
  {
    
    jkest <- apply(psvalues,1,function(psv)
    {
      negpv <- psv[psv <= 0 & !is.na(psv) & !is.nan(psv)]
      pospv <- psv[psv > 0 & !is.na(psv) & !is.nan(psv)]
      
      if(all(is.nan(psv)))
      {
        yes <- NaN
      } else 
      {
        # wir haben hier bei MLE noch ein problem wenn NaN in psvalues enthalten sind. dasselbe problem gilt auch fuer wle, nur wird das hier wesentlich seltener vorkommen.
        an1 <- optim(0,AMTnew,xj=pospv,method="BFGS",control = list(fnscale=-1)) # minimize
        an2 <- optim(0,AMTnew,xj=negpv,method="BFGS",control = list(fnscale=1)) # maximize
        
        yes <- sum(an1$par*length(pospv) + an2$par*length(negpv))/length(psv)
        yes
        
      }
      
    })
    
  }    
  
return(jkest)  
}

# tests #########################
########## 2/3/4PL Model ########
################################################################
context("Jackknife")

# ------------------------- testing 1>>>


set.seed(1622)
# intercepts
diffpar <- seq(-3,3,length=12)
# slope parameters
sl     <- round(runif(12,0.5,1.5),2)
la     <- round(runif(12,0,0.25),2)
ua     <- round(runif(12,0.8,1),2)

# response matrix
awm <- matrix(sample(0:1,10*12,replace=TRUE),ncol=12)

awm <- rbind(awm,c(rep(1,11),0))
awm <- rbind(awm,rep(1,12))
awm <- rbind(awm,rep(1,12))



## 1PL model ##### 

# MLE estimation
res1plmle <- suppressWarnings(PP_4pl(respm = awm,thres = diffpar, slopes = rep(1,length(diffpar)),type = "mle"))
# WLE estimation
res1plwle <- suppressWarnings(PP_4pl(respm = awm,thres = diffpar, slopes = rep(1,length(diffpar)),type = "wle"))
# MAP estimation
res1plmap <- suppressWarnings(PP_4pl(respm = awm,thres = diffpar, slopes = rep(1,length(diffpar)),type = "map"))



res_jk1 <- JKpp(res1plmle)
res_jk2 <- JKpp(res1plwle)
res_jk3 <- JKpp(res1plmap)


test_that("result = matrix with same dimension",{
  expect_that(dim(res1plmle$resPP$resPP),is_identical_to(dim(res_jk1$resjk)))
  expect_that(dim(res1plwle$resPP$resPP),is_identical_to(dim(res_jk2$resjk)))
  expect_that(dim(res1plmap$resPP$resPP),is_identical_to(dim(res_jk3$resjk)))
})

#'@export
#'@rdname PPall
#'@param x an object of class \code{gpcm4pl} which is the result of using the \code{PPall()} function
#'@param ... just some points.
#'@method print ppeo
print.ppeo <-
  function(x, ...)
  {
  print(apply(x$resPP$resPP,2,function(qq) round(qq,4)))
  }

    

#'@export
#'@rdname PPall
#'@param object An object of class \code{gpcm4pl} which is the result of using the \code{PPall()} function
#'@param nrowmax When printing the matrix of estimates - how many rows should be shown? Default = 15.
#'@method summary ppeo
summary.ppeo <-
  function(object, nrowmax=15, ...)
  {
  cat("PP Version: ",as.character(attr(object$call,"version")),"\n")
  cat("\n Call:",deparse(object$call),"\n- job started @",attr(object$call,"date"),"\n\n")  
  
  cat("Estimation type:",object$type,"\n\n")
  
  cat("Number of iterations:",  object$resPP$nsteps,"\n")
  
  cat("-------------------------------------\n")
  
  respps <- apply(object$resPP$resPP,2,function(x) round(x,4))
  if(nrow(respps) <= nrowmax)
    {
      print(respps)
    } else
      {
        print(respps[1:nrowmax,])
        cat("--------> output truncated <--------\n")
      }
  
  }


#'@export
#'@rdname Jkpp
#'@param x an object of class \code{jk} which is the result of using the \code{JKpp()} function
#'@method print jk
print.jk <-
  function(x, ...)
  {
    print(apply(x$resjk,2,function(qq) round(qq,4)))
  }






#'@export
#'@rdname Jkpp
#'@param object An object of class \code{jk} which is the result of using the \code{JKpp()} function
#'@param nrowmax When printing the matrix of estimates - how many rows should be shown? Default = 15.
#'@method summary jk
summary.jk <-
  function(object, nrowmax=15, ...)
  {
    cat("PP Version: ",as.character(attr(object$call,"version")),"\n")
    cat("\n Call:",deparse(object$call),"\n- job started @",attr(object$call,"date"),"\n\n")  
    
    cat("Estimation type: Jackknife --> ",object$type,"\n\n")
    
    
    cat("-------------------------------------\n")
    
    respps <- apply(object$resjk,2,function(x) round(x,4))
    if(nrow(respps) <= nrowmax)
    {
      print(respps)
    } else
    {
      print(respps[1:nrowmax,])
      cat("--------> output truncated <--------\n")
    }
    
  }








#'@export
#'@rdname PV
#'@param x An object of class \code{pv} which is the result of using the \code{PV()} function
#'@method print pv
print.pv <-
  function(x, ...)
  {
    print(apply(x$pvdraws,2,function(qq) round(qq,4)))
  }






#'@export
#'@rdname PV
#'@param object An object of class \code{pv} which is the result of using the \code{PV()} function
#'@param nrowmax When printing the matrix of estimates - how many rows should be shown? Default = 15.
#'@method summary pv
summary.pv <-
  function(object, nrowmax=15, ...)
  {
    cat("PP Version: ",as.character(attr(object$call,"version")),"\n")
    cat("\n Call:",deparse(object$call),"\n- job started @",attr(object$call,"date"),"\n\n")  
    
    cat("Estimation type: Plausible values \n\n")
    
    
    cat("-------------------------------------\n")
    
    respps <- apply(object$pvdraws,2,function(x) round(x,4))
    if(nrow(respps) <= nrowmax)
    {
      print(respps)
    } else
    {
      print(respps[1:nrowmax,])
      cat("--------> output truncated <--------\n")
    }
    
  }

make_dup <- function(respm)
  {
    
  patts <- apply(respm,1,function(x) paste(x,collapse=""))
  ndpat <- !duplicated(patts)
  seppat <- patts[ndpat]
  posvec <- vector(mode="integer",length = length(patts))
  
  for(i in 1:length(seppat))
    {
      posvec[patts %in% seppat[i]] <- i   
    }
  return(list(posvec=posvec,ndpat=ndpat))
  }  

#' Simulate data for the gpcm model
#'
#' This function returns an integer matrix of simulated responses under given item and person parameters.
#'
#'
#'
#'@param thres An numeric matrix which contains threshold parameters for each item. The first row must contain zeroes only!
#'@param alpha A numeric vector, which contains the slope parameters - one parameter per item is expected.
#'@param theta A numeric vector which contains person parameters.
#'
#'@example ./R/.examples_simgpcm.R
#'
#'@seealso \link{sim_4pl}, \link{PP_gpcm}
#'
#'@export
#'
sim_gpcm <- function(thres, alpha, theta)
{

numbcat <- apply(thres,2,function(x) length(x)-sum(is.na(x)) - 1)  
nitem <- ncol(thres)
whereNna <- apply(thres,2,function(x) !is.na(x))

  
mat01 <- sapply(theta, function(th)
    {
      
      
    persX <- sapply(1:nitem, function(it)
        {
        thr <- thres[whereNna[,it],it]
        
        prob <- sapply(0:numbcat[it],function(ct) P_gpcm(thr,alpha[it],th,ct))
        sample(0:numbcat[it],1,prob=prob)
        
        })

    persX  
    })  

t(mat01)
}


#THETA <- rnorm(2000)


#system.time(test <- sim_gpcm(thres = THRES1,alpha = sl,theta = sort(THETA)))
# slow! :-/

stdrdize_hist <- function(At,quadnodes)
{
  
  weigm <- weighted.mean(quadnodes,At) # compute weighted mean
  quadnodesC <- quadnodes - weigm      # centering nodes
  
  weigSD <- sqrt(sum((At*(quadnodesC)^2)/sum(At))) # compute weighted sd
  
  quadnodesCS <- quadnodesC/weigSD # scale nodes
  
  At_new <- At *weigSD  # rescale weights
  
  DELTA <- quadnodesCS[2] - quadnodesCS[1] # estimate the difference between two adjacent categories
  LG <- length(At_new)  # number of nodes
  
  expolERG <- sapply(quadnodes,function(x) # extra/interpolate between nodes - see woods article of EH
  {
    if(x < min(quadnodesCS))
    {
      
      eins <- (At_new[1]/At_new[2])^((quadnodesCS[1] - x)/DELTA)
      Nq <- eins * At_new[1]
      
    } else if(x > max(quadnodesCS))
    {
      eins <- (At_new[LG]/At_new[LG-1])^((x - quadnodesCS[LG])/DELTA)
      Nq <- eins * At_new[LG]
    } else {
      
      drueber <- which(x < quadnodesCS)[1] # gibt mir den darueberliegenden node
      drunter <- which(x > quadnodesCS)[length(which(x > quadnodesCS))]
      
      Nq <- (x - quadnodesCS[drunter])/DELTA * (At_new[drueber] - At_new[drunter]) + At_new[drunter]
    }
    
  }) 

  # normalize and name
  weights <- expolERG / sum(expolERG)
  nodes   <- quadnodes
  
  return(list(nodes=nodes,weights=weights))  
   
}

#' Simulated data set
#' 
#' This dataset contains data simulated with the \code{sim_4pl()} function.
#'
#' @format A data.frame with 60 rows and 14 columns.
#' @source simulation
#' @seealso \link{sim_4pl}
#' @name fourpl_df
NULL

#' Adaptive Matrices Test data
#' 
#' This dataset contains real data from the 'Adaptive Matrices Test' (AMT), which is a computer-administered test.
#' This power test assesses logical reasoning as an indicator of general intelligence. The ability to identify regularities and draw logical conclusions is a very good predictor of long-term success at work. The dataset is sparse, because the test tailores a specific set of items for each examinee's ability level. (More information about adaptive testing: \url{https://en.wikipedia.org/wiki/Computerized_adaptive_testing})
#'
#' 
#' The data are provided from the Unitersity of Vienna, Faculty of Psychology, Department of Psychological Assessment.
#' Thanks to Schuhfried.
#' 
#' @format A list with two data.frames. The first data.frame 'daten_amt' contains 298 columns and 710 rows. Each row contains responses from on examinee. The second data.frame 'betas' contains the difficulty parameter (1PL) (These parameters came with the raw-score extraction.).
#' @source Division of Psychological Assessment and Applied Psychometrics, Fakulty of Psychology, University of Vienna
#' \itemize{
#' \item ID: id of person
#' \item AGE: age in years (with ages below 18 and above 34 are collapsed)
#' \item TE_GA:
#' \itemize{
#' \item TE: self-assessment. To pass a psychological assessment course, the students have to complete several hours self assessment on a bunch of tests, to get familiar with them.
#' \item GA: testing for an assessment report. The students also have to test other people (not psychologists nor psychology students) in order to write an assessment report.
#' }
#' 
#' 
#' \item FORM: There are several different versions of this test, which differ in test length, duration etc \ldots
#' \item TIME1: start time
#' \item TIME2: end time
#' \item REL: reliability for each person
#' \item i: items
#' }
#' @references 
#' \itemize{
#' \item Hornke, L. F., Etzel, S., & Rettig, K. (2003). Manual Adaptive Matrices Test (AMT). \emph{Moedling: SCHUHFRIED GmbH}.
#' }
#' 
#' @seealso \link{PPass}
#' @name pp_amt
NULL

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

P_4pl <- function(delta, alpha, theta, la, ua) {
    .Call(`_PP_P_4pl`, delta, alpha, theta, la, ua)
}

P_4pl4wle <- function(delta, alpha, theta, la, ua) {
    .Call(`_PP_P_4pl4wle`, delta, alpha, theta, la, ua)
}

r_huber_4pl <- function(delta, alpha, theta, la, ua, H) {
    .Call(`_PP_r_huber_4pl`, delta, alpha, theta, la, ua, H)
}

L4pl <- function(awm, DELTA, ALPHA, LOWA, UPPA, THETA, map, mu, sigma2) {
    .Call(`_PP_L4pl`, awm, DELTA, ALPHA, LOWA, UPPA, THETA, map, mu, sigma2)
}

L4pl_wle <- function(awm, DELTA, ALPHA, LOWA, UPPA, THETA) {
    .Call(`_PP_L4pl_wle`, awm, DELTA, ALPHA, LOWA, UPPA, THETA)
}

L4pl_robust <- function(awm, DELTA, ALPHA, LOWA, UPPA, THETA, H) {
    .Call(`_PP_L4pl_robust`, awm, DELTA, ALPHA, LOWA, UPPA, THETA, H)
}

NR_4PL <- function(awm, DELTA, ALPHA, LOWA, UPPA, THETA, wm, maxsteps, exac, mu, sigma2, H) {
    .Call(`_PP_NR_4PL`, awm, DELTA, ALPHA, LOWA, UPPA, THETA, wm, maxsteps, exac, mu, sigma2, H)
}

P_gpcm <- function(delta, alpha, theta, resp) {
    .Call(`_PP_P_gpcm`, delta, alpha, theta, resp)
}

r_huber_gpcm <- function(delta, alpha, theta, H) {
    .Call(`_PP_r_huber_gpcm`, delta, alpha, theta, H)
}

L12gpcm <- function(awm, DELTA, ALPHA, THETA, mu, sigma2, map) {
    .Call(`_PP_L12gpcm`, awm, DELTA, ALPHA, THETA, mu, sigma2, map)
}

Pcorr1_gpcm <- function(delta, alpha, theta) {
    .Call(`_PP_Pcorr1_gpcm`, delta, alpha, theta)
}

L12gpcm_wle <- function(awm, DELTA, ALPHA, THETA) {
    .Call(`_PP_L12gpcm_wle`, awm, DELTA, ALPHA, THETA)
}

L12gpcm_robust <- function(awm, DELTA, ALPHA, THETA, H) {
    .Call(`_PP_L12gpcm_robust`, awm, DELTA, ALPHA, THETA, H)
}

NR_GPCM <- function(awm, DELTA, ALPHA, THETA, wm, maxsteps, exac, mu, sigma2, H) {
    .Call(`_PP_NR_GPCM`, awm, DELTA, ALPHA, THETA, wm, maxsteps, exac, mu, sigma2, H)
}

Lgpcm4pl_mle <- function(awm, DELTA, ALPHA, LOWA, UPPA, THETA, model, mu, sigma2, map) {
    .Call(`_PP_Lgpcm4pl_mle`, awm, DELTA, ALPHA, LOWA, UPPA, THETA, model, mu, sigma2, map)
}

Lgpcm4pl_wle <- function(awm, DELTA, ALPHA, LOWA, UPPA, THETA, model) {
    .Call(`_PP_Lgpcm4pl_wle`, awm, DELTA, ALPHA, LOWA, UPPA, THETA, model)
}

Lgpcm4pl_robust <- function(awm, DELTA, ALPHA, LOWA, UPPA, THETA, model, H) {
    .Call(`_PP_Lgpcm4pl_robust`, awm, DELTA, ALPHA, LOWA, UPPA, THETA, model, H)
}

NR_mixed <- function(awm, DELTA, ALPHA, LOWA, UPPA, THETA, model, wm, maxsteps, exac, mu, sigma2, H) {
    .Call(`_PP_NR_mixed`, awm, DELTA, ALPHA, LOWA, UPPA, THETA, model, wm, maxsteps, exac, mu, sigma2, H)
}

Likgpcm <- function(awv, DELTA, ALPHA, nodes) {
    .Call(`_PP_Likgpcm`, awv, DELTA, ALPHA, nodes)
}

#' Simulate data for 1/2/3/4-pl model
#' 
#' This function returns a dichotomous matrix of simulated responses under given item and person parameters.
#' 
#' @useDynLib PP
#' @importFrom Rcpp evalCpp sourceCpp
#' 
#'@param beta A numeric vector which contains the difficulty parameters for each item.
#'@param alpha A numeric vector, which contains the slope parameters for each item.
#'@param lowerA A numeric vector, which contains the lower asymptote parameters (kind of guessing parameter) for each item.
#'@param upperA numeric vector, which contains the upper asymptote parameters for each item.
#'@param theta A numeric vector which contains person parameters.
#'
#'@seealso \link{sim_gpcm}, \link{PP_4pl}
#'@example ./R/.examples_sim.R
#'
#'@author Manuel Reif
#'
#' @export
sim_4pl <- function(beta, alpha, lowerA, upperA, theta) {
    .Call(`_PP_sim_4pl`, beta, alpha, lowerA, upperA, theta)
}

ansol <- function(awm, maxsc) {
    .Call(`_PP_ansol`, awm, maxsc)
}

#' Estimate Person Parameters for the 4-PL model
#' 
#' Compute Person Parameters for the 1/2/3/4-PL model and choose between five common estimation techniques: ML, WL, MAP, EAP and a robust estimation. All item parameters are treated as fixed.
#'
#' 
#' With this function you can estimate:
#' \itemize{
#' \item \bold{1-PL model} (Rasch model) by submitting: the data matrix, item difficulties and \bold{nothing else}, since the 1-PL model is merely a 4-PL model with: any slope = 1, any lower asymptote = 0 and any upper asymptote = 1!
#' \item \bold{2-PL model} by submitting: the data matrix, item difficulties and slope parameters. Lower and upper asymptotes are automatically set to 0 und 1 respectively.
#' \item \bold{3-PL model} by submitting anything except the upper asymptote parameters
#' \item \bold{4-PL model} ---> submit all parameters ...
#'}
#' 
#' The probability function of the 4-PL model is:
#'\deqn{P(x_{ij} = 1 | \hat \alpha_i, \hat\beta_i, \hat\gamma_i, \hat\delta_i, \theta_j ) = \hat\gamma_i + (\hat\delta_i-\hat\gamma_i) \frac{exp(\hat \alpha_i (\theta_{j} - \hat\beta_{i}))}{\,1 + exp(\hat\alpha_i (\theta_{j} - \hat\beta_{i}))}}
#' 
#' In our case \eqn{\theta} is to be estimated, and the four item parameters are assumed as fixed (usually these are estimates of a former scaling procedure).
#' 
#' The 3-PL model is the same, except that \eqn{\delta_i = 1, \forall i}.
#' 
#' In the 2-PL model \eqn{\delta_i = 1, \gamma_i = 0, \forall i}.
#' 
#' In the 1-PL model \eqn{\delta_i = 1, \gamma_i = 0, \alpha_i = 1, \forall i}.
#' 
#' .
#' 
#' The \bold{robust} estimation method, applies a Huber-type estimator (Schuster & Yuan, 2011), which downweights responses to items which provide little information for the ability estimation. First a residuum is estimated and on this basis, the weight for each observation is computed.
#' 
#' residuum:
#' \deqn{r_i = \alpha_i(\theta - \beta_i)}
#' 
#' weight:
#' 
#' \deqn{w(r_i) =  1 \rightarrow if\, |r_i| \leq H}
#' \deqn{w(r_i) = H/|r| \rightarrow if\, |r_i| > H}
#' 
#'@param respm An integer matrix, which contains the examinees responses. A persons x items matrix is expected.
#'@param thres A numeric vector or a numeric matrix which contains the threshold parameter (also known as difficulty parameter or beta parameter) for each item. If a matrix is submitted, the first row must contain only \bold{zeroes}! 
#'@param slopes A numeric vector, which contains the slope parameters for each item - one parameter per item is expected.
#'@param lowerA A numeric vector, which contains the lower asymptote parameters (kind of guessing parameter) for each item.
#'@param upperA numeric vector, which contains the upper asymptote parameters for each item.
#'@param theta_start A vector which contains a starting value for each person. If NULL is submitted, the starting values are set automatically. If a scalar is submitted, this start value is used for each person.
#'@param mu A numeric vector of location parameters for each person in case of MAP or EAP estimation. If nothing is submitted this is set to 0 for each person for MAP estimation.
#'@param sigma2 A numeric vector of variance parameters for each person in case of MAP or EAP estimation. If nothing is submitted this is set to 1 for each person for MAP estimation.
#'@param type Which maximization should be applied? There are five valid entries possible: "mle", "wle", "map", "eap" and "robust". To choose between the methods, or just to get a deeper understanding the papers mentioned below are quite helpful. The default is \code{"wle"} which is a good choice in many cases. 
#'@param maxsteps The maximum number of steps the NR Algorithm will take. Default = 100.
#'@param exac How accurate are the estimates supposed to be? Default is 0.001.
#'@param H In case \code{type = "robust"} a Huber ability estimate is performed, and \code{H} modulates how fast the downweighting takes place (for more Details read Schuster & Yuan 2011). 
#'@param ctrl more controls:
#'\itemize{
#' \item \code{killdupli}: Should duplicated response pattern be removed for estimation (estimation is faster)? This is especially resonable in case of a large number of examinees and a small number of items.  Use this option with caution (for map and eap), because persons with different \code{mu} and \code{sigma2} will have different ability estimates despite they responded identically. Default value is \code{FALSE}.
#'
#'\item \code{skipcheck}: Default = FALSE. If TRUE data matrix and arguments are not checked - this saves time e.g. when you use this function for simulations.
#'
#'}
#'
#'@template resulttemplate
#'
#' @seealso \link{PPass}, \link{PPall}, \link{PP_gpcm}, \link{JKpp}, \link{PV}
#'
#' @useDynLib PP, .registration = TRUE
#' @importFrom Rcpp evalCpp sourceCpp
#'
#' @import utils
#' @import stats
#' @import graphics
#'
#'@export
#'
#'@author Manuel Reif
#'@references 
#'Baker, Frank B., and Kim, Seock-Ho (2004). Item Response Theory - Parameter Estimation Techniques. CRC-Press.
#'
#' Barton, M. A., & Lord, F. M. (1981). An Upper Asymptote for the Three-Parameter Logistic Item-Response Model.
#'
#' Birnbaum, A. (1968). Some latent trait models and their use in inferring an examinee's ability. In Lord, F.M. & Novick, M.R. (Eds.), Statistical theories of mental test scores. Reading, MA: Addison-Wesley.
#'
#'Magis, D. (2013). A note on the item information function of the four-parameter logistic model. Applied Psychological Measurement, 37(4), 304-315.
#'
#'Samejima, Fumiko (1993). The bias function of the maximum likelihood estimate of ability for the dichotomous response level. Psychometrika,  58, 195-209.
#'
#'Samejima, Fumiko (1993). An approximation of the bias function of the maximum likelihood estimate of a latent variable for the general case where the item responses are discrete. Psychometrika,  58, 119-138.
#'
#'Schuster, C., & Yuan, K. H. (2011). Robust estimation of latent ability in item response models. Journal of Educational and Behavioral Statistics, 36(6), 720-735.
#'
#'Warm, Thomas A. (1989). Weighted Likelihood Estimation Of Ability In Item Response Theory. Psychometrika, 54, 427-450.
#'
#'Yen, Y.-C., Ho, R.-G., Liao, W.-W., Chen, L.-J., & Kuo, C.-C. (2012). An empirical evaluation of the slip correction in the four parameter logistic models with computerized adaptive testing. Applied Psychological Measurement, 36, 75-87.
#'

#'@example ./R/.examples_4pl.R
#'@keywords Person Parameters 4pl
#'@rdname PP_4pl
#'



PP_4pl <- function(respm, thres, slopes=NULL, lowerA=NULL, upperA=NULL, theta_start=NULL,
                   mu = NULL, sigma2 = NULL, type="wle", maxsteps=40, exac=0.001,H=1,ctrl=list())
{
  
  ### 
  call <- match.call()  
  attr(call, "date") <- date() 
  attr(call,"version") <- packageVersion("PP")
  ###

nitem <- ncol(respm)  
  
## --------- user controls
cont <- list(killdupli=FALSE,skipcheck=FALSE)

user_ctrlI <- match(names(ctrl),names(cont))

if(any(is.na(user_ctrlI)))
  {
    notex <- names(ctrl)[is.na(user_ctrlI)]
    warning("The following options in ctrl do not exist: ", paste(notex,collapse=", "))
    ctrl       <- ctrl[!is.na(user_ctrlI)]
    user_ctrlI <- user_ctrlI[!is.na(user_ctrlI)]
  }

  cont[user_ctrlI] <- ctrl
  

## --------- threshold matrix
  
## take care of the threshold 'matrix' - for the 4PL model
# a thres-vector is allowed - but for the internal routines
# it had to be reshaped as a matrix

if(is.matrix(thres))
  {
    #iimm <- nrow(thres) == 1
    
    if(any(thres[1,] != 0))
      {
       thres <- rbind(0,thres)
      }
    
    # compute the maximal score per item
    maxsc <- apply(thres,2,function(x)(length(x) - sum(is.na(x)))-1)
    
    # are there any items with more than 2 categories?
    #allebigger <- any(maxsc > 1)
    
  } else if(is.vector(thres))
      {
        #iimm <- TRUE 
        thres <- rbind(0,thres)
        #allebigger <- FALSE
        maxsc <- apply(thres,2,function(x)(length(x) - sum(is.na(x)))-1)
      }

## starting values
if(is.null(theta_start))
  {
  thresv <- thres[2,]
  theta_start <- apply(respm, 1, function(st)
    {
    mean(thresv[!is.na(st)])  
    })
  } else if(length(theta_start) == 1)
      {
      theta_start <- rep(theta_start, nrow(respm))
      } 
  

## --------- check user inputs
if(!cont$skipcheck) # to save time e.g. when running simulations
{
  ## --------- check user inputs
  checkINP(respm, thres, slopes, theta_start,lowerA,upperA, type)
  # -----------------------------------------------------  
}

# in case map is chosen and no mu and/or sigma2 is/are submitted.
if( (any(is.null(mu)) | any(is.null(sigma2))))
  {
  if(any(is.null(mu))) 
    {
    mu <- rep(0,nrow(respm))
    if(type %in% c("map","eap")) warning("all mu's are set to 0! \n")
    }
  
  if(any(is.null(sigma2))) 
    {
    sigma2 <- rep(1,nrow(respm))
    if(type %in% c("map","eap")) warning("all sigma2's are set to 1! \n")
    }
  
  }

  ## ----------------------
  
  
  
# ----- CHOOSE MODEL -------------#

if(is.null(lowerA) & is.null(upperA) & is.null(slopes))
{modest <- "1pl"} else if(is.null(lowerA) & is.null(upperA)) 
{modest <- "2pl"} else if(is.null(upperA))
{modest <- "3pl"} else if(is.null(lowerA))
{modest <- "3pl_upperA"} else 
{modest <- "4pl"}


cat("Estimating: ",modest,"model ... \n")
cat("type =",type,"\n")

  
  
  #### conditional controls
  
  if(modest == "4pl")
  {
    if(!all(length(slopes) == c(length(lowerA),ncol(respm),length(upperA),ncol(thres)))) stop("Check length of sumitted vectors!\n")  
  } else if(modest == "3pl")
    {
      if(!all(length(slopes) == c(length(lowerA),ncol(respm),ncol(thres)))) stop("Check length of sumitted vectors!\n")  
    } else if(modest == "2pl")
      {
        if(!all(length(slopes) == c(ncol(respm),ncol(thres)))) stop("Check length of sumitted vectors!\n")  
      } else if(modest == "3pl_upperA")
        {
          if(!all(length(slopes) == c(ncol(respm),ncol(thres),length(upperA)))) stop("Check length of sumitted vectors!\n")  
        }
    
  
  
  # ----- ----- -------------#
  
  
  # add NA and Inf in case of mle estimation and full oder 0 score
  if(type=="mle" | type=="robust")
  {
    resPPx <- ansol(respm,maxsc)  
    respm <- respm[!is.na(resPPx[,2]),,drop=FALSE]
  }
  
  
  
  ## kill duplicated if desired #################
  
  if(cont$killdupli)
  {
    # this vector contains the indices, to recreate the large matrix in the end of the estimation
    dupvec <- make_dup(respm)
    respm <- respm[dupvec$ndpat,,drop=FALSE]
  }
  
  
  ##############################################
  
  
  
  
  
  # ----- estimation procedure -------------# 
  

# prepare not submitted vectors
  if(modest == "1pl")
    {
      lowerA <- rep(0,nitem)  
      upperA <- rep(1,nitem)
      slopes <- rep(1,nitem)
    } else if(modest == "2pl")
            {
              lowerA <- rep(0,nitem)  
              upperA <- rep(1,nitem)  
            } else if(modest == "3pl")
              {
                upperA <- rep(1,nitem)  
              } else if(modest == "3pl_upperA")
                {
                  lowerA <- rep(0,nitem)    
              }


if(type %in% c("mle","wle","map"))
  {
    
  resPP <- NR_4PL(respm,DELTA = thres,ALPHA = slopes, LOWA = lowerA, UPPA = upperA, THETA = theta_start, wm=type,maxsteps,exac,mu,sigma2,H=H)
  
  resPP$resPP[,2] <- sqrt(resPP$resPP[,2])
    
  } else if(type == "robust")
    {
    
    resPP <- NR_4PL(respm,DELTA = thres,ALPHA = slopes, LOWA = lowerA, UPPA = upperA, THETA = theta_start, wm=type,maxsteps,exac,mu,sigma2,H=H)
    
    resPP$resPP[,2] <- sqrt(resPP$resPP[,2])
    
    } else if(type == "eap")
        {
          resPP <- list()
          resPP$resPP <- eap_4pl(respm, thres, slopes, lowerA=lowerA, upperA=upperA,
                         mu = mu, sigma2 = sigma2)
          resPP$nsteps <- 0
        }
        

  
### result preperation --------------------------

if(cont$killdupli)
  {
   resPP$resPP <- resPP$resPP[dupvec$posvec,]
  }

if(type=="mle" | type=="robust")
  {
   resPPx[!is.na(resPPx[,2]),] <- resPP$resPP
   resPP$resPP <- resPPx
  }

## ---------------------------------------------

colnames(resPP$resPP) <- c("estimate","SE")

ipar <- list(respm=respm,thres=thres,slopes=slopes,lowerA=lowerA,
             upperA=upperA,theta_start=theta_start,mu=mu,sigma2=sigma2,cont=cont,H=H)


if(cont$killdupli)
  {
  ipar$dupvec <- dupvec 
  }

## ---------------------------------------------
cat("Estimation finished!\n")
rescall <- list(resPP=resPP,call=call,type=type,ipar=ipar)
class(rescall) <- c("fourpl","ppeo")



return(rescall)
  
  

  
  
  
}

context("4PL model")

# ------------------------- testing 1>>>


THRESx  <- c(-2,-1,1,2)
sl     <- c(0.5,1,1.5,1.1)

awmatrix <- matrix(c(1,0,1,0,1,1,1,0),nrow=2,byrow=TRUE)
awmatrix <- rbind(awmatrix,c(1,1,1,1),c(0,0,0,0),c(1,0,1,0),c(1,0,1,0),c(1,1,0,0))

ua     <- c(0.98,0.85,0.9,0.95)
la     <- c(0,0.05,0.12,0.001)


estmod <- rep(c("mle","wle","map"),4)
LA <- vector(mode="list",length=length(estmod))
UA <- vector(mode="list",length=length(estmod))
for(i in c(4,5,6,10,11,12))
  {
    LA[[i]] <- la      
  }

for(i in c(7,8,9,10,11,12))
  {
    UA[[i]] <- ua
  }
    

res234pl_dup1 <- vector(mode="list",length=length(estmod))
res234pl_dup2 <- vector(mode="list",length=length(estmod))


for(a in 1:length(estmod))
  {
    res234pl_dup1[[a]]  <- PP_4pl(awmatrix,THRESx,slopes = sl,type = estmod[[a]],ctrl = list(killdupli=TRUE),upperA = UA[[a]],lowerA = LA[[a]])
  
    res234pl_dup2[[a]] <- PP_4pl(awmatrix,THRESx,slopes = sl,type = estmod[[a]],ctrl = list(killdupli=FALSE),upperA = UA[[a]],lowerA = LA[[a]])
    
  }


#t
test_that("Output = the same - with or without removing duplicates",{

for(te in 1:length(estmod))
{
expect_that(res234pl_dup1[[te]],equals(res234pl_dup1[[te]]))  
}

})




# ------------------------- testing 2>>>


set.seed(1523)
# intercepts
diffpar <- seq(-3,3,length=12)
la     <- round(runif(12,0,0.25),2)
ua     <- round(runif(12,0.8,1),2)
# slope parameters
sl     <- round(runif(12,0.5,1.5),2)

# antwortmatrix (für neue MLE routine)
awm <- matrix(sample(0:1,10*12,replace=TRUE),ncol=12)
awm2 <- awm
awm2[3,2] <- 2

diffparM <- rbind(0,diffpar)
diffparM2 <- diffparM
diffparM2[2,4] <- NA

b1 <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "wle")
b2 <- PP_4pl(respm = awm,thres = diffparM, slopes = sl,type = "wle")

b3 <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "mle")
b4 <- PP_4pl(respm = awm,thres = diffparM, slopes = sl,type = "mle")

#t
test_that("Output = the same on 1,2,3,4pl - with vector or matrix input",{
  expect_that(b1[[1]],equals(b2[[1]]))
  expect_that(b3[[1]],equals(b4[[1]]))
})



# ------------------------- testing 3>>>

#t
test_that("errors - warnings misspelling and length #1",{
  expect_that(PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "aaa"), throws_error())
  expect_that(PP_4pl(respm = awm,thres = diffpar[-1], slopes = sl), throws_error())
  expect_that(PP_4pl(respm = awm[,-1],thres = diffpar, slopes = sl), throws_error()) 
  expect_that(PP_4pl(respm = awm,thres = diffpar[-1], slopes = sl[-1]), throws_error()) 
  expect_that(PP_4pl(respm = awm,thres = diffpar, slopes = sl,lowerA = la[-1]), throws_error()) 
  expect_that(PP_4pl(respm = awm,thres = diffpar[-1], slopes = sl[-1],upperA = ua[-1]), throws_error()) 
  expect_that(PP_4pl(respm = awm,thres = diffpar[-1], slopes = sl[-1],upperA = ua[-1],lowerA = la[-1]), throws_error())
  expect_that(PP_4pl(respm = awm,thres = diffpar[-1], slopes = sl[-1],upperA = ua[-1],lowerA = la[-1]), throws_error())
  expect_that(PP_4pl(respm = awm2,thres = diffpar, slopes = sl), throws_error())
  expect_that(PP_4pl(respm = awm2,thres = diffparM2, slopes = sl), throws_error())
})

quadIT <-
function(nodes=14, mu=0, sigma=1,absrange=5, ngr=1, ergE=NULL) 
{
  # riqv_quer = resulting object of the Estep function. NULL if a parametric prior should be estimated
  # ergE != NULL if nonpar estimation is required
  
  if(all(is.null(ergE)))
  {
    
  nwpgru <- mapply(function(gruu,mug,si)
  {
    
    if(length(nodes)==1)
    {
      
      if(nodes < 7) 
      {
        nodes <- 7
        cat("#nodes == too small --> set to 7\n")
        absrange <- 4
      }
      
      quadP <- seq(absrange*(-1), absrange, length.out = nodes) 
      
      quadweight  <- dnorm(quadP)
      quadweight1 <- quadweight/sum(quadweight)
      quadP_shift <- quadP*si + mug
      
    } else  {
      quadP <- nodes - mean(nodes)
      
      quadweight  <-  dnorm(quadP)
      quadweight1 <- quadweight/sum(quadweight)
      quadP_shift <- quadP*si + mug
    }
    
    list(nodes=quadP_shift,weights=quadweight1)
  },gruu=1:ngr,mug=mu,si=sigma,SIMPLIFY=FALSE)
  
    
  } else { # nonparametric things
    
  # loop over GROUPS 
    nwpgru <- mapply( function(pg,gruu)
    {
      At <- rowSums(pg) # estimate empirical weights
      quadnodes <- seq(absrange*(-1), absrange, length.out = nodes) # create nodes 
      
      if(any(At< 0.0001))
        { # if the values get too small (metric = Expected number of persons) set a small number
        At[At < 0.0001] <- 0.0001
        }
      
      if(gruu == 1) # for reference group or in case there is only one group
        {
        QN <- stdrdize_hist(At,quadnodes) # first group gets an standardized histogram
        } else 
          {
            AtdAt <- At / sum(At)
            QN <- list(nodes=quadnodes,weights=AtdAt)
          }
        
      
  return(QN)
      
    }, pg=ergE$fiqG,gruu=1:ngr, SIMPLIFY=FALSE)
  

    
  }

  
  
  return(nwpgru)  
  
}

## ----loadall------------------------------------------------------------------

library(PP)
library(data.table)


## ----simulate_data1-----------------------------------------------------------
suppressWarnings(RNGversion("3.5.0"))
set.seed(1526)

# intercepts
diffpar <- seq(-3.6,3,length=15)
# slope parameters
sl     <- round(runif(15,0.5,1.5),2)
la     <- round(runif(15,0,0.25),2)
ua     <- round(runif(15,0.8,1),2)


awm <- PP::sim_4pl(beta = diffpar, alpha = sl,
                   lowerA = la, upperA = ua,
                   theta = rnorm(200))

awm <- as.data.frame(awm)

## ----data_wrangling1----------------------------------------------------------
out <- PPass(respdf = awm,thres = diffpar,
             items="all", mod=c("1PL"), 
             fitindices=c("lz","lzstar","infit","outfit"))

# create a data.table
outdt <- data.table(out)

outdt[, ID := 1:.N]
outdt[, roundest := factor(round(estimate,8))]

# Compute Percentile Ranks with the Person Parameters

outdtu <- unique(outdt[,su := .N, keyby=roundest], by="roundest")
outdtu[,PR :=cumsum(su)/sum(su) * 100]

outdt <- outdt[outdtu[,list(roundest,PR)],,on="roundest"]
setorder(outdt,ID)
outdt[, c("su") := NULL]

# done
outdt

## ----simulate_data2-----------------------------------------------------------

awm <- PP::sim_4pl(beta = diffpar, alpha = sl, lowerA = la, 
                   upperA = ua, theta = c(rnorm(500),rnorm(500,0.5,1.2)))

awm <- as.data.frame(awm)



## ----data_wrangling2----------------------------------------------------------
out <- PPass(respdf = awm, thres = diffpar, 
             items="all", mod=c("1PL"), 
             fitindices= c("lz","lzstar","infit","outfit"))


outdt <- data.table(out)

outdt[, ID := 1:.N]
outdt[, agegroup := rep(c("young","old"),each=500)]
outdt[, roundest := factor(round(estimate,8))]

outdtu <- unique(outdt[,su := .N, keyby=c("agegroup","roundest")], by=c("agegroup","roundest"))[,PR :=cumsum(su)/sum(su) * 100, by=agegroup]

outdt <- outdt[outdtu[,list(agegroup,roundest,PR)],,on=c("agegroup","roundest")]

setorder(outdt,ID)
outdt[, c("su") := NULL]
outdt

lzstar <- function(
                    data, 
                    thetas, 
                    betas, 
                    lowerAs=NULL, 
                    slopes=NULL,
                    higherAs=NULL,
                    method=c("mle","wle","map"), 
                    mu=0, 
                    sigma=1 
                    ){ 

  if(is.null(slopes)){slopes <- rep(1,length(betas))}
  if(is.null(lowerAs)){lowerAs <- rep(0,length(betas))}
  if(is.null(higherAs)){higherAs <- rep(1,length(betas))}
  if(!all(apply(data,2,function(x) { all(na.omit(x) %in% 0:1) }))) stop("Please check the input, only 0/1/NA are allowed \n")

  ai <- slopes
  ci <- lowerAs
  di <- higherAs
  # processed Items 
  Xproc <- 1 * !is.na(data)  
  Nproc <- rowSums(Xproc) #instead on ncol(data)
  
  # calculate the propability of each pearson to NOT solve an item
  submatrix <-   t(outer(thetas,betas,'-'))*ai

  # probabilit.
  Pi <- t(ci + ( (( di-ci )*exp(submatrix)) / ( 1+exp(submatrix) ) ))

  Qi <- 1-Pi

  # first derivation of Pi, it is needed for `r`
  d1Pi <- t(((di-ci) * ai * exp(submatrix)) / (1 + exp(submatrix))^2)
  
  # different calculation of ri, in dependend of the choosen method
  # CHECK....
  ri <- d1Pi / (Pi * Qi)
    if( all( method!=c("mle","wle","map") ) ){
      stop("Pleas choose either 'mle', 'wle' or 'map' \n")
    }
      if(method=="mle"){
        # "MLE"
        r0 <- 0
      }
      if(method=="wle"){
        # "WLE"
        d2Pi <- t( ((di-ci) * (ai^2) * exp(submatrix) * (1-exp(submatrix))) / (1 + exp(submatrix))^3 )
        r0 <- rowSums( ri * d2Pi ) / (2 * rowSums( ri * d1Pi  ))       
      }
      if(method=="map"){
        # dependend on the choosen distribution
        r0 <-  (mu - thetas) / sigma^2
      }
  wi            <- log( Pi / Qi )
  Wn            <- rowSums( Xproc*(data - Pi) * wi ,na.rm=TRUE)
  sigmaNtheta2  <- rowSums(wi^2 * Pi  * Qi) / Nproc
  cn            <- rowSums(d1Pi * wi) / rowSums(d1Pi * ri)
  widach        <- wi - cn * ri
  tau2          <- rowSums(widach^2 * Pi * Qi) / Nproc
  
  lzstern       <- (Wn + cn*r0) / sqrt(Nproc * tau2)
  lzstern <- round(lzstern,6)
  out <- matrix(lzstern,ncol=1)
  colnames(out) <- "lzstar"
  return(out)
}

---
title: Analyzing a dataset
author: Manuel Reif
output:
  prettydoc::html_pretty:
    theme: cayman
    highlight: github
---
<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Analyzing a dataset}
%\VignetteEncoding{UTF-8}
-->
# Analyzing a simulated data set


```{r loadall}

library(PP)
library(data.table)

```

## First example

```{r simulate_data1}
suppressWarnings(RNGversion("3.5.0"))
set.seed(1526)

# intercepts
diffpar <- seq(-3.6,3,length=15)
# slope parameters
sl     <- round(runif(15,0.5,1.5),2)
la     <- round(runif(15,0,0.25),2)
ua     <- round(runif(15,0.8,1),2)


awm <- PP::sim_4pl(beta = diffpar, alpha = sl,
                   lowerA = la, upperA = ua,
                   theta = rnorm(200))

awm <- as.data.frame(awm)
```

### Estimating and using person parameters 

```{r data_wrangling1}
out <- PPass(respdf = awm,thres = diffpar,
             items="all", mod=c("1PL"), 
             fitindices=c("lz","lzstar","infit","outfit"))

# create a data.table
outdt <- data.table(out)

outdt[, ID := 1:.N]
outdt[, roundest := factor(round(estimate,8))]

# Compute Percentile Ranks with the Person Parameters

outdtu <- unique(outdt[,su := .N, keyby=roundest], by="roundest")
outdtu[,PR :=cumsum(su)/sum(su) * 100]

outdt <- outdt[outdtu[,list(roundest,PR)],,on="roundest"]
setorder(outdt,ID)
outdt[, c("su") := NULL]

# done
outdt
```


### Using person fit statistics 







## Second example


Now there are two groups.

```{r simulate_data2}

awm <- PP::sim_4pl(beta = diffpar, alpha = sl, lowerA = la, 
                   upperA = ua, theta = c(rnorm(500),rnorm(500,0.5,1.2)))

awm <- as.data.frame(awm)


```

### Estimating and using person parameters 

```{r data_wrangling2}
out <- PPass(respdf = awm, thres = diffpar, 
             items="all", mod=c("1PL"), 
             fitindices= c("lz","lzstar","infit","outfit"))


outdt <- data.table(out)

outdt[, ID := 1:.N]
outdt[, agegroup := rep(c("young","old"),each=500)]
outdt[, roundest := factor(round(estimate,8))]

outdtu <- unique(outdt[,su := .N, keyby=c("agegroup","roundest")], by=c("agegroup","roundest"))[,PR :=cumsum(su)/sum(su) * 100, by=agegroup]

outdt <- outdt[outdtu[,list(agegroup,roundest,PR)],,on=c("agegroup","roundest")]

setorder(outdt,ID)
outdt[, c("su") := NULL]
outdt
```

---
title: Getting started with Person-Fit in PP
author: Jan Steinfeld
output:
  prettydoc::html_pretty:
    theme: cayman
    highlight: github
---
<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Getting started with Person-Fit in PP}
%\VignetteEncoding{UTF-8}
-->
# Getting started with Person-Fit functions

A brief introduction of all currently implemented person-fit functions will be added soon. Currently the **LZ, LZ*** and also the **Infit-Outfit-Statistics** are implemented. We also added the Infit-Outfit-Functions for the Partial-Credit Model. Meanwhile we are working on plots for a better understanding of the person misfit as well as on inference statistic methods.

```{r settings}
knitr::opts_chunk$set(message = FALSE, results='hide')
```

## First example

Now a simple example will be given.
First, we will simulate some data for our hands on example:
```{r start, message=FALSE, warning=FALSE}
library(PP)

suppressWarnings(RNGversion("3.5.0"))
set.seed(1337)

# simulate some intercepts
diffpar <- seq(-3,3,length=15)
# simulate some slope parameters
sl     <- round(runif(15,0.5,1.5),2)
la     <- round(runif(15,0,0.25),2)
ua     <- round(runif(15,0.8,1),2)

# simulate response matrix (not following any IRT model)
awm <- matrix(sample(0:1,100*15,replace=TRUE),ncol=15)
```

We will start with a simple 1PL-Model. First, we have to estimate the person parameters. Here we have to choose an estimation method. It is important, that you can choose **mle, wle or map** only for the LZ and LZ* Index. For the Infit-Outfit statistic we support **mle and wle** estimates.

```{r 1pl, message=FALSE, warning=FALSE}
# MLE
res1plmle <- PP_4pl(respm = awm,thres = diffpar,type = "mle")
# WLE
res1plwle <- PP_4pl(respm = awm,thres = diffpar,type = "wle")
```

We also support the 2PL, 3PL and 4PL Model:
```{r 234-pl, message=FALSE, warning=FALSE}

# ------------------------------------------------------------------------
## 2PL model #####
# ------------------------------------------------------------------------
# MLE
res2plmle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "mle")
# WLE
res2plwle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "wle")

# ------------------------------------------------------------------------
## 3PL model #####
# ------------------------------------------------------------------------
# MLE
res3plmle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,type = "mle")
# WLE
res3plwle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,type = "wle")
# ------------------------------------------------------------------------
## 4PL model #####
# ------------------------------------------------------------------------
# MLE
res4plmle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "mle")
# WLE
res4plwle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "wle")
```

After the estimation of the person parameter we are able to calculate the person fits. At this point you are able to calculate only one kind of personfit as well as all various simultaneously (as shown next).

```{r pfit, message=FALSE, warning=FALSE}

# ------------------------------------------------------------------------
## 1PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
pfit1pl_lz <- Pfit(respm=awm,pp=res1plwle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit #####
pfit1pl_li <- Pfit(respm=awm,pp=res1plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------
## 2PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
pfit2pl_lz <- Pfit(respm=awm,pp=res2plwle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit #####
pfit2pl_li <- Pfit(respm=awm,pp=res2plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------
## 3PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
pfit3pl_lz <- Pfit(respm=awm,pp=res3plwle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit #####
pfit3pl_li <- Pfit(respm=awm,pp=res3plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------
## 4PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
pfit4pl_lz <- Pfit(respm=awm,pp=res4plwle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit #####
pfit4pl_li <- Pfit(respm=awm,pp=res4plwle,fitindices=c("lzstar","infit","outfit"))
```

We can also use different person parameter estimates.

```{r wle, message=FALSE, warning=FALSE}
# ------------------------------------------------------------------------
## 1PL model #####
# ------------------------------------------------------------------------
## LZ*-Index #####
## mle ####
pfit1pl_mle_l <- Pfit(respm=awm,pp=res1plmle,fitindices="lzstar")
## wle ####
pfit1pl_wle_l <- Pfit(respm=awm,pp=res1plwle,fitindices="lzstar")
```

If desired you can simply plot the results of the person-fit statistics as shown below.
```{r example-1, message=FALSE, warning=FALSE}
# eine Grafik erzeugen

res.pp <- Pfit(respm=awm,pp=res1plmle,fitindices=c("lzstar"),SE=TRUE)
x<-seq(-4,4,length=200)
s <- 1
mu <- 0
y <- (1/(s*sqrt(2*pi))) * exp(-((x-mu)^2)/(2*s^2))
plot(x,y, type="l", lwd=2, col = "blue", xlim = c(-8.5,8.5),xlab="", ylab="")
title(main="Density plot of lz* Person-Fit", xlab="density", ylab="score")
lines(density(res.pp$lzstar[,"lzstar"], bw = 0.5), lwd = 2, lty = 2)
rug(res.pp$lzstar[,"lzstar"],col="red")

# zweite Grafik erzeugen
x <- 1:nrow(res.pp$lzstar)
avg <- res.pp$lzstar[,"lzstar"]
sdev <- res.pp$lzstar[,"lzs_se"]

plot(avg, x,
     xlim=range(c(avg-sdev, avg+sdev)),
     pch=19, ylab="Person", xlab="Person-Fit +/- SD",
     main="Plot of Person-Fit with SE"
)
arrows(avg-sdev, x, avg+sdev, length=0.05, angle=90, code=3)
abline(v=0,col = "red", lwd = 3)
```

## Real data example


First we have to load the dataset
```{r example-2, message=FALSE, warning=FALSE}
data(pp_amt)
betas <- pp_amt$betas$Itemparameter
diffpar <- pp_amt$Itemparameter
# slope parameters
awm <- pp_amt$daten_amt[,grep("i\\d{1,3}",colnames(pp_amt$daten_amt))]

# estimate ability parameter and personfit
# the computation of the standard error takes a while, therefore we use only a part of the provided data
suppressWarnings(RNGversion("3.5.0"))
set.seed(1800)
# sample items
sampi <- order(sample(1:ncol(awm),40,replace = F))
# sample persons
sampp <- order(sample(1:nrow(awm),100,replace = F))
awm.samp <- awm[sampp,sampi]
# apply(awm.samp, 2, function(x)!all(is.na(x)))
awm.samp <- awm.samp[apply(awm.samp, 1, function(x)!all(is.na(x))),] #only persons with no NA
out <- PPass(respdf = awm.samp,thres = betas[sampi], items="all",type = "wle",
             mod=c("1PL"), fitindices= c("lz","lzstar","infit","outfit"),SE=TRUE)

# first example of illustration
lim <- max(abs(c(min(out$estimate),max(out$estimate))))
x <- seq(-lim,lim,length=200)
s  <- 1
mu <- 0
y <- (1/(s*sqrt(2*pi))) * exp(-((x-mu)^2)/(2*s^2))
plot(x,y, type="l", lwd=2, col = "blue", xlim = c(-lim,lim),xlab="", ylab="")
title(main="Density plot of lz* Person-Fit", xlab="density", ylab="score")
lines(density(out[,"lzstar"], bw = 0.5), lwd = 2, lty = 2)
rug(out[,"lzstar"],col="red")

# second example of illustration
x <- 1:nrow(out)
avg <- out[,"lzstar"]
sdev <- out[,"lzs_se"]

plot(avg, x,
     yaxt="n",
     xlim=range(c(avg-sdev, avg+sdev)),
     pch=19, ylab="Person", xlab="Person-Fit +/- SD",
     main="Plot of Person-Fit with SE"
)
axis(side=2, at = c(1:nrow(out)),labels = c(1:nrow(out)), las = 2,cex.axis=0.66)
arrows(avg-sdev, x, avg+sdev, length=0.05, angle=90, code=3)
abline(v=0,col = "red", lwd = 3)
```

**Interpretation of some selected person-fit statistics.**

In the second example we used the provided real dataset of the 'Adaptive Matrices Test'. Theses data where collected at the Unitersity of Vienna, Fakulty of Psychology, Division of Psychological Assessment and Applied Psychometrics. In the second plot we used only a subset of the provided data, because the computation of the standard error (here we used a jackknife (Efron & Stein, 1981)) takes a while.
Magis, Raîche & Béland (2012) and also Armstrong, Stoumbos, Kung and Shi (2007) provide useful information for the inrepretation of the lz (Drasgow, Levine & Williams, 1985) and lz* (Snijders, 2001) person-fit index.
The interpretation of the lz-values is very simple. The smaller (negative) the values, the stronger the indicated misfit. The lz-values ar asymptotically standard normally distributed. Molenaar and Hoijtink (1990) (and others) showed, that the asymptotically standard normal distrubution only holds, if the $\theta$ values are known. To overcome thes problem Snijders (2001) intruduced his lz* person-fit index. The inrepration of this index does not change. In summary Magis, Raîche & Béland (2012) propose if the lz* index is lower than a certain critical value $z_\alpha$, than this person-fit value indicates a misfit.


# References
Armstrong, R.D., Z.G. Stoumbos, M.T. Kung, and M. Shi. 2007. On the Performance of the Lz Person-Fit Statistic. *Practical Assessment, Research & Evaluation*, 12(16).

Drasgow, F., M.V. Levine, and E.A. Williams. 1985. Appropriateness Measurement with Polychotomous Item Response Models and Standardized Indices. *British Journal of Mathematical and Statistical Psychology*, 38(1). Wiley Online Library: 67-86.

Efron, B., and C. Stein. 1981. The Jackknife Estimate of Variance. *The Annals of Statistics JSTOR*, 586-96.

Magis, D., G. Raîche, and S. Béland. 2012. A Didactic Presentation of Snijders's Lz* Index of Person Fit with Emphasis on Response Model Selection and Ability Estimation *Journal of Educational and Behavioral Statistics*, 37(1). SAGE Publications Sage CA: Los Angeles, CA: 57-81.

Molenaar, I.W., and H. Hoijtink. 1990. The Many Null Distributions of Person Fit Indices. *Psychometrika*, 55(1). Springer: 75-106.

Snijders, T.AB. 2001. Asymptotic Null Distribution of Person Fit Statistics with Estimated Person Parameter. *Psychometrika*}, 66(3). Springer: 331-342.

Snijders, T.B. 2001. Asymptotic Null Distribution of Person Fit Statistics with Estimated Person Parameter. *Rasch Measurement Transactions*, 66(3): 331-342.

---
title: Getting started with PP
author: Manuel Reif
output:
  prettydoc::html_pretty:
    theme: cayman
    highlight: github
---
<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Getting started with PP}
%\VignetteEncoding{UTF-8}
-->
# Getting started with PP

Is the `PP` package the right package for you?


1. You have a data set with item responses from several persons
2. You have item parameter estimates (e.g. difficulty parameters) from these items (perhaps from a former application of these items)
3. and these items follow the 1/2/3/4-PL model or the generalized partial credit model
4. You are interested in estimating each persons **ability** with traditional approaches (mle, wle, map, eap) or
5. drawing **plausible values** for each person e.g. to apply secondary analysis or
6. getting **robust** ability estimates with Jackknife Methods or with weighted likelihood approaches


If you agree to question 1-3 and at least to one of question 4-6, load the package an start estimating person parameters.


Now a simple example.


## 2-PL Model


In this example, we want to gain ability estimates for each person in our sample. Each person answered questions in a test which has been scaled by the 2-PL model. Item parameters (difficulty and slope) are known and are supposed as fixed. What are the next steps?

1. Get the item responses and the item parameters inside the R workspace
2. Check your data and reshape them, so they can be processed by the PP functions
3. Decide which type of estimation you prefer (`?PP_4pl`)
4. Run the estimation


### Get

Getting the data inside the R workspace is quite easy in this case, because we merely load the data set which already comes with the PP package. It contains 60 response sets (60 persons answered to a test of 12 items) - and we have additional information, which we do not take into account for now. We first inspect the data, and in a second step informations about item difficulty and the slope parameters are extracted from the dataset attributes.

```{r get}

library(PP)

data(fourpl_df)

dim(fourpl_df)

head(fourpl_df)

# extracting the information
diff_par <- attr(fourpl_df,"diffpar")
slope_par <- attr(fourpl_df,"slopes")

```



### Check and Reshape


Now we explore the item response data regarding full or zero score, missing values etc. . Considering this information, we are able to decide which estimation method we will apply.


```{r check}

# extract items and transform the data.frame to matrix
itmat <- as.matrix(fourpl_df[,-(1:2)])


# are there any full scores?
fullsc <- apply(itmat,1,function(x) (sum(x,na.rm=TRUE)+sum(is.na(x))) == length(x))
any(fullsc)

# are there 0 scores?
nullsc <- apply(itmat,1,function(x) sum(x,na.rm=TRUE) == 0)
any(nullsc)

# are there missing values? how many and where?
nasc <- apply(itmat,1,function(x) sum(is.na(x)))
any(nasc > 0)
#which(nasc)

# is our data dichotomous as expected?
apply(itmat,2,function(x) table(x))

# are there any duplicates?
rdup <- duplicated(itmat)
sum(rdup)


```



### Decide and Run


We use the `PP_4pl()` function for our estimation. So perhaps you are thinking:"Why are we using a function to fit the 4-PL model, when we acutally have a dataset which stems from a 2-PL model scaled test?!" This is because with the `PP_4pl()` function you can fit the:

1. **1-PL model** (Rasch model) by submitting: the data matrix, item difficulties and **nothing else**, since the 1-PL model is merely a 4-PL model with: any slope = 1, any lower asymptote = 0 and any upper asymptote = 1!
2. **2-PL model** by submitting: the data matrix, item difficulties and slope parameters. Lower and upper asymptotes are automatically set to 0 und 1 respectively.
3. **3-PL model** by submitting anything except the upper asymptote parameters
4. **4-PL model** $\rightarrow$ submit all parameters ...

In this case, difficulty parameters and slopes are available, so we will submit them, and a 2-PL model is fitted **automatically**.

We decide to apply a common maximum likelihood estimation (`type = "mle"`), and do not remove duplicated response patterns (see argument: `ctrl=list()`) from estimation because there are only `r sum(rdup)` duplicated patterns. If the data set would have been much larger, duplicated patterns are more likely and therefore choosing to remove these patterns would speed up the estimation process significantly. (Choosing this option or not, does not change the numerical results!)


```{r decide}
library(PP)

res1plmle <- PP_4pl(respm = itmat, thres = diff_par, slopes = slope_par, type = "mle")

summary(res1plmle)

```

Some facts:

1. The function returns the **point estimators** and **standard errors** for each person. Whatever the (valid) input was, the main result is always a matrix with two columns and a number of rows which equals the number of persons (`res1plmle$resPP$resPP`).
2. The function deals with **missing values**. These are treated as if the person has never seen this item.
3. The order of the output estimates = order of the input. So the first estimate belongs to the first response pattern of the input data matrix.



### Edit

In the last step, we add the estimates to the data.frame we extracted the item responses from in the first place.


```{r edit}

dafest <- data.frame(fourpl_df, res1plmle$resPP$resPP)

head(dafest,10)
```


One shortcoming of the plain maximum likelihood estimate is the fact, that the **extreme scores** do not lead to valid parameter estimates (`-Inf` and `Inf` are hardly useful for practitioners). One possibility to overcome this issue, is to change the estimation method - for instance `type = wle` performs weighted likelihood estimation, which is on the one hand less biased than the mle estimate, and on the other hand provides reasonable estimates for the extreme scores.


### Rerun


```{r rerun}
library(PP)

res1plwle <- PP_4pl(respm = itmat,thres = diff_par, slopes = slope_par, type = "wle")

summary(res1plwle)
```


So, this was what we were finally looking for.


## Parameters and Person Fit 

For estimating person parameters and examining person fit in one step, use `PPass()` (for ass stands for *assessment*). Using this function has several advantages over the using the other methods consecutively.

1. you ony need 1 Function
2. the data input format now is a `data.frame`
3. the data output format is a new `data.frame` with person fit statistics added


```{r ppass}

pres <- PPass(fourpl_df, items = 3:14, mod="2PL", thres = diff_par, slopes = slope_par, type = "wle")


```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataRd.R
\name{fourpl_df}
\alias{fourpl_df}
\title{Simulated data set}
\format{
A data.frame with 60 rows and 14 columns.
}
\source{
simulation
}
\description{
This dataset contains data simulated with the \code{sim_4pl()} function.
}
\seealso{
\link{sim_4pl}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PP_4pl.R
\name{PP_4pl}
\alias{PP_4pl}
\title{Estimate Person Parameters for the 4-PL model}
\usage{
PP_4pl(
  respm,
  thres,
  slopes = NULL,
  lowerA = NULL,
  upperA = NULL,
  theta_start = NULL,
  mu = NULL,
  sigma2 = NULL,
  type = "wle",
  maxsteps = 40,
  exac = 0.001,
  H = 1,
  ctrl = list()
)
}
\arguments{
\item{respm}{An integer matrix, which contains the examinees responses. A persons x items matrix is expected.}

\item{thres}{A numeric vector or a numeric matrix which contains the threshold parameter (also known as difficulty parameter or beta parameter) for each item. If a matrix is submitted, the first row must contain only \bold{zeroes}!}

\item{slopes}{A numeric vector, which contains the slope parameters for each item - one parameter per item is expected.}

\item{lowerA}{A numeric vector, which contains the lower asymptote parameters (kind of guessing parameter) for each item.}

\item{upperA}{numeric vector, which contains the upper asymptote parameters for each item.}

\item{theta_start}{A vector which contains a starting value for each person. If NULL is submitted, the starting values are set automatically. If a scalar is submitted, this start value is used for each person.}

\item{mu}{A numeric vector of location parameters for each person in case of MAP or EAP estimation. If nothing is submitted this is set to 0 for each person for MAP estimation.}

\item{sigma2}{A numeric vector of variance parameters for each person in case of MAP or EAP estimation. If nothing is submitted this is set to 1 for each person for MAP estimation.}

\item{type}{Which maximization should be applied? There are five valid entries possible: "mle", "wle", "map", "eap" and "robust". To choose between the methods, or just to get a deeper understanding the papers mentioned below are quite helpful. The default is \code{"wle"} which is a good choice in many cases.}

\item{maxsteps}{The maximum number of steps the NR Algorithm will take. Default = 100.}

\item{exac}{How accurate are the estimates supposed to be? Default is 0.001.}

\item{H}{In case \code{type = "robust"} a Huber ability estimate is performed, and \code{H} modulates how fast the downweighting takes place (for more Details read Schuster & Yuan 2011).}

\item{ctrl}{more controls:
\itemize{
\item \code{killdupli}: Should duplicated response pattern be removed for estimation (estimation is faster)? This is especially resonable in case of a large number of examinees and a small number of items.  Use this option with caution (for map and eap), because persons with different \code{mu} and \code{sigma2} will have different ability estimates despite they responded identically. Default value is \code{FALSE}.

\item \code{skipcheck}: Default = FALSE. If TRUE data matrix and arguments are not checked - this saves time e.g. when you use this function for simulations.

}}
}
\value{
The function returns a list with the estimation results and pretty much everything which has been submitted to fit the model. The estimation results can be found in \code{OBJ$resPP}. The core result is a number_of_persons x 2 matrix, which contains the ability estimate and the SE for each submitted person.
}
\description{
Compute Person Parameters for the 1/2/3/4-PL model and choose between five common estimation techniques: ML, WL, MAP, EAP and a robust estimation. All item parameters are treated as fixed.
}
\details{
With this function you can estimate:
\itemize{
\item \bold{1-PL model} (Rasch model) by submitting: the data matrix, item difficulties and \bold{nothing else}, since the 1-PL model is merely a 4-PL model with: any slope = 1, any lower asymptote = 0 and any upper asymptote = 1!
\item \bold{2-PL model} by submitting: the data matrix, item difficulties and slope parameters. Lower and upper asymptotes are automatically set to 0 und 1 respectively.
\item \bold{3-PL model} by submitting anything except the upper asymptote parameters
\item \bold{4-PL model} ---> submit all parameters ...
}

The probability function of the 4-PL model is:
\deqn{P(x_{ij} = 1 | \hat \alpha_i, \hat\beta_i, \hat\gamma_i, \hat\delta_i, \theta_j ) = \hat\gamma_i + (\hat\delta_i-\hat\gamma_i) \frac{exp(\hat \alpha_i (\theta_{j} - \hat\beta_{i}))}{\,1 + exp(\hat\alpha_i (\theta_{j} - \hat\beta_{i}))}}

In our case \eqn{\theta} is to be estimated, and the four item parameters are assumed as fixed (usually these are estimates of a former scaling procedure).

The 3-PL model is the same, except that \eqn{\delta_i = 1, \forall i}.

In the 2-PL model \eqn{\delta_i = 1, \gamma_i = 0, \forall i}.

In the 1-PL model \eqn{\delta_i = 1, \gamma_i = 0, \alpha_i = 1, \forall i}.

.

The \bold{robust} estimation method, applies a Huber-type estimator (Schuster & Yuan, 2011), which downweights responses to items which provide little information for the ability estimation. First a residuum is estimated and on this basis, the weight for each observation is computed.

residuum:
\deqn{r_i = \alpha_i(\theta - \beta_i)}

weight:

\deqn{w(r_i) =  1 \rightarrow if\, |r_i| \leq H}
\deqn{w(r_i) = H/|r| \rightarrow if\, |r_i| > H}
}
\examples{
################# 4 PL #############################################################

### real data ##########

data(pp_amt)

d <- as.matrix(pp_amt$daten_amt[,-(1:7)])

rd_res <- PP_4pl(respm = d, thres = pp_amt$betas[,2], type = "wle")
summary(rd_res)

rd_res1 <- PP_4pl(respm = d, thres = pp_amt$betas[,2], theta_start = 0,type = "wle")
summary(rd_res1)

### fake data ##########
# smaller ... faster

set.seed(1522)
# intercepts
diffpar <- seq(-3,3,length=12)
# slope parameters
sl     <- round(runif(12,0.5,1.5),2)
la     <- round(runif(12,0,0.25),2)
ua     <- round(runif(12,0.8,1),2)

# response matrix
awm <- matrix(sample(0:1,10*12,replace=TRUE),ncol=12)


## 1PL model ##### 

# MLE
res1plmle <- PP_4pl(respm = awm,thres = diffpar,type = "mle")
# WLE
res1plwle <- PP_4pl(respm = awm,thres = diffpar,type = "wle")
# MAP estimation
res1plmap <- PP_4pl(respm = awm,thres = diffpar,type = "map")
# EAP estimation
res1pleap <- PP_4pl(respm = awm,thres = diffpar,type = "eap")
# robust estimation
res1plrob <- PP_4pl(respm = awm,thres = diffpar,type = "robust")

# summarize results
summary(res1plmle)
summary(res1plwle)
summary(res1plmap)


## 2PL model ##### 

# MLE
res2plmle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "mle")
# WLE
res2plwle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "wle")
# MAP estimation
res2plmap <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "map")
# EAP estimation
res2pleap <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "eap")
# robust estimation
res2plrob <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "robust")


## 3PL model ##### 

# MLE
res3plmle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,type = "mle")
# WLE
res3plwle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,type = "wle")
# MAP estimation
res3plmap <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,type = "map")
# EAP estimation
res3pleap <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la, type = "eap")


## 4PL model ##### 

# MLE
res4plmle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "mle")
# WLE
res4plwle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "wle")
# MAP estimation
res4plmap <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "map")
# EAP estimation
res4pleap <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "eap")


## A special on robust estimation:
# it reproduces the example given in Schuster & Ke-Hai 2011:

diffpar <- c(-3,-2,-1,0,1,2,3)

AWM <- matrix(0,7,7)
diag(AWM) <- 1

res1plmle <- PP_4pl(respm = AWM,thres = diffpar, type = "mle")

summary(res1plmle)

res1plrob <- PP_4pl(respm = AWM,thres = diffpar, type = "robust")

summary(res1plrob)
}
\references{
Baker, Frank B., and Kim, Seock-Ho (2004). Item Response Theory - Parameter Estimation Techniques. CRC-Press.

Barton, M. A., & Lord, F. M. (1981). An Upper Asymptote for the Three-Parameter Logistic Item-Response Model.

Birnbaum, A. (1968). Some latent trait models and their use in inferring an examinee's ability. In Lord, F.M. & Novick, M.R. (Eds.), Statistical theories of mental test scores. Reading, MA: Addison-Wesley.

Magis, D. (2013). A note on the item information function of the four-parameter logistic model. Applied Psychological Measurement, 37(4), 304-315.

Samejima, Fumiko (1993). The bias function of the maximum likelihood estimate of ability for the dichotomous response level. Psychometrika,  58, 195-209.

Samejima, Fumiko (1993). An approximation of the bias function of the maximum likelihood estimate of a latent variable for the general case where the item responses are discrete. Psychometrika,  58, 119-138.

Schuster, C., & Yuan, K. H. (2011). Robust estimation of latent ability in item response models. Journal of Educational and Behavioral Statistics, 36(6), 720-735.

Warm, Thomas A. (1989). Weighted Likelihood Estimation Of Ability In Item Response Theory. Psychometrika, 54, 427-450.

Yen, Y.-C., Ho, R.-G., Liao, W.-W., Chen, L.-J., & Kuo, C.-C. (2012). An empirical evaluation of the slip correction in the four parameter logistic models with computerized adaptive testing. Applied Psychological Measurement, 36, 75-87.
}
\seealso{
\link{PPass}, \link{PPall}, \link{PP_gpcm}, \link{JKpp}, \link{PV}
}
\author{
Manuel Reif
}
\keyword{4pl}
\keyword{Parameters}
\keyword{Person}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findmodel.R
\name{findmodel}
\alias{findmodel}
\title{Create a model-type vector template}
\usage{
findmodel(thres)
}
\arguments{
\item{thres}{A numeric matrix which contains the threshold parameter for each item. NA is allowed - in fact expected!}
}
\description{
This is a small helper function which creates a vector template  quick and easily for the \code{PPall()} function. Modify this template as you like.
}
\details{
This function tries to guess the model which was applied to each item by using the matrix of threshold parameters. It only discriminates between GPCM and 4-PL model, and returns a character vector of length equal to the number of items, that contains \code{"GPCM"} or \code{"4PL"} entries depending on the structure of the thres matrix.
}
\examples{
################# GPCM  and 4PL mixed #########################################

# some threshold parameters
THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
# slopes
sl     <- c(0.5,1,1.5,1.1,1,0.98)

THRESx <- THRES
THRESx[2,1:3] <- NA

# for the 4PL item the estimated parameters are submitted, 
# for the GPCM items the lower asymptote = 0 
# and the upper asymptote = 1.
la     <- c(0.02,0.1,0,0,0,0)
ua     <- c(0.97,0.91,1,1,1,1)

awmatrix <- matrix(c(1,0,1,0,1,1,1,0,0,1
                     ,2,0,0,0,0,0,0,0,0,1
                     ,1,2,2,1,1,1,1,0,0,1),byrow=TRUE,nrow=5)

# create model2est
# this function tries to help finding the appropriate 
# model by inspecting the THRESx.
model2est <- findmodel(THRESx)


# MLE
respmixed_mle <- PPall(respm = awmatrix,thres = THRESx, 
                     slopes = sl,lowerA = la, upperA=ua,type = "mle",
                     model2est=model2est)
# WLE
respmixed_wle <- PPall(respm = awmatrix,thres = THRESx, 
                    slopes = sl,lowerA = la, upperA=ua,type = "wle",
                    model2est=model2est)
# MAP estimation
respmixed_map <- PPall(respm = awmatrix,thres = THRESx, 
                    slopes = sl,lowerA = la, upperA=ua, type = "map",
                    model2est=model2est)

# EAP estimation
respmixed_eap <- PPall(respm = awmatrix,thres = THRESx, 
                    slopes = sl,lowerA = la, upperA=ua, type = "eap",
                    model2est=model2est)

# Robust estimation
respmixed_rob <- PPall(respm = awmatrix,thres = THRESx, 
                    slopes = sl,lowerA = la, upperA=ua, type = "robust",
                    model2est=model2est)


# summary to summarize the results
summary(respmixed_mle)
summary(respmixed_wle)
summary(respmixed_map)
summary(respmixed_eap)
summary(respmixed_rob)

}
\seealso{
\link{PPall}
}
\author{
Manuel Reif
}
\keyword{Parameters}
\keyword{Person}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lzstar_mixed.R
\name{lzstar_mixed}
\alias{lzstar_mixed}
\title{Compute lz and lzstar person-fit statistics for mixed-format tests}
\usage{
lzstar_mixed(
  scores,
  theta,
  vtheta,
  a,
  c,
  b,
  extreme_handling = c("na", "cap", "none"),
  theta_cap = 4
)
}
\arguments{
\item{scores}{Matrix of item scores (n_persons x n_items).}

\item{theta}{Vector of ability estimates, length must equal nrow(scores).}

\item{vtheta}{Vector of estimated variances Var(theta_hat), length must equal nrow(scores).}

\item{a}{Vector of discrimination parameters, length must equal ncol(scores).}

\item{c}{Vector of lower asymptote (guessing) parameters, length must equal ncol(scores).}

\item{b}{Matrix of thresholds, with nrow(b) = ncol(scores) and NA padding for shorter items.}

\item{extreme_handling}{Policy for extreme raw scores (all-0 or perfect):
"na" returns NA for that person, "cap" truncates theta to +/- theta_cap, "none" does nothing.}

\item{theta_cap}{Cap used when extreme_handling = "cap".}
}
\value{
A matrix with columns "lz" and "lzstar".
}
\description{
Compute lz and lzstar person-fit statistics for mixed-format tests
}
\details{
The function implements the mixed-format person-fit statistic using 3PL for dichotomous
items and GPCM for polytomous items. The calculation is kept unchanged: dichotomous item
contributions are accumulated with pr3PL, polytomous item contributions are accumulated
with prGPCM_stable and d1prGPCM, and lz and lzstar are computed from
the same denominators used in the implementation.
}
\examples{
scores <- matrix(c(
  1, 0, 2,
  0, 1, 1,
  1, 1, 3
), nrow = 3, byrow = TRUE)
theta  <- c(-0.5, 0.2, 1.1)
vtheta <- c(0.12, 0.10, 0.15)
a      <- c(1.1, 0.9, 1.3)
cpar   <- c(0.15, 0.10, 0.00)
b <- matrix(c(
  0.0,  NA,
 -0.2,  NA,
 -1.0,  0.3
), nrow = 3, byrow = TRUE)
lzstar_mixed(scores, theta, vtheta, a, cpar, b)

scores2 <- matrix(c(
  0, 1,
  1, 0
), nrow = 2, byrow = TRUE)
theta2  <- c(-1, 1)
vtheta2 <- c(0.08, 0.08)
a2      <- c(1.0, 1.2)
c2      <- c(0.05, 0.10)
b2 <- matrix(c(
  0.0, NA,
  0.3, NA
), nrow = 2, byrow = TRUE)
lzstar_mixed(scores2, theta2, vtheta2, a2, c2, b2)

}
\references{
Sinharay, S. (2026). Refining the asymptotically correct standardization of person-fit
statistics for mixed-format tests. \emph{British Journal of Mathematical and Statistical
Psychology}, \bold{00}, 1--24.

Sinharay, S. (2016). Asymptotically correct standardization of person-fit statistics beyond
dichotomous items. \emph{Psychometrika}, \bold{81}(4), 992--1013.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PP_gpcm.R
\name{PP_gpcm}
\alias{PP_gpcm}
\title{Estimate Person Parameters for the GPCM}
\usage{
PP_gpcm(
  respm,
  thres,
  slopes,
  theta_start = NULL,
  mu = NULL,
  sigma2 = NULL,
  type = "wle",
  maxsteps = 100,
  exac = 0.001,
  H = 1,
  ctrl = list()
)
}
\arguments{
\item{respm}{An integer matrix, which contains the examinees responses. A persons x items matrix is expected.}

\item{thres}{A numeric matrix which contains the threshold parameter for each item. If the first row of the matrix is not set to zero (only zeroes in the first row) - then a row-vector with zeroes is added by default.}

\item{slopes}{A numeric vector, which contains the slope parameters for each item - one parameter per item is expected.}

\item{theta_start}{A vector which contains a starting value for each person. If NULL is submitted, the starting values are set automatically. If a scalar is submitted, this start value is used for each person.}

\item{mu}{A numeric vector of location parameters for each person in case of MAP or EAP estimation. If nothing is submitted this is set to 0 for each person for MAP estimation.}

\item{sigma2}{A numeric vector of variance parameters for each person in case of MAP  or EAP estimation. If nothing is submitted this is set to 1 for each person for MAP estimation.}

\item{type}{Which maximization should be applied? There are five valid entries possible: "mle", "wle", "map", "eap" and "robust". To choose between the methods, or just to get a deeper understanding the papers mentioned below are quite helpful. The default is \code{"wle"} which is a good choice in many cases.}

\item{maxsteps}{The maximum number of steps the NR Algorithm will take. Default = 100.}

\item{exac}{How accurate are the estimates supposed to be? Default is 0.001.}

\item{H}{In case \code{type = "robust"} a Huber ability estimate is performed, and \code{H} modulates how fast the downweighting takes place (for more Details read Schuster & Yuan 2011).}

\item{ctrl}{more controls
\itemize{
\item \code{killdupli}: Should duplicated response pattern be removed for estimation (estimation is faster)? This is especially resonable in case of a large number of examinees and a small number of items.  Use this option with caution (for map and eap), because persons with different \code{mu} and \code{sigma2} will have different ability estimates despite they responded identically. Default value is \code{FALSE}.

\item \code{skipcheck}: Default = FALSE. If TRUE data matrix and arguments are not checked - this saves time e.g. when you use this function for simulations.
}}
}
\value{
The function returns a list with the estimation results and pretty much everything which has been submitted to fit the model. The estimation results can be found in \code{OBJ$resPP}. The core result is a number_of_persons x 2 matrix, which contains the ability estimate and the SE for each submitted person.
}
\description{
Compute person parameters for the GPCM and choose between five common estimation techniques: ML, WL, MAP, EAP and a robust estimation. All item parameters are treated as fixed.
}
\details{
Please note, that \code{robust} estimation with (Huber ability estimate) polytomous items is still experimental!


The probability choosing the k-th category is as follows:

\deqn{P(x_{ij} = k | \hat \alpha_i, \hat\beta_{iv}, \theta_j) = \frac{exp(\sum_{v=0}^{(k-1)}\hat \alpha_{i}(\theta_j - \hat \beta_{iv}))}{\,\sum_{c=0}^{m_i - 1}exp(\sum_{v=0}^{c}\hat \alpha_{i}(\theta_j - \hat \beta_{iv})))}}

In our case \eqn{\theta} is to be estimated. The item parameters are assumed as fixed (usually these are estimates of a former scaling procedure).

The model simplifies to the Partial Credit Model by setting \eqn{\alpha_{i} = 1, \forall i}.
}
\examples{
################# GPCM ###########################################################################

# some threshold parameters
THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
# slopes
sl     <- c(0.5,1,1.5,1.1,1,0.98)
awmatrix <- matrix(c(1,0,2,0,1,1,1,0,0,1
                     ,2,0,0,0,0,0,0,0,0,1,1,2,2,1,1,1,1,0,0,1),byrow=TRUE,nrow=5)

## GPCM model ##### 

# MLE
resgpcmlmle <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = sl,type = "mle")
# WLE
resgpcmwle <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = sl,type = "wle")
# MAP estimation
resgpcmmap <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = sl,type = "map")
# EAP estimation
resgpcmeap <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = sl,type = "eap")
# robust estimation
resgpcmrob <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = sl,type = "robust")

## PCM model ##### 

# MLE
respcmlmle <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = rep(1,ncol(THRES)),type = "mle")
# WLE
respcmwle <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = rep(1,ncol(THRES)),type = "wle")
# MAP estimation
respcmmap <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = rep(1,ncol(THRES)),
                     type = "map")
# EAP estimation
respcmeap <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = rep(1,ncol(THRES)),
                     type = "eap")

#### with different number of categories ##

THRES  <- matrix(c(-2,-1.23,1.11,3.48,1,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
THRES1 <- rbind(THRES,c(NA,NA,NA,NA,1.7,1))

awmatrix1 <- matrix(c(1,0,2,0,1,3,1,0,0,1,3,1,0,0
                      ,0,0,0,0,0,1,1,2,2,1,1,1,1,0,0,1), byrow=TRUE, nrow=5)

# MLE estimation
respcmlmle1 <- PP_gpcm(respm = awmatrix1,thres = THRES1, slopes = sl,type = "mle")
# WLE estimation
respcmwle1 <- PP_gpcm(respm = awmatrix1,thres = THRES1, slopes = sl,type = "wle")
# MAP estimation
respcmmap1 <- PP_gpcm(respm = awmatrix1,thres = THRES1, slopes = sl,type = "map")
# EAP estimation
respcmeap1 <- PP_gpcm(respm = awmatrix1, thres = THRES1, slopes = sl,type = "eap")
}
\references{
Baker, Frank B., and Kim, Seock-Ho (2004). Item Response Theory - Parameter Estimation Techniques. CRC-Press.

Masters, G. N. (1982). A Rasch model for partial credit scoring. Psychometrika, 47(2), 149-174.

Muraki, Eiji (1992). A Generalized Partial Credit Model: Application of an EM Algorithm. Applied Psychological Measurement, 16, 159-176.

Muraki, Eiji (1993). Information Functions of the Generalized Partial Credit Model. Applied Psychological Measurement, 17, 351-363.

Samejima, Fumiko (1993). The bias function of the maximum likelihood estimate of ability for the dichotomous response level. Psychometrika,  58, 195-209.

Samejima, Fumiko (1993). An approximation of the bias function of the maximum likelihood estimate of a latent variable for the general case where the item responses are discrete. Psychometrika,  58, 119-138.

Schuster, C., & Yuan, K. H. (2011). Robust estimation of latent ability in item response models. Journal of Educational and Behavioral Statistics, 36(6), 720-735.

Wang, S. and Wang, T. (2001). Precision of Warm's Weighted Likelihood Estimates for a Polytomous Model in Computerized Adaptive Testing. Applied Psychological Measurement, 25, 317-331.

Warm, Thomas A. (1989). Weighted Likelihood Estimation Of Ability In Item Response Theory. Psychometrika, 54, 427-450.
}
\seealso{
\link{PPass}, \link{PPall}, \link{PP_4pl}, \link{JKpp}, \link{PV}
}
\author{
Manuel Reif
}
\keyword{GPCM}
\keyword{Parameters}
\keyword{Person}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataRd.R
\name{pp_amt}
\alias{pp_amt}
\title{Adaptive Matrices Test data}
\format{
A list with two data.frames. The first data.frame 'daten_amt' contains 298 columns and 710 rows. Each row contains responses from on examinee. The second data.frame 'betas' contains the difficulty parameter (1PL) (These parameters came with the raw-score extraction.).
}
\source{
Division of Psychological Assessment and Applied Psychometrics, Fakulty of Psychology, University of Vienna
\itemize{
\item ID: id of person
\item AGE: age in years (with ages below 18 and above 34 are collapsed)
\item TE_GA:
\itemize{
\item TE: self-assessment. To pass a psychological assessment course, the students have to complete several hours self assessment on a bunch of tests, to get familiar with them.
\item GA: testing for an assessment report. The students also have to test other people (not psychologists nor psychology students) in order to write an assessment report.
}


\item FORM: There are several different versions of this test, which differ in test length, duration etc \ldots
\item TIME1: start time
\item TIME2: end time
\item REL: reliability for each person
\item i: items
}
}
\description{
This dataset contains real data from the 'Adaptive Matrices Test' (AMT), which is a computer-administered test.
This power test assesses logical reasoning as an indicator of general intelligence. The ability to identify regularities and draw logical conclusions is a very good predictor of long-term success at work. The dataset is sparse, because the test tailores a specific set of items for each examinee's ability level. (More information about adaptive testing: \url{https://en.wikipedia.org/wiki/Computerized_adaptive_testing})
}
\details{
The data are provided from the Unitersity of Vienna, Faculty of Psychology, Department of Psychological Assessment.
Thanks to Schuhfried.
}
\references{
\itemize{
\item Hornke, L. F., Etzel, S., & Rettig, K. (2003). Manual Adaptive Matrices Test (AMT). \emph{Moedling: SCHUHFRIED GmbH}.
}
}
\seealso{
\link{PPass}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PP.R
\docType{package}
\name{PP-package}
\alias{PP}
\alias{PP-package}
\title{PP: Estimation of Person Parameters and Person Fit}
\description{
The PP package provides functions for estimating person parameters for the
1PL, 2PL, 3PL, and 4PL models, as well as the generalized partial credit
model (GPCM). Supported estimation methods include maximum likelihood (ML),
weighted likelihood (WL; Warm, 1989), maximum a posteriori (MAP), expected
a posteriori (EAP), and robust estimation.
}
\details{
In addition, the package includes routines for person-fit analysis, including
infit, outfit, lz, and lzstar statistics. The implementation is designed for
efficient computation and includes compiled code for fast estimation.

For an introduction to the package workflow, see the package vignettes.
}
\examples{
set.seed(1522)

diffpar <- seq(-3, 3, length = 12)
sl <- round(runif(12, 0.5, 1.5), 2)
la <- round(runif(12, 0, 0.25), 2)
ua <- round(runif(12, 0.8, 1), 2)

awm <- matrix(sample(0:1, 10 * 12, replace = TRUE), ncol = 12)

res3plmle <- PP_4pl(
  respm = awm,
  thres = diffpar,
  slopes = sl,
  lowerA = la,
  type = "mle"
)

res3plwle <- PP_4pl(
  respm = awm,
  thres = diffpar,
  slopes = sl,
  lowerA = la,
  type = "wle"
)

res3plmap <- PP_4pl(
  respm = awm,
  thres = diffpar,
  slopes = sl,
  lowerA = la,
  type = "map"
)

res3plmlepfit <- Pfit(
  respm = awm,
  pp = res3plmle,
  fitindices = c("infit", "outfit")
)

out <- PPass(
  respdf = data.frame(awm),
  thres = diffpar,
  items = "all",
  mod = c("1PL"),
  fitindices = c("lz", "lzstar", "infit", "outfit")
)

}
\references{
Barton, M. A., & Lord, F. M. (1981). An upper asymptote for the
three-parameter logistic item-response model.

Birnbaum, A. (1968). Some latent trait models and their use in inferring an
examinee's ability. In F. M. Lord & M. R. Novick (Eds.), \emph{Statistical
theories of mental test scores}. Reading, MA: Addison-Wesley.

Drasgow, F., Levine, M. V., & Williams, E. A. (1985). Appropriateness
measurement with polychotomous item response models and standardized indices.
\emph{British Journal of Mathematical and Statistical Psychology},
\bold{38}(1), 67--86.

Muraki, E. (1992). A generalized partial credit model: Application of an EM
algorithm. \emph{Applied Psychological Measurement}, \bold{16}, 159--176.

Samejima, F. (1993). An approximation of the bias function of the maximum
likelihood estimate of a latent variable for the general case where the item
responses are discrete. \emph{Psychometrika}, \bold{58}, 119--138.

Snijders, T. B. (2001). Asymptotic null distribution of person fit statistics
with estimated person parameter. \emph{Psychometrika}, \bold{66}(3), 331--342.

Warm, T. A. (1989). Weighted likelihood estimation of ability in item
response theory. \emph{Psychometrika}, \bold{54}, 427--450.

Wright, B. D., & Masters, G. N. (1990). Computation of OUTFIT and INFIT
statistics. \emph{Rasch Measurement Transactions}, \bold{3}(4), 84--85.

Yen, Y.-C., Ho, R.-G., Liao, W.-W., Chen, L.-J., & Kuo, C.-C. (2012). An
empirical evaluation of the slip correction in the four parameter logistic
models with computerized adaptive testing. \emph{Applied Psychological
Measurement}, \bold{36}, 75--87.
}
\seealso{
[PPass()], [PP_gpcm()], [PP_4pl()], [PPall()], [Pfit()]
}
\author{
Jan Steinfeld and Manuel Reif
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_gpcm.R
\name{sim_gpcm}
\alias{sim_gpcm}
\title{Simulate data for the gpcm model}
\usage{
sim_gpcm(thres, alpha, theta)
}
\arguments{
\item{thres}{An numeric matrix which contains threshold parameters for each item. The first row must contain zeroes only!}

\item{alpha}{A numeric vector, which contains the slope parameters - one parameter per item is expected.}

\item{theta}{A numeric vector which contains person parameters.}
}
\description{
This function returns an integer matrix of simulated responses under given item and person parameters.
}
\examples{
################# simulate GPCM ###############################################

set.seed(1750)

THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
# slopes
sl     <- c(0.5,1,1.5,1.1,1,0.98)

THRESx <- rbind(0,THRES)

THETA  <- rnorm(100)

simdat_gpcm <- sim_gpcm(thres = THRESx,alpha = sl,theta = THETA)

head(simdat_gpcm)


### simulate with a different number of categories


THRES1 <- rbind(THRESx,c(NA,NA,NA,NA,1.7,1))
THRES1 # last 2 items have +1 category

simdat_gpcm2 <- sim_gpcm(thres = THRES1,alpha = sl,theta = THETA)

head(simdat_gpcm2)

# check the maximum category
apply(simdat_gpcm2,2,max)


}
\seealso{
\link{sim_4pl}, \link{PP_gpcm}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{sim_4pl}
\alias{sim_4pl}
\title{Simulate data for 1/2/3/4-pl model}
\usage{
sim_4pl(beta, alpha, lowerA, upperA, theta)
}
\arguments{
\item{beta}{A numeric vector which contains the difficulty parameters for each item.}

\item{alpha}{A numeric vector, which contains the slope parameters for each item.}

\item{lowerA}{A numeric vector, which contains the lower asymptote parameters (kind of guessing parameter) for each item.}

\item{upperA}{numeric vector, which contains the upper asymptote parameters for each item.}

\item{theta}{A numeric vector which contains person parameters.}
}
\description{
This function returns a dichotomous matrix of simulated responses under given item and person parameters.
}
\examples{
################# simulate 4PL ###############################################

set.seed(1700)
 
# simulate 1-PL model ----------

thetas <- c(0.231,-1.313,1.772,1.601,1.733,-2.001,0.443,3.111,-4.156)
sl <- c(1,1.1,0.9,0.89,1.5,1.1,1)
la <- c(0,0,0.2,0.15,0.04,0,0.21)
ua <- c(0.9,0.98,0.97,1,1,1,0.97)

simdat1pl <- sim_4pl(beta=seq(-4,4,length.out=7), alpha=rep(1,7),
                  lowerA=rep(0,7), upperA=rep(1,7), theta=thetas)


head(simdat1pl)


# simulate 2-PL model ----------

simdat2pl <- sim_4pl(beta=seq(-4,4,length.out=7), alpha=sl,
                  lowerA=rep(0,7), upperA=rep(1,7), theta=thetas)


head(simdat2pl)


# simulate 3-PL model ----------

simdat3pl <- sim_4pl(beta=seq(-4,4,length.out=7), alpha=sl,
                     lowerA=la, upperA=rep(1,7), theta=thetas)


head(simdat3pl)


# simulate 4-PL model ----------

simdat4pl <- sim_4pl(beta=seq(-4,4,length.out=7), alpha=sl,
                     lowerA=la, upperA=ua, theta=thetas)


head(simdat4pl)

}
\seealso{
\link{sim_gpcm}, \link{PP_4pl}
}
\author{
Manuel Reif
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Pfit.R
\name{Pfit}
\alias{Pfit}
\alias{Pfit.gpcm}
\title{Person-Fit statistics}
\usage{
Pfit(respm, pp, fitindices, SE = FALSE)

\method{Pfit}{gpcm}(respm, pp, fitindices = c("infit", "outfit"), SE = FALSE)
}
\arguments{
\item{respm}{numeric response matrix}

\item{pp}{object of the class fourpl with estimated person parameter}

\item{fitindices}{character vector of desired person fit statistics c("lz","lzstar","infit","outfit")}

\item{SE}{logical: if true standard errors are computed using jackknife method}
}
\value{
list of person-fits for each person-fit statistic

\itemize{
\item the list of person-fits contains the calculated person-fit (like lz, lzstar) and also additional information like p-value or standard error if desired.
\item the additional information is provided after the short form of the personfit
\item lz (lz)
\item lzstar (lzs)
\item infit the mean-square statistic (in)
\item outfit the mean-square statistic (ou)
\item _unst: unstandardised
\item _se: standard error
\item _t: t-value
\item _chisq: $chi^2$-value
\item _df: defrees of freedom
\item _pv: p-value
}
}
\description{
Compute several person fit statistic for the 1-PL, 2-PL, 3-PL, 4-PL and PCM.
}
\details{
Please note that currently only the likelihood based LZ-Index (Drasgow, Levine, and Williams, 1985) and LZ*-Index (Snijders, 2001) are implemented. Also the INFIT-OUTIFT (Wright and Masters, 1982, 1990) statistic as well as the polytomouse version of INFIT-OUTFIT are supported. Other person fit statistics will be added soon.

The calculation of the person fit statistics requires the numeric response-matrix as well as an object of the fourpl-class. So first you should estimate the person parameter and afterwards calculate the person fit statistics. You could also use our PPass-function to estimate the person parameter and calculate the desired person fit simultaneously.
It is possible to calculate several person fit statistics at once, you only have to specify them in a vector.

For the Partial Credit model we currently support the infit-outfit statistic. Please submit also the numeric response-matrix as well as the estimated person parameter with an gpcm-class.
}
\examples{
################# Pfit ###################################################


### data creation ##########

set.seed(1337)


# intercepts
diffpar <- seq(-3,3,length=15)
# slope parameters
sl     <- round(runif(15,0.5,1.5),2)
la     <- round(runif(15,0,0.25),2)
ua     <- round(runif(15,0.8,1),2)

# response matrix
awm <- matrix(sample(0:1,100*15,replace=TRUE),ncol=15)

# ------------------------------------------------------------------------
## 1PL model ##### 
# ------------------------------------------------------------------------
# MLE
res1plmle <- PP_4pl(respm = awm,thres = diffpar,type = "mle")
# WLE
res1plwle <- PP_4pl(respm = awm,thres = diffpar,type = "wle")
# MAP estimation
res1plmap <- PP_4pl(respm = awm,thres = diffpar,type = "map")
# ------------------------------------------------------------------------
## LZ*-Index ##### 
Pfit(respm=awm,pp=res1plwle,fitindices="lzstar")
Pfit(respm=awm,pp=res1plmle,fitindices="lzstar")
Pfit(respm=awm,pp=res1plmap,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit ##### 
Pfit(respm=awm,pp=res1plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------

##########################################################################

# ------------------------------------------------------------------------
## 2PL model ##### 
# ------------------------------------------------------------------------
# MLE
res2plmle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "mle")
# WLE
res2plwle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "wle")
# ------------------------------------------------------------------------
## LZ*-Index ##### 
Pfit(respm=awm,pp=res2plwle,fitindices="lzstar")
Pfit(respm=awm,pp=res2plmle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit ##### 
Pfit(respm=awm,pp=res2plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------

##########################################################################

# ------------------------------------------------------------------------
## 3PL model ##### 
# ------------------------------------------------------------------------
# MLE
res3plmle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,type = "mle")
# WLE
res3plwle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,type = "wle")
# ------------------------------------------------------------------------
## LZ*-Index ##### 
Pfit(respm=awm,pp=res3plwle,fitindices="lzstar")
Pfit(respm=awm,pp=res3plmle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit ##### 
Pfit(respm=awm,pp=res3plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------

##########################################################################

# ------------------------------------------------------------------------
## 4PL model ##### 
# ------------------------------------------------------------------------
# MLE
res4plmle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "mle")
# WLE
res4plwle <- PP_4pl(respm = awm,thres = diffpar,
                    slopes = sl,lowerA = la,upperA=ua,type = "wle")
# ------------------------------------------------------------------------
## LZ*-Index ##### 
Pfit(respm=awm,pp=res4plwle,fitindices="lzstar")
Pfit(respm=awm,pp=res4plmle,fitindices="lzstar")
## LZ*-Index combined with Infit-Outfit ##### 
Pfit(respm=awm,pp=res4plwle,fitindices=c("lzstar","infit","outfit"))
# ------------------------------------------------------------------------

##########################################################################
}
\references{
Armstrong, R. D., Stoumbos, Z. G., Kung, M. T. & Shi, M. (2007). On the performance of the lz person-fit statistic.  \emph{Practical Assessment, Research & Evaluation}, \bold{12(16)}. Chicago	

De La Torre, J., & Deng, W. (2008). Improving Person-Fit Assessment by Correcting the Ability Estimate and Its Reference Distribution. Journal of Educational Measurement, \bold{45(2)}, 159-177.

Drasgow, F., Levine, M. V. & Williams, E. A. (1985) Appropriateness measurement with polychotomous item response models and standardized indices. \emph{British Journal of Mathematical and Statistical Psychology}, \bold{38(1)}, 67--86.

Efron, B., & Stein, C. (1981). The jackknife estimate of variance. \emph{The Annals of Statistics}, \bold{9(3)}, 586-596.	

Karabatsos, G. (2003) Comparing the Aberrant Response Detection Performance of Thirty-Six Person-Fit Statistics. \emph{Applied Measurement In Education}, \bold{16(4)}, 277--298.

Magis, D., Raiche, G. & Beland, S. (2012) A didactic presentation of Snijders's l[sub]z[/sub] index of person fit with emphasis on response model selection and ability estimation. \emph{Journal of Educational and Behavioral Statistics}, \bold{37(1)}, 57--81.

Meijer, R. R. & Sijtsma, K. (2001) Methodology review: Evaluating person fit. \emph{Applied Psychological Measurement}, \bold{25(2)}, 107--135.

Molenaar, I. W. & Hoijtink, H. (1990) The many null distributions of person fit indices. \emph{Psychometrika}, \bold{55(1)}, 75--106. 

Mousavi, A. & Cui, Y. Evaluate the performance of and of person fit: A simulation study.

Reise, S. P. (1990). A comparison of item-and person-fit methods of assessing model-data fit in IRT.  \emph{Applied Psychological Measurement}, \bold{14(2)}, 127-137.

Snijders, T. B. (2001) Asymptotic null distribution of person fit statistics with estimated person parameter. \emph{Psychometrika}, \bold{66(3)}, 331--342. 

Wright, B. D. & Masters, G. N. (1990). Computation of OUTFIT and INFIT Statistics.  \emph{Rasch Measurement Transactions}, 3:4, 84-85.

Wright, B. D., & Masters, G. N. (1982). \emph{Rating Scale Analysis. Rasch Measurement.} MESA Press, 5835 S. Kimbark Avenue, Chicago, IL 60637.
}
\seealso{
\link{PPall}, \link{PP_4pl}, \link{PPass}
}
\author{
Jan Steinfeld
}
\keyword{Infit-Outfit}
\keyword{LZ-Index}
\keyword{Person}
\keyword{fit}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PPall.R, R/prints.R
\name{PPall}
\alias{PPall}
\alias{print.ppeo}
\alias{summary.ppeo}
\title{Estimate Person Parameters}
\usage{
PPall(
  respm,
  thres,
  slopes,
  lowerA,
  upperA,
  theta_start = NULL,
  mu = NULL,
  sigma2 = NULL,
  type = "wle",
  model2est,
  maxsteps = 100,
  exac = 0.001,
  H = 1,
  ctrl = list()
)

\method{print}{ppeo}(x, ...)

\method{summary}{ppeo}(object, nrowmax = 15, ...)
}
\arguments{
\item{respm}{An integer matrix, which contains the examinees responses. A persons x items matrix is expected.}

\item{thres}{A numeric matrix which contains the threshold parameter for each item. If the first row of the matrix is not set to zero (only zeroes in the first row) - then a row-vector with zeroes is added by default.}

\item{slopes}{A numeric vector, which contains the slope parameters for each item - one parameter per item is expected.}

\item{lowerA}{A numeric vector, which contains the lower asymptote parameters (kind of guessing parameter) for each item. In the case of polytomous items, the value must be 0.}

\item{upperA}{numeric vector, which contains the upper asymptote parameters for each item. In the case of polytomous items, the value must be 1.}

\item{theta_start}{A vector which contains a starting value for each person. If NULL is submitted, the starting values are set automatically. If a scalar is submitted, this start value is used for each person.}

\item{mu}{A numeric vector of location parameters for each person in case of MAP estimation. If nothing is submitted this is set to 0 for each person for MAP estimation.}

\item{sigma2}{A numeric vector of variance parameters for each person in case of MAP or EAP estimation. If nothing is submitted this is set to 1 for each person for MAP estimation.}

\item{type}{Which maximization should be applied? There are five valid entries possible: "mle", "wle", "map", "eap" and "robust". To choose between the methods, or just to get a deeper understanding the papers mentioned below are quite helpful. The default is \code{"wle"} which is a good choice in many cases.}

\item{model2est}{A character vector with length equal to the number of submitted items. It defines itemwise the response model under which the item parameter was estimated. There are 2 valid inputs up to now: \code{"GPCM"} and \code{"4PL"}.}

\item{maxsteps}{The maximum number of steps the NR algorithm will take. Default = 100.}

\item{exac}{How accurate are the estimates supposed to be? Default is 0.001.}

\item{H}{In case \code{type = "robust"} a Huber ability estimate is performed, and \code{H} modulates how fast the downweighting takes place (for more Details read Schuster & Yuan 2011).}

\item{ctrl}{More controls:

\itemize{
\item \code{killdupli}: Should duplicated response pattern be removed for estimation (estimation is faster)? This is especially resonable in case of a large number of examinees and a small number of items.  Use this option with caution (for map and eap), because persons with different \code{mu} and \code{sigma2} will have different ability estimates despite they responded identically. Default value is \code{FALSE}.

\item \code{skipcheck}: Default = FALSE. If TRUE data matrix and arguments are not checked - this saves time e.g. when you use this function for simulations.

}}

\item{x}{an object of class \code{gpcm4pl} which is the result of using the \code{PPall()} function}

\item{...}{just some points.}

\item{object}{An object of class \code{gpcm4pl} which is the result of using the \code{PPall()} function}

\item{nrowmax}{When printing the matrix of estimates - how many rows should be shown? Default = 15.}
}
\value{
The function returns a list with the estimation results and pretty much everything which has been submitted to fit the model. The estimation results can be found in \code{OBJ$resPP}. The core result is a number_of_persons x 2 matrix, which contains the ability estimate and the SE for each submitted person.
}
\description{
Compute person parameters for the 1,2,3,4-PL model and for the GPCM. Choose between ML, WL, MAP, EAP and robust estimation. Use this function if 4-PL items and GPCM items are mixed for each person.
}
\details{
For a test with both: dichotomous and polytomous items which have been scaled under 1/2/3/4-PL model or the GPCM, use this function to estimate the person ability parameters. You have to define the appropriate model for each item. 

Please note, that  \code{robust} estimation with (Huber ability estimate) polytomous items is still experimental!
}
\examples{
################# GPCM  and 4PL mixed #########################################

# some threshold parameters
THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
# slopes
sl     <- c(0.5,1,1.5,1.1,1,0.98)

THRESx <- THRES
THRESx[2,1:3] <- NA

# for the 4PL item the estimated parameters are submitted, 
# for the GPCM items the lower asymptote = 0 
# and the upper asymptote = 1.
la     <- c(0.02,0.1,0,0,0,0)
ua     <- c(0.97,0.91,1,1,1,1)

awmatrix <- matrix(c(1,0,1,0,1,1,1,0,0,1
                     ,2,0,0,0,0,0,0,0,0,1
                     ,1,2,2,1,1,1,1,0,0,1),byrow=TRUE,nrow=5)

# create model2est
# this function tries to help finding the appropriate 
# model by inspecting the THRESx.
model2est <- findmodel(THRESx)


# MLE
respmixed_mle <- PPall(respm = awmatrix,thres = THRESx, 
                     slopes = sl,lowerA = la, upperA=ua,type = "mle",
                     model2est=model2est)
# WLE
respmixed_wle <- PPall(respm = awmatrix,thres = THRESx, 
                    slopes = sl,lowerA = la, upperA=ua,type = "wle",
                    model2est=model2est)
# MAP estimation
respmixed_map <- PPall(respm = awmatrix,thres = THRESx, 
                    slopes = sl,lowerA = la, upperA=ua, type = "map",
                    model2est=model2est)

# EAP estimation
respmixed_eap <- PPall(respm = awmatrix,thres = THRESx, 
                    slopes = sl,lowerA = la, upperA=ua, type = "eap",
                    model2est=model2est)

# Robust estimation
respmixed_rob <- PPall(respm = awmatrix,thres = THRESx, 
                    slopes = sl,lowerA = la, upperA=ua, type = "robust",
                    model2est=model2est)


# summary to summarize the results
summary(respmixed_mle)
summary(respmixed_wle)
summary(respmixed_map)
summary(respmixed_eap)
summary(respmixed_rob)

}
\references{
Baker, Frank B., and Kim, Seock-Ho (2004). Item Response Theory - Parameter Estimation Techniques. CRC-Press.

Barton, M. A., & Lord, F. M. (1981). An Upper Asymptote for the Three-Parameter Logistic Item-Response Model.

Magis, D. (2013). A note on the item information function of the four-parameter logistic model. Applied Psychological Measurement, 37(4), 304-315.

Muraki, Eiji (1992). A Generalized Partial Credit Model: Application of an EM Algorithm. Applied Psychological Measurement, 16, 159-176.

Muraki, Eiji (1993). Information Functions of the Generalized Partial Credit Model. Applied Psychological Measurement, 17, 351-363.

Samejima, Fumiko (1993). The bias function of the maximum likelihood estimate of ability for the dichotomous response level. Psychometrika,  58, 195-209.

Samejima, Fumiko (1993). An approximation of the bias function of the maximum likelihood estimate of a latent variable for the general case where the item responses are discrete. Psychometrika,  58, 119-138.

Schuster, C., & Yuan, K. H. (2011). Robust estimation of latent ability in item response models. Journal of Educational and Behavioral Statistics, 36(6), 720-735.

Wang, S. and Wang, T. (2001). Precision of Warm's Weighted Likelihood Estimates for a Polytomous Model in Computerized Adaptive Testing. Applied Psychological Measurement, 25, 317-331.

Warm, Thomas A. (1989). Weighted Likelihood Estimation Of Ability In Item Response Theory. Psychometrika, 54, 427-450.

Yen, Y.-C., Ho, R.-G., Liao, W.-W., Chen, L.-J., & Kuo, C.-C. (2012). An empirical evaluation of the slip correction in the four parameter logistic models with computerized adaptive testing. Applied Psychological Measurement, 36, 75-87.
}
\seealso{
\link{PPass}, \link{PP_gpcm}, \link{PP_4pl}, \link{JKpp}, \link{PV}
}
\author{
Manuel Reif
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PV.R, R/prints.R
\name{PV}
\alias{PV}
\alias{PV.fourpl}
\alias{PV.gpcm}
\alias{PV.gpcm4pl}
\alias{print.pv}
\alias{summary.pv}
\title{Draw Plausible values}
\usage{
PV(estobj, ...)

\method{PV}{fourpl}(estobj, npv = 10, approx = TRUE, thinning = 6, burnin = 10, mult = 2, ...)

\method{PV}{gpcm}(estobj, npv = 10, approx = TRUE, thinning = 6, burnin = 10, mult = 2, ...)

\method{PV}{gpcm4pl}(estobj, npv = 10, approx = TRUE, thinning = 6, burnin = 10, mult = 2, ...)

\method{print}{pv}(x, ...)

\method{summary}{pv}(object, nrowmax = 15, ...)
}
\arguments{
\item{estobj}{An object which originates from using \code{PP_4pl()}, \code{PP_gpcm()} or \code{PPall()}. EAP estimation is strongly recommended (\code{type = "eap"}), when plausible values are drawn afterwards, because the EAP estimate is used as starting point for the MH algorithm.}

\item{...}{More arguments}

\item{npv}{The number of (effectively returned) plausible values - default is 10.}

\item{approx}{Whether a normal approximation \code{N(mu,sigma2)} is used to draw the plausible values. Default = TRUE. If FALSE a Metropolitan-Hastings-Algorithm will draw the values.}

\item{thinning}{A numeric vector of length = 1. If approx = FALSE, a Metropolitan-Hastings-Algorithm draws the plausible values. To avoid autocorrelation, thinning takes every \bold{kth} value as effective plausible value. The default is 6 (every 6th value is taken), which works appropriately in almost all cases here (with default settings).}

\item{burnin}{How many draws should be discarded at the chains beginning? Default is 10 - and this seems reasonable high (probably 5 will be enough as well), because starting point is the EAP.}

\item{mult}{Multiplication constant (default = 2). Use this parameter to vary the width of the proposal distribution - which is \code{N(theta_v,mult*SE_eap)} - when a MH-Algorithm is applied. So the constant quantifies the width in terms of multiples of the EAP standard error. 2 works fine with the default thinning. If the supplied value is large, thinning can take lower values without causing autocorrelation.}

\item{x}{An object of class \code{pv} which is the result of using the \code{PV()} function}

\item{object}{An object of class \code{pv} which is the result of using the \code{PV()} function}

\item{nrowmax}{When printing the matrix of estimates - how many rows should be shown? Default = 15.}
}
\value{
The function returns a list which main element is \code{pvdraws}. This is a matrix of size number_of_persons x npv - so if 10 plausible values are requested for 100 persons, a 100x10 matrix is returned.
}
\description{
This function draws \code{npv} plausible values for each person from their posterior density.
}
\examples{
################# Plausible values #############################################################


### 4 PL model ######


### data creation ##########

set.seed(1522)
# intercepts
diffpar <- seq(-3,3,length=12)
# slope parameters
sl     <- round(runif(12,0.5,1.5),2)
la     <- round(runif(12,0,0.25),2)
ua     <- round(runif(12,0.8,1),2)

# response matrix
awm <- matrix(sample(0:1,10*12,replace=TRUE),ncol=12)


# EAP estimation - 2pl model
res2pleap <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "eap")

# draw 10 plausible values
res_pv  <- PV(res2pleap)
summary(res_pv)

# draw 10 plausible values - use a metropolitan hastings algorithm
res_pv2  <- PV(res2pleap,approx = FALSE)
summary(res_pv2)

# ------ check the PVs


# -- autocorrelation?
autocor <- function(acv)
  {
  cor(acv[-1],acv[-length(acv)]) 
  }


res_pvac  <- PV(res2pleap,approx = FALSE,npv = 200)

# independent draws - so there cannot be any systematic autocorrelation when
# approx = TRUE. So this acts as a kind of benchmark for the MH-Alg.
res_pvac2  <- PV(res2pleap,approx = TRUE,npv = 200)

apply(res_pvac$pvdraws,1,autocor)
apply(res_pvac2$pvdraws,1,autocor)

# -- autocorrelation distr?


apply(res_pvac$pvdraws,1,quantile)
apply(res_pvac2$pvdraws,1, quantile)




### GPCM model ######


# some threshold parameters
THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
# slopes
sl     <- c(0.5,1,1.5,1.1,1,0.98)
awmatrix <- matrix(c(1,0,2,0,1,1,1,0,0,1
                     ,2,0,0,0,0,0,0,0,0,1,1,2,2,1,1,1,1,0,0,1),byrow=TRUE,nrow=5)


# EAP estimation
resgpcmeap <- PP_gpcm(respm = awmatrix,thres = THRES, slopes = sl,type = "eap")


res_gpcmpv  <- PV(resgpcmeap,approx = FALSE,npv = 20)




### GPCM and 4PL model ######


# some threshold parameters
THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
# slopes
sl     <- c(0.5,1,1.5,1.1,1,0.98)

THRESx <- THRES
THRESx[2,1:3] <- NA

# for the 4PL item the estimated parameters are submitted, 
# for the GPCM items the lower asymptote = 0 
# and the upper asymptote = 1.
la     <- c(0.02,0.1,0,0,0,0)
ua     <- c(0.97,0.91,1,1,1,1)

awmatrix <- matrix(c(1,0,1,0,1,1,1,0,0,1
                     ,2,0,0,0,0,0,0,0,0,1
                     ,1,2,2,1,1,1,1,0,0,1),byrow=TRUE,nrow=5)

model2est <- findmodel(THRESx)


# EAP estimation
respcmeap1 <- PPall(respm = awmatrix,thres = THRESx, 
                    slopes = sl,lowerA = la, upperA=ua, type = "eap",
                    model2est=model2est)


res_mixedpv_1  <- PV(respcmeap1,approx = FALSE,npv = 200)

# rowMeans of plausible values should approximate the EAPs
rowMeans(res_mixedpv_1$pvdraws)
# EAPs
respcmeap1

# show the quantiles of the empirical distribution
apply(res_mixedpv_1$pvdraws,1,quantile)

}
\references{
Mislevy, R. J. (1991). Randomization-based inference about latent variables from complex samples. Psychometrika, 56(2), 177-196.

Von Davier, M., Gonzalez, E., & Mislevy, R. (2009). What are plausible values and why are they useful. IERI monograph series, 2, 9-36.

Kruschke, J. (2010). Doing Bayesian data analysis: A tutorial introduction with R. Academic Press.
}
\seealso{
\link{PP_gpcm}, \link{PP_4pl}, \link{JKpp}
}
\author{
Manuel Reif
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JKpp.R, R/prints.R
\name{JKpp}
\alias{JKpp}
\alias{JKpp.fourpl}
\alias{JKpp.gpcm}
\alias{JKpp.gpcm4pl}
\alias{print.jk}
\alias{summary.jk}
\title{Run a jackknife}
\usage{
JKpp(estobj, ...)

\method{JKpp}{fourpl}(
  estobj,
  cmeth = "mean",
  maxsteps = 500,
  exac = 0.001,
  fullmat = FALSE,
  ctrl = list(),
  ...
)

\method{JKpp}{gpcm}(
  estobj,
  cmeth = "mean",
  maxsteps = 500,
  exac = 0.001,
  fullmat = FALSE,
  ctrl = list(),
  ...
)

\method{JKpp}{gpcm4pl}(
  estobj,
  cmeth = "mean",
  maxsteps = 500,
  exac = 0.001,
  fullmat = FALSE,
  ctrl = list(),
  ...
)

\method{print}{jk}(x, ...)

\method{summary}{jk}(object, nrowmax = 15, ...)
}
\arguments{
\item{estobj}{An object which originates from using \code{PP_gpcm()}, \code{PP_4pl()} or \code{PPall()}.}

\item{...}{More input.}

\item{cmeth}{Choose the centering method, to summarize the n jackknife results to one single ability estimate. There are three valid entries: "mean", "median" and "AMT" (see Details for further description).}

\item{maxsteps}{The maximum number of steps the NR Algorithm will take.}

\item{exac}{How accurate are the estimates supposed to be? Default is 0.001.}

\item{fullmat}{Default = FALSE. If TRUE, the function returns the whole jackknife matrix, which is the basis for the jackknife estimator.}

\item{ctrl}{More controls}

\item{x}{an object of class \code{jk} which is the result of using the \code{JKpp()} function}

\item{object}{An object of class \code{jk} which is the result of using the \code{JKpp()} function}

\item{nrowmax}{When printing the matrix of estimates - how many rows should be shown? Default = 15.}
}
\description{
This function uses a jackknife approach to compute person parameters. The jackknife ability measure is based on primarily estimated models (\code{PP_4pl()}, \code{PP_gpcm()} or \code{PPall()}) - so the function is applied on the estimation objects, and jackknifed ability measures are returned.
}
\details{
Please use the Jackknife \bold{Standard-Error} output with \bold{caution}! It is implemented as suggested in Wainer and Wright (1980), but the results seem a bit strange, because the JK-SE is supposed to overestimate the SE compared to the MLE-SE. Actually, in all examples an underestimation of the SE was observed compared to the MLE/WLE-SE! 


\bold{AMT-robustified jackknife:} When choosing \code{cmeth = AMT}, the jackknife ability subsample estimates and the original supplied ability estimate are combined to a single jackknife-ability value by the Sine M-estimator. The AMT (or Sine M-estimator) is one of the winners in the Princeton Robustness Study of 1972. To get a better idea how the estimation process works, take a closer look to the paper which is mentioned below (Wainer & Wright, 1980).
}
\examples{
################# Jackknife ###################################################

### 4 PL model ######

### data creation ##########

set.seed(1623)
# intercepts
diffpar <- seq(-3,3,length=12)
# slope parameters
sl     <- round(runif(12,0.5,1.5),2)
la     <- round(runif(12,0,0.25),2)
ua     <- round(runif(12,0.8,1),2)

# response matrix
abpar <- rnorm(10,0,1.7)
awm <- sim_4pl(beta = diffpar,alpha = sl,lowerA = la,upperA=ua,theta = abpar)

## 1PL model ##### 

# MLE
res1plmle <- PP_4pl(respm = awm,thres = diffpar,type = "mle")
# WLE
res1plwle <- PP_4pl(respm = awm,thres = diffpar,type = "wle")
# MAP estimation
res1plmap <- PP_4pl(respm = awm,thres = diffpar,type = "map")
# EAP estimation
res1pleap <- PP_4pl(respm = awm,thres = diffpar,type = "eap")
# robust estimation
res1plrob <- PP_4pl(respm = awm,thres = diffpar,type = "robust")

## centering method = mean
res_jk1 <- JKpp(res1plmle)
res_jk2 <- JKpp(res1plwle)
res_jk3 <- JKpp(res1plmap)
res_jk4 <- JKpp(res1plrob)
res_jk5 <- JKpp(res1pleap)

summary(res_jk1)

## centering method = median
res_jk1a <- JKpp(res1plmle,cmeth = "median")
res_jk2a <- JKpp(res1plwle,cmeth = "median")
res_jk3a <- JKpp(res1plmap,cmeth = "median")

summary(res_jk2a)

## centering method = AMT
res_jk1b <- JKpp(res1plmle,cmeth = "AMT")
res_jk2b <- JKpp(res1plwle,cmeth = "AMT")
res_jk3b <- JKpp(res1plmap,cmeth = "AMT")

summary(res_jk3b)


## 2PL model ##### 

# MLE
res2plmle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "mle")
# WLE
res2plwle <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "wle")
# MAP estimation
res2plmap <- PP_4pl(respm = awm,thres = diffpar, slopes = sl,type = "map")
# EAP estimation
res2pleap <- PP_4pl(respm = awm,thres = diffpar,slopes = sl,type = "eap")
# robust estimation
res2plrob <- PP_4pl(respm = awm,thres = diffpar,slopes = sl,type = "robust")

res_jk6 <- JKpp(res2plmle)
res_jk7 <- JKpp(res2plwle)
res_jk8 <- JKpp(res2plmap)
res_jk9 <- JKpp(res2pleap)
res_jk10 <- JKpp(res2plrob)

### GPCM model ######

# some threshold parameters
THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
# slopes
sl     <- c(0.5,1,1.5,1.1,1,0.98)
awmatrix <- matrix(c(1,0,2,0,1,1,1,0,0,1,2,0,0,0,0,0,0,0,0,1,
                     1,2,2,1,1,1,1,0,0,1),byrow=TRUE,nrow=5)


### PCM model ######

# MLE
respcmlmle <- PP_gpcm(respm = awmatrix,thres = THRES,
                      slopes = rep(1,ncol(THRES)),type = "mle")
# WLE
respcmwle <- PP_gpcm(respm = awmatrix,thres = THRES,
                     slopes = rep(1,ncol(THRES)),type = "wle")
# MAP estimation
respcmmap <- PP_gpcm(respm = awmatrix,thres = THRES,
                     slopes = rep(1,ncol(THRES)),type = "map")


res_jk11 <- JKpp(respcmlmle)
res_jk12 <- JKpp(respcmwle)
res_jk13 <- JKpp(respcmmap)

### GPCM/4-PL mixed model ######

THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)

sl     <- c(0.5,1,1.5,1.1,1,0.98)

THRESx <- THRES
THRESx[2,1:3] <- NA

# for the 4PL item the estimated parameters are submitted, 
# for the GPCM items the lower asymptote = 0 
# and the upper asymptote = 1.
la     <- c(0.02,0.1,0,0,0,0)
ua     <- c(0.97,0.91,1,1,1,1)

awmatrix <- matrix(c(1,0,1,0,1,1,1,0,0,1
                     ,2,0,0,0,1,0,0,0,0,1
                     ,0,2,2,1,1,1,1,0,0,1),byrow=TRUE,nrow=5)

# create model2est
# this function tries to help finding the appropriate 
# model by inspecting the THRESx.
model2est <- findmodel(THRESx)


# MLE estimation
respmixed_mle <- PPall(respm = awmatrix,
                       thres = THRESx, 
                       slopes = sl,
                       lowerA = la,
                       upperA=ua,
                       type = "mle",
                       model2est=model2est)
# WLE estimation
respmixed_wle <- PPall(respm = awmatrix,
                       thres = THRESx, 
                       slopes = sl,
                       lowerA = la,
                       upperA=ua,
                       type = "wle",
                       model2est=model2est)


res_jk114 <- JKpp(respmixed_mle)
res_jk115 <- JKpp(respmixed_wle)
}
\references{
Wainer, H., & Wright, B. D. (1980). Robust estimation of ability in the Rasch model. Psychometrika, 45(3), 373-391.
}
\seealso{
\link{PP_gpcm}, \link{PP_4pl}, \link{PPall}
}
\author{
Manuel Reif
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lzstar_mixed.R
\name{print_personfit_summary}
\alias{print_personfit_summary}
\title{Print summary of person-fit results}
\usage{
print_personfit_summary(results, alpha = 0.05)
}
\arguments{
\item{results}{Matrix from lzstar_mixed.}

\item{alpha}{Significance level for flagging.}
}
\description{
Print summary of person-fit results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PPass.R
\name{PPass}
\alias{PPass}
\alias{PPass.default}
\alias{PPass.Rm}
\title{Person Assessment function}
\usage{
PPass(...)

\method{PPass}{default}(
  respdf,
  items = "all",
  mod = c("1PL", "2PL", "3PL", "4PL", "PCM", "GPCM", "MIXED"),
  fitindices = c("lz", "lzstar", "infit", "outfit"),
  ...
)

\method{PPass}{Rm}(RMobj, fitindices = c("lz", "lzstar", "infit", "outfit"), ...)
}
\arguments{
\item{respdf}{A data.frame which contains the items, and perhaps other informations. Each row is a person related resonse patter. Each column denotes a variable.}

\item{items}{A numeric (integer) vector which indicates the positions of the items in the data.frame (\code{respdf}). If \code{items} = 'all', \bold{all columns} are treated as items.}

\item{mod}{Choose your data generating model. This argument switches between the three person parameter estimating functions \code{PP_4pl}, \code{PP_gpcm} and \code{PPall}.}

\item{fitindices}{A character vector which denotes the fit indices to compute.}

\item{RMobj}{A fitted Rasch Model (\code{RM()}) object which stems from the \code{eRm} package.}

\item{\ldots}{Submit arguments to the underlying functions: \code{PP_4pl}, \code{PP_gpcm} and \code{PPall} (see documentation files) for person parameter estimation.}
}
\value{
The original data.frame and

\itemize{

\item The Person Parameter estimates incl. Standard Errors (2 columns)

\item Person Fit Indices you chose (1 or more)

}
}
\description{
Estimate Person Paramters and calculate Person Fit in one step to gain resonse pattern assessment. Submit a data.frame which contains item responses, or an fitted model (Rasch Model and Partial Credit Model are supported) of the \code{eRm} package.
}
\details{
PPass fuses Person Parameter estimation and Person Fit computation into a single function.
}
\examples{

library(eRm)
### real data ##########

data(pp_amt)

d <- pp_amt$daten_amt

rd_res <- PPass(respdf = d, 
                items = 8:ncol(d),
                mod="1PL",
                thres = pp_amt$betas[,2], 
                fitindices = "lz")

head(rd_res)


## ========== RM - eRm 
my_data <- eRm::sim.rasch(200, 12)
my_rm <- eRm::RM(my_data)

res_pp1 <- PPass(my_rm)


## ========== PCM - eRm

set.seed(2751)

THRES  <- matrix(c(-2,-1.23,1.11,3.48,1
                   ,2,-1,-0.2,0.5,1.3,-0.8,1.5),nrow=2)
THRES <- rbind(THRES,c(-0.2,NA,NA,NA,NA,NA))

sl          <- rep(1,6)
THRESx      <- rbind(0,THRES)
THETA       <- rnorm(200)
simdat_gpcm <- sim_gpcm(thres = THRESx,alpha = sl,theta = THETA)

my_pcm <- eRm::PCM(simdat_gpcm)

res_pp2 <- PPass(my_pcm)

## ==========  1PL model

set.seed(1337)

# intercepts
diffpar <- seq(-3,3,length=15)
# slope parameters
sl     <- round(runif(15,0.5,1.5),2)
la     <- round(runif(15,0,0.25),2)
ua     <- round(runif(15,0.8,1),2)

# response matrix
awm <- matrix(sample(0:1,100*15,replace=TRUE),ncol=15)
awm <- as.data.frame(awm)
# estimate person parameter

# estimate person parameter and person fit
out <- PPass(respdf = awm,thres = diffpar, items="all",
             mod=c("1PL"), fitindices= c("lz","lzstar","infit","outfit"))

# show first rows
head(out)
}
\seealso{
\link{PP_4pl}, \link{PP_gpcm}, \link{PPall}, \link{Pfit}
}
\author{
Manuel Reif, Jan Steinfeld
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lzstar_mixed.R
\name{Pfit_lzstarmix}
\alias{Pfit_lzstarmix}
\title{Universal PPall person-fit wrapper with full diagnostics}
\usage{
Pfit_lzstarmix(
  respm,
  ppobj,
  model2est = NULL,
  orig_thres = NULL,
  orig_slopes = NULL,
  orig_lowerA = NULL
)
}
\arguments{
\item{respm}{Response matrix.}

\item{ppobj}{PPall output.}

\item{model2est}{Original model specification.}

\item{orig_thres}{Original thres input to PPall.}

\item{orig_slopes}{Original slopes input.}

\item{orig_lowerA}{Original lowerA input.}
}
\description{
Universal PPall person-fit wrapper with full diagnostics
}
\details{
This wrapper tries multiple extraction strategies and then calls
\code{lzstar_mixed()} without changing the core calculation.
}
